#include <windows.h>
#include <windowsx.h>
#include "RegFunc.h"


//	Public API functions.

//*******************************************************************
//	PURPOSE:	To get a string from Registry.

BOOL	RegFunc_GetString(LPCWSTR pszSection, LPCWSTR pszEntry,
								char* pszString, DWORD dwSize, HKEY hKey)
{
	DWORD	dwResult, dwType;
	HKEY	h; 

	dwResult = RegOpenKeyEx(hKey, pszSection, 0, KEY_READ,&h);//KEY_ALL_ACCESS, &h);
	if (dwResult != ERROR_SUCCESS)
	{ 
		 RETAILMSG(TRUE, (TEXT("RegFunc_GetString RegOpenKeyEx fail\r\n")));
		  return FALSE;
	}
	if (pszEntry)
	{
		dwResult = RegQueryValueEx(h, pszEntry, 0, &dwType, (LPBYTE)pszString, &dwSize);
		if (dwResult == ERROR_SUCCESS && dwType == REG_SZ)
		{
			RegCloseKey(h);
			return TRUE;
		}
		 RETAILMSG(TRUE, (TEXT("RegFunc_GetString RegQueryValueEx dwType=%d fail\r\n"),dwType));
	}
	RegCloseKey(h);
	return FALSE;
}

//*******************************************************************
//	PURPOSE:	To set a string to Registry.

BOOL	RegFunc_SetString(LPCWSTR pszSection, LPCWSTR pszEntry,
								const char* pszString, HKEY hKey)
{
	DWORD	dwResult, dwDisp;
	HKEY	h;
	if(NULL == pszString)
		return FALSE;
	dwResult = RegCreateKeyEx(hKey,pszSection , 0, NULL, 
									   REG_OPTION_NON_VOLATILE,
										KEY_ALL_ACCESS, NULL, &h, &dwDisp);
	if (dwResult != ERROR_SUCCESS) 
		  return FALSE;
	if (pszEntry)
	{
		dwResult = RegSetValueEx(h, pszEntry, 0, REG_SZ, (LPBYTE)pszString, strlen(pszString)+1);
		if (dwResult == ERROR_SUCCESS)
		{
			RegFlushKey(h);
			RegCloseKey(h);
			return TRUE;
		}
	}
	RegCloseKey(h);
	return FALSE;
}

//*******************************************************************
//	PURPOSE:	To get a dword from Registry.

BOOL	RegFunc_GetDWord(LPCWSTR pszSection, LPCWSTR pszEntry,
							  LPDWORD lpdwValue, HKEY hKey)
{
	//char	szBuffer[_MAX_PATH];
	DWORD	dwResult, dwSize, dwType;
	HKEY	h;
	DWORD   dwValueTemp;

	dwResult = RegOpenKeyEx(hKey, pszSection, 0, KEY_ALL_ACCESS, &h);

	if (dwResult != ERROR_SUCCESS) 
	{
    RETAILMSG(TRUE, (TEXT("RegOpenKeyEx fail\r\n")));
		return FALSE;
	}
	if (pszEntry)
	{
		dwSize	= sizeof(DWORD);
		dwResult	= RegQueryValueEx(h, pszEntry, 0, &dwType, (LPBYTE)&dwValueTemp, &dwSize);
		if (dwResult == ERROR_SUCCESS)// && dwType == REG_DWORD)
		{
			RegCloseKey(h);
			*lpdwValue=dwValueTemp;
			return TRUE;
		}
		RETAILMSG(TRUE, (TEXT("RegQueryValueEx fail\r\n")));
	
	}
	RegCloseKey(h);
	return FALSE;
}

//*******************************************************************
//	PURPOSE:	To set a dword to Registry.

BOOL	RegFunc_SetDWord(LPCWSTR pszSection, LPCWSTR pszEntry,
							  DWORD dwValue, HKEY hKey)
{
	DWORD	dwResult, dwDisp;
	HKEY	h;

	dwResult = RegCreateKeyEx(hKey, pszSection , 0, NULL, REG_OPTION_NON_VOLATILE,
										KEY_ALL_ACCESS, NULL, &h, &dwDisp);
	if (dwResult != ERROR_SUCCESS) return FALSE;
	if (pszEntry)
	{
		dwResult = RegSetValueEx(h,pszEntry, 0, REG_DWORD, (LPBYTE)&dwValue, sizeof(DWORD));
		if (dwResult == ERROR_SUCCESS)
		{
			RegCloseKey(h);
			return TRUE;
		}
	}
	RegCloseKey(h);
	return FALSE;
}

//*******************************************************************
//	PURPOSE:	To delete a key in the registry.

BOOL	RegFunc_DeleteKey(HKEY hkey, LPCWSTR pszSection)
{
	if (RegDeleteKey(hkey, pszSection) != ERROR_SUCCESS) 
		 return FALSE;
  else
	   return TRUE;
}


//////////////////////////////////////////////////////////////////////////////
// Function name	: RegFunc_GetBinaryData
// Description	    : 
// Return type		: BOOL 
// Argument         : const char* pszSection
// Argument         : const char* pszEntry
// Argument         : LPVOID *pData
// Argument         : DWORD dwSize
BOOL RegFunc_GetBinaryData(LPCWSTR pszSection,LPCWSTR pszEntry, 
												LPVOID pData, DWORD dwSize, HKEY hKey)
{

	DWORD	dwResult, dwType, dwActualSize;
	HKEY	h; 
	if(NULL == pData)
	{
		return FALSE;
	}
	
	dwResult = RegOpenKeyEx(hKey, pszSection, 0, KEY_ALL_ACCESS, 
															&h);
	if(dwResult != ERROR_SUCCESS) 
		return FALSE;
	if (NULL != pszEntry)
	{
		dwActualSize = dwSize;
		dwResult = RegQueryValueEx(h, pszEntry, 0, &dwType, 
							(LPBYTE)pData, &dwActualSize);
		if ((dwResult == ERROR_SUCCESS) && (dwType == REG_BINARY) && 
											(dwSize == dwActualSize ))
		{
			RegCloseKey(h);
			return TRUE;
		}
	}

	return FALSE;
} //end RegFunc_GetBinaryData()
//////////////////////////////////////////////////////////////////////////////

// Function name	: RegFunc_SetBinaryData
// Description	    : 
// Return type		: BOOL 
// Argument         : const char* pszSection
// Argument         : const char* pszEntry
// Argument         : LPVOID *pData
// Argument         : DWORD dwSize
BOOL RegFunc_SetBinaryData(LPCWSTR pszSection, LPCWSTR pszEntry, 
												LPVOID pData, DWORD dwSize, HKEY hKey)
{
	
	DWORD	dwResult, dwDispose;
	HKEY	h = NULL;  
	if(NULL == pData)
	{
		return FALSE;
	}

  //dwResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE, pszSection, 0, KEY_ALL_ACCESS, &h);
	//if(dwResult != ERROR_SUCCESS) 
		//return FALSE;

	if (NULL != pszEntry)
	{
		RegCreateKeyEx(hKey, pszSection, 0, NULL, 
								REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS,
								NULL, &h, &dwDispose);
		dwResult = RegSetValueEx(h, pszEntry, 0, REG_BINARY, 
									(LPBYTE)pData, dwSize);
		if (dwResult == ERROR_SUCCESS)
		{
			RegCloseKey(h);
			return TRUE;
		}
	}

	return FALSE;
} //end RegFunc_SetBinaryData()
