/* ---------------------------------------------------------------------------
          Copyright (c) 2003-2006 Micrel, Inc.  All rights reserved.
   ---------------------------------------------------------------------------

    NdisISR.c - NDIS driver interrupt processing.

    Author      Date        Description
    THa         12/01/03    Created file.
    THa         06/27/05    Updated for version 0.1.5.
    THa         02/23/06    Updated for WinCE 5.0.
    THa         06/02/06    Report statistics from MIB counters.
    THa         10/04/06    Report link state in interrupt routine.
   ---------------------------------------------------------------------------
*/


#include "target.h"
#include "NdisDevice.h"

// Handle VLAN receiving side 
//--------------------------------------------------------------------------------------------------
//|Configured VLAN ID |Packet Contains |VLAN ID in the Tag Header|  Miniport Driver Action          | 
//|for the NIC        |a Tag Header?   |                         |                                  |
//|___________________|________________|_________________________|__________________________________|
//|Zero     (case 1)  |Yes             |Any value                |Remove priority and VLAN ID from the packet.
//|                   |                |                         |Insert removed priority and VLAN ID 
//|                   |                |                         |into per-packet information. Indicate the packet.
//|___________________|________________|_________________________|__________________________________ 
//|Zero     (case 2)  |No              |Not applicable           |Set the priority and VLAN ID 
//|                   |                |                         |in per-packet information to zero. 
//|                   |                |                         |Indicate the packet.
//|___________________|________________|_________________________|___________________________________
//|Nonzero  (case 3)  |Yes             |Matches the configured   |Remove priority and VLAN ID 
//|                   |                |VLAN ID                  |from the packet. Insert removed priority 
//|                   |                |                         |and VLAN ID into per-packet information.
//|                   |                |                         |Indicate the packet. 
//|___________________|________________|_________________________|____________________________________
//|Nonzero  (case 4)  |Yes             |Does not match the       |
//|                   |                |configured VLAN ID       |Do not indicate the packet. 
//|___________________|________________|_________________________|____________________________________
//|Nonzero  (case 5)  |No              |Not applicable           |Do not indicate the packet.
//|___________________|________________|_________________________|___________________________ 



#ifdef KS_PCI_BUS
extern void PortGetDefaultVID_PCI(PHARDWARE pHardware,UCHAR  bPort,PUSHORT   pwVID );
#else
extern void PortGetDefaultVID_ISA(PHARDWARE pHardware,UCHAR  bPort,PUSHORT   pwVID );
#endif
/* -------------------------------------------------------------------------- */

/*
    ProcessReceive

    Description:
        This function processes the buffer receiving.

    Parameters:
        PNDIS_ADAPTER pAdapter
            Pointer to adapter information structure.

    Return (BOOLEAN):
        TRUE if a packet is received; FALSE otherwise.
*/

BOOLEAN ProcessReceive (
    IN  PNDIS_ADAPTER pAdapter )
{
    PHARDWARE pHardware = &pAdapter->m_Hardware;
    BOOLEAN   bReceiveDone = FALSE;
    UINT      uiIndicateLength;
    UINT      uiLength;
    USHORT    wData;
    int       port = MAIN_PORT;
    BOOLEAN   bUseEmergencyPacket=FALSE;      
#ifdef DEBUG_INTERRUPT
    DbgPrint("rev-process\n"); 
#endif
    // DAVIDCAI first find a available packet
    if(pHardware->m_nReadyBuf[pHardware->m_nCurBufIndex]==PKT_NOT_READY) // not available
    {
   		UINT n;
   		for(n=0;n<MAX_NDIS_PKT;n++)
   		{
   			 if(pHardware->m_nReadyBuf[n]==PKT_READY)	
   			 	{
   			 	  pHardware->m_bLookahead=pHardware->m_pBuf[n]; 
   			 	  pHardware->m_nCurBufIndex=n;
   			 	  RETAILMSG(TRUE, (TEXT("current pkt=%d\r\n"),pHardware->m_nCurBufIndex));
   			 	  break;	
   			 	}
   		}
   		if(n>=MAX_NDIS_PKT) //all packets is not available
		   {
			    RETAILMSG(TRUE, (TEXT("no packets, using emergency pkt\r\n")));
			    bUseEmergencyPacket=TRUE; 
		   }
   	}   
	
    
    // At this point, receive interrupts are disabled.
    do
    {
        
        HardwareReceive( pHardware );
        if ( !pHardware->m_nPacketLen )
            goto next_packet;
        
        //RETAILMSG(TRUE, (TEXT("(r=%d) "),pHardware->m_nPacketLen));
        //wincePrintHexData(pHardware->m_bLookahead,pHardware->m_nPacketLen,0);//0: PRINTF_BYTE
        
        uiLength = pHardware->m_nPacketLen - ETHERNET_HEADER_SIZE;
        uiIndicateLength = uiLength;
        
        // Only allow received packets after Filter is defined.
        if ( pAdapter->m_bPacketFilterSet )
        {
#ifdef VLAN_ENABLE
         //NDIS_STATUS   Status;
           NDIS_PACKET_8021Q_INFO VlanPriInfo;
#endif
           NDIS_PACKET *PacketArray[MAX_PACKETS_PER_INDICATE];
#if defined( DEBUG_RX_DATA )  &&  !defined( UNDER_CE )
            UINT   wLength;
#endif
            
           
		
#ifdef VLAN_ENABLE
			VlanPriInfo.Value =          // Get current value.
			NDIS_PER_PACKET_INFO_FROM_PACKET(CURRENT_PACKET, Ieee8021QInfo); 
		
		//RETAILMSG(TRUE, (TEXT("**recv bufsize=%d packetsize=%d vpacket=0x%x\r\n"),
      //     uiIndicateLength, pHardware->m_nPacketLen,VlanPriInfo.Value ));
   
			VlanPriInfo.TagHeader.UserPriority = pHardware->m_PerPacketUserPriority; // Set priority.
			VlanPriInfo.TagHeader.VlanId = pHardware->m_PerPacketVlanID; // Set VLAN identifier.
			VlanPriInfo.TagHeader.CanonicalFormatId = 0; // Should be zero.
			VlanPriInfo.TagHeader.Reserved = 0; // Should be zero.
			NDIS_PER_PACKET_INFO_FROM_PACKET(CURRENT_PACKET, Ieee8021QInfo) = 
								VlanPriInfo.Value; 
#endif
          
    if(bUseEmergencyPacket==FALSE)
	{
		   PacketArray[0] = CURRENT_PACKET;
           NdisAdjustBufferLength(CURRENT_BUFFER,pHardware->m_nPacketLen);

           NDIS_SET_PACKET_STATUS(CURRENT_PACKET, NDIS_STATUS_SUCCESS);
           pHardware->m_nReadyBuf[pHardware->m_nCurBufIndex]=PKT_NOT_READY;
           NdisMIndicateReceivePacket(pAdapter->m_hAdapter, PacketArray, 1);
          {
           NDIS_STATUS ReturnStatus;
           ReturnStatus = NDIS_GET_PACKET_STATUS(PacketArray[0]);
           //DbgPrint("status=%d \n",ReturnStatus); 
           if(ReturnStatus!= NDIS_STATUS_PENDING)
           	 pHardware->m_nReadyBuf[pHardware->m_nCurBufIndex]=PKT_READY;//CURRENT_PACKET->Reserved[0]=PKT_READY; //unmark the packet
          
          } 
          
	}
	else 
	{
      //all pass to up-level packets have been used. we need to use this 
	  //Emergency Packet and ask NDIS do a immediately copy
		   PacketArray[0] = pHardware->m_pEmergencyPacket;
           NdisAdjustBufferLength(pHardware->m_pEmergencyNdisBuf,pHardware->m_nPacketLen);
           //indicate NDIS to do immediately copy
           NDIS_SET_PACKET_STATUS(pHardware->m_pEmergencyPacket, NDIS_STATUS_RESOURCES);
           NdisMIndicateReceivePacket(pAdapter->m_hAdapter, PacketArray, 1);
           
	}

 
 
#if defined( DEBUG_RX_DATA )  &&  !defined( UNDER_CE )
            for ( wLength = 0; wLength <ETHERNET_HEADER_SIZE; wLength++ ) {
  
                DbgPrint( "%02X ", pBuf[ wLength ]);
                if ( ( wLength % 16 ) == 15 ) {
                    DbgPrint( "\n" );
                }
            }
            DbgPrint( "\n" );

            DbgPrint( "--------------data----------------\n" );

			for ( wLength = 0; wLength < uiIndicateLength; wLength++ ) {
           
                DbgPrint( "%02X ", pBuffer[ wLength ]);
                if ( ( wLength % 16 ) == 15 ) {
                    DbgPrint( "\n" );
                }
            }
            DbgPrint( "\n" );
#endif
            bReceiveDone |= TRUE;

#ifndef NO_STATS
            if ( pHardware->m_bLookahead[ 0 ] == 0xFF )
            {
                pHardware->m_cnCounter[ port ]
                    [ OID_COUNTER_BROADCAST_FRAMES_RCV ]++;
                pHardware->m_cnCounter[ port ]
                    [ OID_COUNTER_BROADCAST_BYTES_RCV ] +=
                        pHardware->m_nPacketLen;
            }
            else
            if ( pHardware->m_bLookahead[ 0 ] == 0x01 )
            {
                pHardware->m_cnCounter[ port ]
                    [ OID_COUNTER_MULTICAST_FRAMES_RCV ]++;
            }
            else
            {
                pHardware->m_cnCounter[ port ]
                    [ OID_COUNTER_UNICAST_FRAMES_RCV ]++;

                pHardware->m_cnCounter[ port ]
                    [ OID_COUNTER_DIRECTED_FRAMES_RCV ]++;
                pHardware->m_cnCounter[ port ]
                    [ OID_COUNTER_DIRECTED_BYTES_RCV ] +=
                        pHardware->m_nPacketLen;
            }
#endif
            pHardware->m_cnCounter[ port ][ OID_COUNTER_RCV_OK ]++;
        }

next_packet:

        // Acknowledge the interrupt
        HardwareAcknowledgeReceive( pHardware );

        HardwareReadRegWord( pHardware, REG_RX_STATUS_BANK,
            REG_RX_STATUS_OFFSET, &wData );
    } while ( wData );

    return( bReceiveDone );
}  // ProcessReceive


/*
    MiniportHandleInterrupt

    Description:
        This routine is used by NDIS for deferred interrupt processing.  It
        reads from the Interrupt Status Register any outstanding interrupts and
        handles them.

    Parameters:
        NDIS_HANDLE hAdapaterContext
            Handle to adapter context containing adapter information.

    Return (None):
*/

VOID MiniportHandleInterrupt (
    IN  NDIS_HANDLE hAdapterContext )
{
    PNDIS_ADAPTER pAdapter = ( PNDIS_ADAPTER ) hAdapterContext;
    PHARDWARE     pHardware = &pAdapter->m_Hardware;
    UINT          IntEnable;
    BOOLEAN       bReceiveDone = FALSE;
    int           port = MAIN_PORT;

#ifdef DEBUG_DRIVER_INTERRUPT
DbgPrint( "MiniportHandleInterrupt"NEWLINE );
#endif

#ifdef DEBUG_COUNTER
    pHardware->m_nGood[ COUNT_GOOD_INT ]++;
#endif
    HardwareReadInterrupt( pHardware, &IntEnable );
    while ( IntEnable )
    {

#ifdef DEBUG_COUNTER
        pHardware->m_nGood[ COUNT_GOOD_INT_LOOP ]++;
#endif
        HardwareAcknowledgeInterrupt( pHardware, IntEnable );

#ifdef DEBUG_INTERRUPT
DbgPrint( " intr:%04x"NEWLINE, IntEnable );
#endif

        if ( ( IntEnable & INT_TX ) )
        {
            HardwareTransmitDone( pHardware );

            // Acknowledge the interrupt
            HardwareAcknowledgeTransmit( pHardware );

#ifdef DEBUG_COUNTER
            pHardware->m_nGood[ COUNT_GOOD_INT_TX ]++;
#endif

#ifdef SEND_QUEUE
            SendNextPacket( pAdapter );
#endif
        }

        if ( ( IntEnable & INT_RX ) )
        {
            // Receive the packet
           //DAVIDCAI added lock to protect shared buffer pHardware->m_bLookahead 03-29-2007
            NdisAcquireSpinLock( &pAdapter->m_lockAdapter );  
            
            bReceiveDone |= ProcessReceive( pAdapter );
            
            NdisReleaseSpinLock( &pAdapter->m_lockAdapter );
            
#ifdef DEBUG_COUNTER
            pHardware->m_nGood[ COUNT_GOOD_INT_RX ]++;
#endif

#ifdef EARLY_RECEIVE
            if ( ( IntEnable & INT_RX_EARLY ) )
            {
                // Acknowledge the interrupt
                HardwareAcknowledgeEarly( pHardware );
            }
#endif
        }

        if ( ( IntEnable & INT_PHY ) )
        {
            pHardware->m_bLinkIntWorking = TRUE;
            SwitchGetLinkStatus( pHardware );
            if ( pAdapter->m_ulNdisMediaState !=
                    pHardware->m_ulHardwareState )
            {
                NDIS_STATUS nsStatus;

                nsStatus = ( NdisMediaStateConnected ==
                    pHardware->m_ulHardwareState ) ?
                    NDIS_STATUS_MEDIA_CONNECT : NDIS_STATUS_MEDIA_DISCONNECT;

                NdisMIndicateStatus( pAdapter->m_hAdapter, nsStatus, NULL, 0 );
                NdisMIndicateStatusComplete( pAdapter->m_hAdapter );
                pAdapter->m_ulNdisMediaState = pHardware->m_ulHardwareState;
            }
            if ( ( pAdapter->m_ulDriverState & DRIVER_STATE_RESET )  &&
                    NdisMediaStateConnected == pAdapter->m_ulNdisMediaState )
            {
                pAdapter->m_ulDriverState &= ~DRIVER_STATE_RESET;
                NdisMResetComplete( pAdapter->m_hAdapter, NDIS_STATUS_SUCCESS,
                    FALSE );
            }
        }

#ifdef EARLY_RECEIVE
        if ( ( IntEnable & INT_RX_EARLY ) )
        {
            HardwareReceiveEarly( pHardware );

            // Acknowledge the interrupt
            HardwareAcknowledgeEarly( pHardware );

#ifdef DEBUG_COUNTER
            pHardware->m_nGood[ COUNT_GOOD_INT_RX_EARLY ]++;
#endif
        }
#endif

        if ( ( IntEnable & INT_RX_OVERRUN ) )
        {

#ifdef EARLY_RECEIVE
            if ( pHardware->m_bReceiveDiscard )
                pHardware->m_bReceiveDiscard = FALSE;
            else
#endif
            pHardware->m_cnCounter[ port ][ OID_COUNTER_RCV_NO_BUFFER ]++;

            // Acknowledge the interrupt
            HardwareAcknowledgeOverrun( pHardware );

#ifdef DEBUG_COUNTER
            pHardware->m_nGood[ COUNT_GOOD_INT_RX_OVERRUN ]++;
#endif
        }

#ifdef DBG
        if ( ( IntEnable & INT_RX_STOPPED ) )
        {
            USHORT wData;

            DBG_PRINT( "Rx stopped"NEWLINE );
            HardwareReadRegWord( pHardware, REG_RX_CTRL_BANK,
                REG_RX_CTRL_OFFSET, &wData );
            if ( ( wData & RX_CTRL_ENABLE ) )
            {
                DBG_PRINT( "Rx disabled"NEWLINE );
            }
            if ( ( pHardware->m_wReceiveConfig & RX_CTRL_ENABLE ) )
            {
                HW_WRITE_WORD( pHardware, REG_RX_CTRL_OFFSET, 0 );
                HW_WRITE_WORD( pHardware, REG_RX_CTRL_OFFSET,
                    pHardware->m_wReceiveConfig );
            }
            else
                pHardware->m_wInterruptMask &= ~INT_RX_STOPPED;
            HardwareAcknowledgeInterrupt( pHardware, INT_RX_STOPPED );
            break;
        }

        if ( ( IntEnable & INT_TX_STOPPED ) )
        {
            USHORT wData;

            DBG_PRINT( "Tx stopped"NEWLINE );
            HardwareReadRegWord( pHardware, REG_TX_CTRL_BANK,
                REG_TX_CTRL_OFFSET, &wData );
            if ( ( wData & TX_CTRL_ENABLE ) )
            {
                DBG_PRINT( "Tx disabled"NEWLINE );
            }
            if ( ( pHardware->m_wTransmitConfig & TX_CTRL_ENABLE ) )
            {
                HW_WRITE_WORD( pHardware, REG_TX_CTRL_OFFSET, 0 );
                HW_WRITE_WORD( pHardware, REG_TX_CTRL_OFFSET,
                    pHardware->m_wTransmitConfig );
            }
            else
                pHardware->m_wInterruptMask &= ~INT_TX_STOPPED;
            HardwareAcknowledgeInterrupt( pHardware, INT_TX_STOPPED );
            break;
        }
#endif
        HardwareReadInterrupt( pHardware, &IntEnable );
    }

    // Finally, indicate ReceiveComplete to all protocols which received
    // packets.
    if ( bReceiveDone )
    {
        // only allow received packets after Filter is defined.
        if ( pAdapter->m_bPacketFilterSet )
        {
#ifdef DEBUG_COUNTER
            pHardware->m_nGood[ COUNT_GOOD_RCV_COMPLETE ]++;
#endif
            NdisMEthIndicateReceiveComplete( pAdapter->m_hAdapter );
        }
    }
}  // MiniportHandleInterrupt


/*
    MiniportISR

    Description:
        This routine is used by NDIS for the hardware to check if the interrupt
        is its own in the shared interrupt environment.

    Parameters:
        PBOOLEAN pbInterruptRecognized
            Buffer to indicate the interrupt is recognized

        PBOOLEAN pbQueueDpc
            Buffer to indicate the deferred interrupt processing should be
            scheduled.

        PVOID pContext
            Pointer to adapter information structure.

    Return (None):
*/

VOID MiniportISR (
    OUT PBOOLEAN pbInterruptRecognized,
    OUT PBOOLEAN pbQueueDpc,
    IN  PVOID    pContext )
{
    PNDIS_ADAPTER pAdapter = ( PNDIS_ADAPTER ) pContext;
    PHARDWARE     pHardware = &pAdapter->m_Hardware;
    UINT    wIntStatus;

    HardwareSaveBank( pHardware );
    HardwareReadInterrupt( pHardware, &wIntStatus );
  #ifdef DEBUG_INTERRUPT   
    DbgPrint("[MiniportISR]\n");
  #endif
    if ( wIntStatus )
    {
        HardwareDisableInterrupt( pHardware );
        *pbInterruptRecognized = TRUE;
        *pbQueueDpc = TRUE;
    }
    else
    {
        // not our interrupt!
        *pbInterruptRecognized = FALSE;
        *pbQueueDpc = FALSE;
    }
  
    HardwareRestoreBank( pHardware );
}  // MiniportISR

/* -------------------------------------------------------------------------- */

/*
    MiniportTransferData

    Description:
        A protocol driver calls the TransferData request (indirectly via
        NdisTransferData) from within its Receive event handler to instruct the
        driver to copy the contents of the received packet to a specified
        packet buffer.

    Parameters:
        PNDIS_PACKET pPacket
            Pointer to NDIS packet.

        PUINT puiBytesTransferred
            Number of bytes written into the packet buffer.  This value is not
            valid if the return status is STATUS_PENDING.

        NDIS_HANDLE hAdapterContext
            Handle to adapter context containing adapter information.

        NDIS_HANDLE hReceiveContext
            Handle to receive context passed by the driver on its call to
            NdisMEthIndicateReceive.  The driver can use this value to
            determine which packet, on which adapter, is being received.

        UINT uiBytesOffset
            A number specifying the offset within the received packet at which
            the copy is to begin.  If the entire packet is to be copied,
            uiByteOffset must be zero.

        UINT uiBytesToTransfer
            A number specifying the number of bytes to copy.  It is legal to
            transfer zero bytes; this has no effect.  If the sum of
            uiByteOffset and uiBytesToTransfer is greater than the size of the
            received packet, then the remainder of the packet (starting from
            uiByteOffset) is transferred, and the trailing portion of the
            receive buffer is not modified.

    Return (NDIS_STATUS):
        NDIS_STATUS_SUCCESS if successful; otherwise an error code indicating
        failure.
*/

NDIS_STATUS MiniportTransferData (
    OUT PNDIS_PACKET pPacket,
    OUT PUINT        puiBytesTransferred,
    IN  NDIS_HANDLE  hAdapterContext,
    IN  NDIS_HANDLE  hReceiveContext,
    IN  UINT         uiByteOffset,
    IN  UINT         uiBytesToTransfer )
{
    PNDIS_ADAPTER pAdapter = ( PNDIS_ADAPTER ) hAdapterContext;
    PHARDWARE     pHardware = &pAdapter->m_Hardware;
    NDIS_STATUS   nsStatus;
    PNDIS_BUFFER  pndisBuffer;          // Current NDIS_BUFFER to copy into
    PUCHAR        pBuffer;              // Virtual address of the buffer
    UINT          uiBufferLength;       // Length of the buffer
    UINT          uiBufferOffset;       // Offset into the buffer
    UINT          uiBytesCopy;
    UINT          uiBytesLeft;
    UINT          uiLength;
    UINT          uiOffset;

//#ifdef DBG
//DbgPrint( "MiniportTransferData: %u, %u"NEWLINE, uiByteOffset,
  //  uiBytesToTransfer );
RETAILMSG(TRUE, (TEXT("MiniportTransferData: 0x%x, %d\r\n"),uiByteOffset,uiBytesToTransfer));
//#endif

    // Take care of boundary condition of zero length copy.
    if ( !uiBytesToTransfer )
    {
#ifdef DEBUG_COUNTER
        pHardware->m_nGood[ COUNT_GOOD_XFER_ZERO ]++;
#endif
        
        return NDIS_STATUS_SUCCESS;
    }
    uiLength = pHardware->m_nPacketLen - ETHERNET_HEADER_SIZE;

    // See how much data there is to transfer.
    if ( uiByteOffset + uiBytesToTransfer > uiLength )
    {
        if ( uiByteOffset >= uiLength )
        {
#ifdef DEBUG_COUNTER
            pHardware->m_nBad[ COUNT_BAD_XFER_ZERO ]++;
#endif
            goto TransferDataError;
        }
        uiBytesToTransfer = uiLength - uiByteOffset;
    }

    // Set the number of bytes left to transfer.
    uiBytesLeft = uiBytesToTransfer;

    // Address on the adapter to copy from.
    uiOffset = ETHERNET_HEADER_SIZE + uiByteOffset;

    // Get location to copy into.
    NdisQueryPacket( pPacket, NULL, NULL, &pndisBuffer, NULL );
    NdisQueryBuffer( pndisBuffer, ( PVOID* ) &pBuffer, &uiBufferLength );
    uiBufferOffset = 0;

    // Loop, filling each buffer in the packet until there
    // are no more buffers or the data has all been copied.
    while ( uiBytesLeft > 0 )
    {
        // See how much data to read into this buffer.
        uiBytesCopy = uiBufferLength - uiBufferOffset;
        if ( uiBytesCopy > uiBytesLeft )
        {
            uiBytesCopy = uiBytesLeft;
        }

        NdisMoveMemory( pBuffer + uiBufferOffset,
            &pHardware->m_bLookahead[ uiOffset ], uiBytesCopy );

        // Update offsets and counts
        uiOffset += uiBytesCopy;
        uiBytesLeft -= uiBytesCopy;

        // Is the transfer done now?
        if ( !uiBytesLeft )
            break;

        // Was the end of this packet buffer reached?
        uiBufferOffset += uiBytesCopy;
        if ( uiBufferOffset == uiBufferLength )
        {
            NdisGetNextBuffer( pndisBuffer, &pndisBuffer );
            if ( !pndisBuffer )
                break;

            NdisQueryBuffer( pndisBuffer, ( PVOID* ) &pBuffer,
                &uiBufferLength );
            uiBufferOffset = 0;
        }
    }
    nsStatus = NDIS_STATUS_SUCCESS;
    *puiBytesTransferred = uiBytesToTransfer - uiBytesLeft;
    return( nsStatus );

TransferDataError:
    *puiBytesTransferred = 0;
    return NDIS_STATUS_FAILURE;
}  // MiniportTransferData

/* -------------------------------------------------------------------------- */

/*
    MiniportReturnPacket

    Description:
        This routine attempts to return to the receive free list the packet
        passed to us by NDIS.

    Parameters:
        NDIS_HANDLE hAdapaterContext
            Handle to adapter context containing adapter information.

        PNDIS_PACKET pPacket
            Pointer to NDIS packet.

    Return (None):
*/

VOID MiniportReturnPacket (
    IN  NDIS_HANDLE  hAdapterContext,
        PNDIS_PACKET pPacket )
{
    UINT i;
    PNDIS_ADAPTER pAdapter = ( PNDIS_ADAPTER ) hAdapterContext;
    PHARDWARE     pHardware = &pAdapter->m_Hardware;
 
    RETAILMSG(TRUE, (TEXT("MiniportReturnPacket\r\n ")));
#ifdef DBG
    DBG_PRINT( "MiniportReturnPacket"NEWLINE );
#endif
    //DAVIDCAI changed 06-07-2007 NdisAcquireSpinLock( &pAdapter->m_lockAdapter );

    NDIS_SET_PACKET_STATUS( pPacket, NDIS_STATUS_SUCCESS );
    //pPacket->Reserved[0]=PKT_READY; //unmark the packet
	  RETAILMSG(TRUE, (TEXT("return pkt=%x\r\n"),pPacket));   
   for(i=0;i<MAX_NDIS_PKT;i++)
   {
     if(pHardware->m_pPacket[i]==pPacket)
     	{
     	 	pHardware->m_nReadyBuf[i]=PKT_READY;
	      RETAILMSG(TRUE, (TEXT("return pkt index=%d\r\n"),i));   
     	  
     	} 
   }
   
    //DAVIDCAI changed 06-07-2007 NdisReleaseSpinLock( &pAdapter->m_lockAdapter );
}  // MiniportReturnPacket
