/* ---------------------------------------------------------------------------
          Copyright (c) 2003-2006 Micrel, Inc.  All rights reserved.
   ---------------------------------------------------------------------------

    NdisSend.c - NDIS driver send functions.

    Author      Date        Description
    DC					03/01/2007	Add per-packet VALN support 
    THa         12/01/03    Created file.
    THa         06/27/05    Updated for version 0.1.5.
    THa         02/23/06    Updated for WinCE 5.0.
    THa         06/02/06    Report statistics from MIB counters.
   ---------------------------------------------------------------------------
*/


#include "target.h"
#include "NdisDevice.h"
#include <Ndis.h>
#include <halether.h>
//#include "utilities.h"
#include "RegFunc.h"
//-----------------------------   NDIS VLAN supports for sending --------------------------------
//|Configured VLAN ID  | Priority or VLAN ID      |VLAN ID in the      | Miniport Driver Action|
//|for the NIC         | in the Packet's OOB Data |Packet's Tag Header |                       |
//|____________________|__________________________|____________________|_______________________| 
//| Zero    (case 1)   |Priority and VLAN ID      | Any value or       |Send the packet        |
//|                    |are zero                  | not present        |with no changes.       |
//|____________________|__________________________|____________________|_______________________|
//| Zero    (case 2)   |Priority or VLAN ID       | Not present        |Insert tag header into packet                        
//|                    |is nonzero.               |                    |by using VID in the OOB|                       
//|____________________|__________________________|____________________|_______________________|
//|Nonzero  (case 3)   |VLAN ID is zero or        |Not present         |Insert tag header into | 
//|                    |matches the configured    |                    |packet.by using configured
//|                    |VLAN ID for the NIC.      |                    |VID for the NIC        |  
//|____________________|__________________________|____________________|_______________________|
//|Nonzero  (case 4)   |VLAN ID matches the       | Zero               |Set VLAN ID in tag header
//|                    |configured VLAN ID        |                    |to the value of configured 
//|                    |for the NIC.              |                    |VLAN ID for the NIC.
//|____________________|__________________________|____________________|_______________________|
//|Nonzero  (case 5)   |Not present.              |Matches configured  |Send the packet with   | 
//|                    |                          |VLAN ID for the NIC.|no changes.            |
//|____________________|__________________________|____________________|_______________________| 
//|Nonzero  (case 6)   |VLAN ID does not match    |Any value or not    |Fail the send.         |
//|                    |configured VLAN ID for NIC|present             |                       |
//|____________________|__________________________|____________________|_______________________|
//|Nonzero  (case 7)   |Not present               |Does not match the  |                       |
//|                    |                          |configured VLAN ID  | Fail the send.        |         
//|                    |                          |for the NIC.        |                       |
//|____________________|__________________________|____________________|_______________________|
//|Any value (case 8)  |Nonzero                   |Nonzero             |Fail the send because    
//|                    |                          |                    |VLAN ID should not be set 
//|____________________|__________________________|____________________|in both OOB data and tag header.   
//|            DAVID CAI thinks there is case 9,  as following         |                        |
//|____________________________________________________________________|________________________|
//|Nonzero (case 9)    |Not present               |Not present         |Insert tag header into  |
//|                    |                          |                    |packet.by using configured
//|                    |                          |                    |VID for the NIC         |
//|____________________|__________________________|____________________|________________________|    
// miniport sending action
typedef enum{
DIRECT_SEND=0,
INSERT_TAG,		
CHANGE_VID,
DISCARD_PACKET	
}ASEND_ACTION;

#if 0
/* -------------------------------------------------------------------------- */

USHORT ntohs (
    USHORT wNetValue )
{
    USHORT wHostValue;

    wHostValue = (( UCHAR ) wNetValue << 8 ) | ( UCHAR )( wNetValue >> 8 );
    return( wHostValue );
}  // ntohs
#endif

/*
    GetFirstBuffer

    Description:
        This function retrieves the first NDIS buffer in the NDIS packet.

    Parameters:
        PNDIS_BUFFER* ppndisBuffer
            Pointer to NDIS buffer pointer.

        PUINT puiLength
            Buffer to store the length of the data buffer.

    Return (PUCHAR):
        Pointer to data buffer in the NDIS buffer.
*/

__inline
PUCHAR GetFirstBuffer (
    PNDIS_BUFFER* ppndisBuffer,
    PUINT         puiLength )
{
    PUCHAR pBuffer;

    // Each packet may be made up of several buffers, linked together.  Get the
    // virtual address (in pBuffer) and the length (in uiLength) of the first
    // buffer in the packet by calling NdisQueryBuffer.
    NdisQueryBuffer( *ppndisBuffer, ( PVOID* ) &pBuffer, puiLength );

    // If the pointer to the buffer is not NULL and the length returned from
    // the above call is zero, skip to the next buffer and query it for its
    // length.
    while ( !*puiLength )
    {
        NdisGetNextBuffer( *ppndisBuffer, ppndisBuffer );
        if ( !*ppndisBuffer )
        {
            return NULL;
        }
        NdisQueryBuffer( *ppndisBuffer, ( PVOID* ) &pBuffer, puiLength );
    }
    return( pBuffer );
}  // GetFirstBuffer


/*
    GetNextBuffer

    Description:
        This function retrieves the next NDIS buffer in the NDIS packet.

    Parameters:
        PNDIS_BUFFER* ppndisBuffer
            Pointer to NDIS buffer pointer.

        PUINT puiLength
            Buffer to store the length of the data buffer.

    Return (PUCHAR):
        Pointer to data buffer in the NDIS buffer.
*/

__inline
PUCHAR GetNextBuffer (
    PNDIS_BUFFER* ppndisBuffer,
    PUINT         puiLength )
{
    PUCHAR pBuffer;

    *puiLength = 0;

    // Go through the process of eliminating buffers with no data in them.
    // Exit the following while loop when we run out of buffers or have one
    // that contains some data.
    while ( !*puiLength )
    {
        NdisGetNextBuffer( *ppndisBuffer, ppndisBuffer );
        if ( !*ppndisBuffer )
        {
            return NULL;
        }
        NdisQueryBuffer( *ppndisBuffer, ( PVOID* ) &pBuffer, puiLength );
    }
    return( pBuffer );
}  // GetNextBuffer


/*
    AdapterCopyDownPacket

    Description:
        This function copies the packet down to the adapter buffer.

    Parameters:
        PNDIS_ADAPTER pAdapter
            Pointer to adapter information structure.

        PNDIS_BUFFER pndisBuffer
            Pointer to NDIS buffer.

        UINT uiPacketLength
            The length of the buffer.

        PUINT puiLength
            Buffer to store the actual length of data copied.

        BOOLEAN* pbBroadcast
            Buffer to store the broadcast flag.

    Return (BOOLEAN):
        TRUE if successful; otherwise, FALSE.
*/
BOOLEAN AdapterCopyDownPacket (
    IN  PNDIS_ADAPTER pAdapter,
    PNDIS_PACKET      pPacket,
    IN  PNDIS_BUFFER  pndisBuffer,
    IN  UINT          uiPacketLength,
    OUT PUINT         puiLength,
    OUT BOOLEAN*      pbBroadcast )
{
    PHARDWARE pHardware = &pAdapter->m_Hardware;
    // Length of current source buffer
    UINT      uiBufferLength, nSecondLen;
    static    UCHAR pFirstBuf[ MAX_BUF_SIZE ];
    // Address of current source buffer
    PUCHAR    pBuffer;
    PETHERHDR pEhterHdr;
    ASEND_ACTION sendAction=DIRECT_SEND; 
    BOOLEAN fHdr=TRUE;     
    DWORD dwCase=0;
	 
#ifdef VLAN_ENABLE
	  NDIS_PACKET_8021Q_INFO VlanPriInfo;

	  UINT32 UserPriorityInOOB=0;
	  UINT32 VIDInOOB=0;
	  UINT32 UserPriorityInsert=0;
	  UINT32 VIDInsert=0;
	  UINT32 VIDInHdr=0;
	  VlanPriInfo.Value=NULL;
#endif           

	*puiLength = 0; // initialize;
	
    pBuffer = GetFirstBuffer( &pndisBuffer, &uiBufferLength );
  // DAVIDCAI:following code is added at 04-20-2007. As microsoft found the first buffer
  // may include less than 16 byte data  
    if(pBuffer==NULL)
    	return FALSE;
    
    MOVE_MEM(pFirstBuf, pBuffer,uiBufferLength);
    //DbgPrint("firstbuf l=%d\n",uiBufferLength); 
	while(uiBufferLength <16) // first buffer must include more than 16 byte 
    {
    	 pBuffer = GetNextBuffer( &pndisBuffer, &nSecondLen);
    	 if(pBuffer==NULL)
    	 	 return FALSE;
    	MOVE_MEM(pFirstBuf+uiBufferLength,pBuffer,nSecondLen); 	 
       uiBufferLength+=nSecondLen;
    }
    //DbgPrint("firstbuf l2=%d\n",uiBufferLength); 

	pBuffer=pFirstBuf;
  // DAVIDCAI end  
    
    pEhterHdr=(PETHERHDR)pBuffer;
    //RETAILMSG(TRUE, (TEXT("ether type =0x%x \r\n"),ntohs(pEhterHdr->EtherType)));

#ifdef VLAN_ENABLE
  
  if(ntohs(pEhterHdr->EtherType)==TYPE_VLAN)
	{
		//get VID;
	  USHORT * pvid=(USHORT *)(pEhterHdr+1); 
	  VIDInHdr=(UINT32)(*pvid & 0x0FFF);// 12 bit VID 
	}


do
{
    if(pHardware->m_nNICVID!=0 && (VlanPriInfo.Value==NULL) && 
    	      ntohs(pEhterHdr->EtherType)!=TYPE_VLAN) // case 9 will be most happen, put first
  	{
  		 dwCase=9;
		 VIDInsert=pHardware->m_nNICVID;
  		 sendAction=INSERT_TAG;
  		 break;
  	}
    
    else if(pHardware->m_nNICVID==0 && 
      (VlanPriInfo.Value!=NULL) &&
     (VIDInOOB==0 && UserPriorityInOOB==0)) // case 1
    {
      dwCase=1;
      sendAction=DIRECT_SEND;
      break;
    }
    else if((pHardware->m_nNICVID==0) &&
    	      (VlanPriInfo.Value!=NULL) &&
            (VIDInOOB!=0 || UserPriorityInOOB!=0) && 
            (ntohs(pEhterHdr->EtherType)!=TYPE_VLAN))  //case 2
	{
       dwCase=2;
       UserPriorityInsert=UserPriorityInOOB;
	   VIDInsert=VIDInOOB;
       sendAction=INSERT_TAG;
       break;   
	}
    else if(pHardware->m_nNICVID!=0 && 
         (VlanPriInfo.Value!=NULL)  &&
         (VIDInOOB==0 || VIDInOOB==pHardware->m_nNICVID) &&
         (ntohs(pEhterHdr->EtherType)!=TYPE_VLAN))  //case 3
	{
        dwCase=3;
        UserPriorityInsert=UserPriorityInOOB;
	    VIDInsert=pHardware->m_nNICVID;
        sendAction=INSERT_TAG;
        break;		 	
	}
  else if(pHardware->m_nNICVID!=0 && 
        (VlanPriInfo.Value!=NULL) &&
        (VIDInOOB==pHardware->m_nNICVID) &&
        (ntohs(pEhterHdr->EtherType)==TYPE_VLAN && VIDInHdr==0)) //case 4
        {
        	dwCase=4;
			VIDInsert=VIDInOOB;
        	sendAction=CHANGE_VID; //change vid in the header to pHardware->m_nNICVID
        	break;
        } 
  else if(pHardware->m_nNICVID!=0 &&
          (VlanPriInfo.Value==NULL) && (VIDInHdr==pHardware->m_nNICVID))//case 5
        {
          dwCase=5;
          sendAction=DIRECT_SEND;
          break;
         	
        }
  else if(pHardware->m_nNICVID!=0 && 
  	     (VlanPriInfo.Value!=NULL) && 
  	     (VIDInOOB!=pHardware->m_nNICVID)) //case 6
  	{
  		   dwCase=6;
  		   sendAction=DISCARD_PACKET;
  		   break;
  	}
  else if(pHardware->m_nNICVID!=0 && (VlanPriInfo.Value==NULL) &&
  	 	(ntohs(pEhterHdr->EtherType)==TYPE_VLAN && VIDInHdr!=pHardware->m_nNICVID)) //case 7
  	 {
  		   dwCase=7;
  		   sendAction=DISCARD_PACKET;
  		   break;
  	 		
  	 }
  else if( (VIDInOOB!=0) &&
  	       //VIDInOOB != 0 means -->(VlanPriInfo.Value!=NULL) && 
  	       (VIDInHdr!=0))// case 8
  	{
  		 dwCase=8;
  		 sendAction=DISCARD_PACKET;
  		 break;
  	}
  else
    sendAction=DIRECT_SEND;
}while(0);
#else
    sendAction=DIRECT_SEND;
#endif
  
    //RETAILMSG(TRUE, (TEXT("send case=%d vid=%d\r\n"),dwCase,VIDInsert));
   
    if ( pBuffer )
    {
        //DAVIDCAI changed if ( ( pBuffer[ 0 ] & 0x01 ) )
       if ( ( pBuffer[ 0 ] & 0xFF) )
        {
            *pbBroadcast = TRUE;
        }
        else
        {
            *pbBroadcast = FALSE;
        }
        //DAVIDCAI changed 02-27-2007 HardwareSetTransmitLength( pHardware, uiPacketLength );
    }

	fHdr=TRUE;
    while ( pBuffer )
    {
        if(fHdr) // prepare header
		{
          fHdr=FALSE;

#ifdef VLAN_ENABLE			 

		  do
		  {
			USHORT * pw;
			USHORT vid;
			if( sendAction==DIRECT_SEND)
			    break;
			else if(sendAction==DISCARD_PACKET)
				   break; // let it do copy,but does not let it sent to hardware
			 else if(sendAction==INSERT_TAG)
             {
             
			   #ifdef MS_VIDEO_PHONE //
			   // check to see if this packet is IP 
			   if(ntohs(pEhterHdr->EtherType)==TYPE_IP)
			   {
			    UCHAR * pTos=pBuffer;
			    pTos+=15;// point to IP header's TOS field
			    //RETAILMSG(TRUE, (TEXT("ToS=0x%x \r\n"),*pTos));
   			    //check the TOS field 		
			    if(*pTos!=0)
			    {
			      BOOL  f;
			      DWORD dw=0;
			      f=RegFunc_GetDWord(TEXT("Comm\\Tcpip\\DSCPNdisMapping"), TEXT("Priority"),&dw, HKEY_LOCAL_MACHINE);   		
			      if(f)
			      	{
			      		UserPriorityInsert=dw&0x7; 
			      	}
			    }
			   }
				#endif
			        //copy MAC first
			   
			  

			   MOVE_MEM( &pHardware->m_bLookahead[ *puiLength ], pBuffer, 12);
               (*puiLength) +=12;
			   uiBufferLength-=12;
			   pBuffer+=12;
			   // insert VLAN type
               pHardware->m_bLookahead[ *puiLength]=0x81;
               (*puiLength)+=1;
               pHardware->m_bLookahead[ *puiLength]=0x00;
			   (*puiLength)+=1;
			   //inset VLAN Tag;
               vid=(UserPriorityInsert<<13)|VIDInsert;
			   pw= (USHORT *)&pHardware->m_bLookahead[ *puiLength];
			   *pw=htons(vid);
               (*puiLength)+=sizeof(USHORT);
			   //insert original packet's ethernet type
               MOVE_MEM( &pHardware->m_bLookahead[ *puiLength ], pBuffer, 2);
               pBuffer+=sizeof(USHORT);
               (*puiLength)+=sizeof(USHORT);
			   uiBufferLength-=sizeof(USHORT);
               break;
			 }
        else if(sendAction==CHANGE_VID)
			 {
                //copy VLAN portion first
               MOVE_MEM( &pHardware->m_bLookahead[ *puiLength ], pBuffer, 16);
               pBuffer+=16;
               (*puiLength)+=16;
			   uiBufferLength-=16;
               //now last two bytes in the m_bLookahead buffer is vlan field
               pw=(USHORT *)&pHardware->m_bLookahead[ *puiLength ]; 
			   pw--;// point to vlan field 	
			   vid=ntohs(*pw);
			   //clean old one;
			   vid&=0xF000;
			   vid|=(USHORT)VIDInsert; //replace new one
			   *pw=htons(vid); // write back;
			   break;   
			}
			}while(0);
	#endif

		}
	
		
		MOVE_MEM( &pHardware->m_bLookahead[ *puiLength ], pBuffer,
            uiBufferLength );
        *puiLength += uiBufferLength;
        pBuffer = GetNextBuffer( &pndisBuffer, &uiBufferLength );
    }

    if ( !*puiLength )
    {
        goto CopyDownPacketError;
    }
 
 #if 0   
    wincePrintHexData(pHardware->m_bLookahead,*puiLength,0);//0: PRINTF_BYTE
 #endif
    
   //RETAILMSG(TRUE, (TEXT("[S=%d]"),*puiLength));   

    
  if(sendAction!=DISCARD_PACKET)
  {
    HardwareSetTransmitLength( pHardware,*puiLength);//DAVIDCAI adfd 02-27-2007
    HW_WRITE_BUFFER( pHardware, REG_DATA_OFFSET, pHardware->m_bLookahead,
        *puiLength );

#ifdef SH_16BIT_WRITE
    HW_WRITE_WORD( pHardware, REG_TXQ_CMD_OFFSET, TXQ_CMD_ENQUEUE_PACKET );

#else
    HW_WRITE_BYTE( pHardware, REG_TXQ_CMD_OFFSET, TXQ_CMD_ENQUEUE_PACKET );
#endif
  }
#if defined( DEBUG_TX )  &&  defined( UNDER_CE )
DBG_PRINT( "%04d: %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X"NEWLINE,
    *puiLength,
    pHardware->m_bLookahead[ 0 ], pHardware->m_bLookahead[ 1 ],
    pHardware->m_bLookahead[ 2 ], pHardware->m_bLookahead[ 3 ],
    pHardware->m_bLookahead[ 4 ], pHardware->m_bLookahead[ 5 ],
    pHardware->m_bLookahead[ 6 ], pHardware->m_bLookahead[ 7 ],
    pHardware->m_bLookahead[ 8 ], pHardware->m_bLookahead[ 9 ],
    pHardware->m_bLookahead[ 10 ], pHardware->m_bLookahead[ 11 ],
    pHardware->m_bLookahead[ 12 ], pHardware->m_bLookahead[ 13 ],
    pHardware->m_bLookahead[ 14 ], pHardware->m_bLookahead[ 15 ],
    pHardware->m_bLookahead[ 16 ], pHardware->m_bLookahead[ 17 ]);
#endif

    return TRUE;

CopyDownPacketError:

#ifdef DEBUG_COUNTER
    pHardware->m_nBad[ COUNT_BAD_COPY_DOWN ]++;
#endif
    return FALSE;
}  // AdapterCopyDownPacket

/* -------------------------------------------------------------------------- */

/*
    SendPacket

    Description:
        This function sends a packet to hardware.

    Parameters:
        PNDIS_ADAPTER pAdapter
            Pointer to adapter information structure.

        PNDIS_BUFFER pBuffer
            Pointer to NDIS buffer.

        UINT uiPacketLength
            The length of the buffer.

    Return (NDIS_STATUS):
        NDIS_STATUS_SUCCESS if successful; otherwise, an error code indicating
        failure.
*/

NDIS_STATUS SendPacket (
    PNDIS_ADAPTER pAdapter,
    PNDIS_PACKET  pPacket,
    PNDIS_BUFFER  pBuffer,
    UINT          uiPacketLength )
{
    PHARDWARE   pHardware = &pAdapter->m_Hardware;
    NDIS_STATUS nsStatus = NDIS_STATUS_FAILURE;
    UINT        uiLength;
    BOOLEAN     bBroadcast;
    BOOLEAN     bResult;
    int         port = MAIN_PORT;

     //DAVIDCAI added lock to protect shared buffer pHardware->m_bLookahead 03-29-2007
    NdisAcquireSpinLock( &pAdapter->m_lockAdapter );     
	 
    bResult = AdapterCopyDownPacket( pAdapter, pPacket,pBuffer, uiPacketLength,
        &uiLength, &bBroadcast );
   
   NdisReleaseSpinLock( &pAdapter->m_lockAdapter );
      
   
    if ( !bResult )
    {
        goto SendIntrDone;
    }

#if 0
    if ( HardwareSendPacket( pHardware ) )
#endif
    {
        nsStatus = NDIS_STATUS_SUCCESS;
        if ( uiLength )
        {
            pHardware->m_cnCounter[ port ][ OID_COUNTER_XMIT_OK ]++;

#ifndef NO_STATS
            if ( bBroadcast )
            {
                pHardware->m_cnCounter[ port ]
                    [ OID_COUNTER_BROADCAST_FRAME_XMIT ]++;
                pHardware->m_cnCounter[ port ]
                    [ OID_COUNTER_BROADCAST_BYTES_XMIT ] += uiLength;
            }
            else
            {
                pHardware->m_cnCounter[ port ]
                    [ OID_COUNTER_DIRECTED_FRAMES_XMIT ]++;
                pHardware->m_cnCounter[ port ]
                    [ OID_COUNTER_DIRECTED_BYTES_XMIT ] += uiLength;
            }
#endif
        }
    }

SendIntrDone:

    return( nsStatus );
}  // SendPacket

/* -------------------------------------------------------------------------- */

/*
    MiniportSend

    Description:
        This function sends one packet in NDIS 3.0.

    Parameters:
        PNDIS_ADAPTER pAdapter
            Pointer to adapter information structure.

        PNDIS_PACKET pPacket
            Pointer to NDIS packet.

        UINT uiFlags
            Send flags.

    Return (NDIS_STATUS):
        NDIS_STATUS_SUCCESS if successful; otherwise, an error code indicating
        failure.
*/

NDIS_STATUS MiniportSend (
    IN  NDIS_HANDLE  hAdapterContext,
    IN  PNDIS_PACKET pPacket,
    IN  UINT         uiFlags )
{
    PNDIS_ADAPTER pAdapter = ( PNDIS_ADAPTER ) hAdapterContext;
    PHARDWARE     pHardware = &pAdapter->m_Hardware;
    NDIS_STATUS   nsStatus = NDIS_STATUS_FAILURE;
    PNDIS_BUFFER  pndisBuffer;
    UINT          uiPacketLength;
    UINT          InterruptMask;
    
    //DbgPrint("send\n"); 
    if ( !AcquireAdapter( pAdapter, FALSE ) )
    {
        goto SendDone;
    }

#ifdef DEBUG_COUNTER
    pHardware->m_nGood[ COUNT_GOOD_SEND_PACKET ]++;
#endif
    NdisQueryPacket( pPacket, NULL, NULL, &pndisBuffer, &uiPacketLength );

    // If the returned packet length is zero, there is nothing to transmit.
    if ( !uiPacketLength )
    {
#ifdef DEBUG_COUNTER
        pHardware->m_nGood[ COUNT_GOOD_SEND_ZERO ]++;
#endif
        nsStatus = NDIS_STATUS_SUCCESS;
        goto SendLockDone;
    }

    /* Save the current interrupt mask and block all interrupts. */
    InterruptMask = HardwareBlockInterrupt( pHardware );

    if ( !HardwareAllocPacket( pHardware, uiPacketLength ) )
    {
        nsStatus = NDIS_STATUS_RESOURCES;
        goto SendBlockDone;
    }

    nsStatus = SendPacket( pAdapter, pPacket,pndisBuffer, uiPacketLength );

SendBlockDone:
    /* Restore the interrupt mask. */
    HardwareSetInterrupt( pHardware, InterruptMask );

SendLockDone:
    ReleaseAdapter( pAdapter );

SendDone:
    NDIS_SET_PACKET_STATUS( pPacket, nsStatus );
    return( nsStatus );
}  // MiniportSend

/* -------------------------------------------------------------------------- */

/*
    AdapterTransmitQueueLength

    Description:
        This function returns the number of packets waiting in the transmit
        queue.

    Parameters:
        PNDIS_ADAPTER pAdapter
            Pointer to adapter information structure.

    Return (ULONG):
        The number of packets in the send queue.
*/

ULONG AdapterTransmitQueueLength (
    IN  PNDIS_ADAPTER pAdapter )
{
    PNDIS_PACKET pPacket;
    ULONG        ulCount = 0;

    if ( !AcquireAdapter( pAdapter, FALSE ) )
        return( 4 );

    pPacket = pAdapter->m_pPacketQueueHead;
    while ( pPacket )
    {
        ++ulCount;
        pPacket = RESERVED( pPacket )->Next;
    }
    ReleaseAdapter( pAdapter );
    return( ulCount );
}  // AdapterTransmitQueueLength


#ifdef SEND_QUEUE
/*
    AddPacketToTransmitQueue

    Description:
        This routine adds packets to the transmit queue.

    Parameters:
        PNDIS_ADAPTER pAdapter
            Pointer to adapter information structure.

        PNDIS_PACKET pPacket
            Pointer to NDIS packet.

    Return (None):
*/

void AddPacketToTransmitQueue (
    PNDIS_ADAPTER pAdapter,
    PNDIS_PACKET  pPacket )
{
    if ( pAdapter->m_pPacketQueueTail )
        RESERVED( pAdapter->m_pPacketQueueTail )->Next = pPacket;
    else
        pAdapter->m_pPacketQueueHead = pPacket;
    RESERVED( pPacket )->Next = NULL;
    pAdapter->m_pPacketQueueTail = pPacket;
    NDIS_SET_PACKET_STATUS( pPacket, NDIS_STATUS_PENDING );
}  // AddPacketToTransmitQueue


/*
    SendNextPacket

    Description:
        This routine is used to send the next packet waiting in the queue.

    Parameters:
        PNDIS_ADAPTER pAdapter
            Pointer to adapter information structure.

    Return (None):
*/

void SendNextPacket (
    PNDIS_ADAPTER pAdapter )
{
    PHARDWARE    pHardware = &pAdapter->m_Hardware;
    NDIS_STATUS  nsStatus;
    PNDIS_PACKET pPacket;
    PNDIS_BUFFER pndisBuffer;
    UINT         uiPacketLength;

    if ( !AcquireAdapter( pAdapter, FALSE ) )
    {
        return;
    }

    // Return if we don't have a packet to send.
    if ( !pAdapter->m_pPacketQueueHead )
    {
#ifdef DEBUG_COUNTER
        pHardware->m_nGood[ COUNT_GOOD_NO_NEXT_PACKET ]++;
#endif
        goto SendNextPacketDone;
    }

    pPacket = pAdapter->m_pPacketQueueHead;

    NdisQueryPacket( pPacket, NULL, NULL, &pndisBuffer, &uiPacketLength );

    // If the returned packet length is zero, there is nothing to transmit.
    if ( !uiPacketLength )
    {
#ifdef DEBUG_COUNTER
        pHardware->m_nGood[ COUNT_GOOD_SEND_ZERO ]++;
#endif
        nsStatus = NDIS_STATUS_SUCCESS;
        NDIS_SET_PACKET_STATUS( pPacket, nsStatus );
        goto SendNextDone;
    }

    if ( !HardwareAllocPacket( pHardware, uiPacketLength ) )
    {
        goto SendNextPacketDone;
    }

    nsStatus = SendPacket( pAdapter, pPacket,pndisBuffer, uiPacketLength );

SendNextDone:

    // Remove the packet from the queue.
#ifdef DEBUG_COUNTER
    pHardware->m_nGood[ COUNT_GOOD_NEXT_PACKET ]++;
#endif
    pAdapter->m_pPacketQueueHead = RESERVED( pPacket )->Next;
    if ( pPacket == pAdapter->m_pPacketQueueTail )
        pAdapter->m_pPacketQueueTail = NULL;

    NdisMSendComplete( pAdapter->m_hAdapter, pPacket, nsStatus );

SendNextPacketDone:
    ReleaseAdapter( pAdapter );
}  // SendNextPacket
#endif


/*
    MiniportSendPackets

    Description:
        This routine sends one or more packets in NDIS 4.0.

    Parameters:
        PNDIS_ADAPTER pAdapter
            Pointer to adapter information structure.

        PPNDIS_PACKET pPacketArray
            Array of pointers to NDIS packets.

        UINT uiCount
            Number of packets in the array.

    Return (None):
*/

VOID MiniportSendPackets (
    IN  NDIS_HANDLE   hAdapterContext,
    IN  PPNDIS_PACKET pPacketArray,
    IN  UINT          uiCount )
{
    PNDIS_ADAPTER pAdapter = ( PNDIS_ADAPTER ) hAdapterContext;
    NDIS_STATUS   nsStatus;
    PNDIS_PACKET  pPacket;
    PNDIS_BUFFER  pndisBuffer;
    PHARDWARE     pHardware = &pAdapter->m_Hardware;
    UINT          uiIndex;
    UINT          uiPacketLength;
    UINT          InterruptMask;

//DbgPrint( "MiniportSendPackets\n" );

    // If the count of packets in the array is zero, exit after noting this
    // fact.
    if ( !uiCount )
    {

#ifdef DEBUG_COUNTER
        pHardware->m_nBad[ COUNT_BAD_SEND_ZERO ]++;
#endif
        return;
    }

    // Acquire the hardware to avoid having 2 threads messing with the adapter
    // at the same time.  If this can't happen, return all of the packets with
    // a failure indication and exit.
    if ( !AcquireAdapter( pAdapter, FALSE ) )
    {
        for ( uiIndex = 0; uiIndex < uiCount; uiIndex++ )
        {
            pPacket = pPacketArray[ uiIndex ];
            NDIS_SET_PACKET_STATUS( pPacket, NDIS_STATUS_FAILURE );
        }
        return;
    }

    /* Save the current interrupt mask and block all interrupts. */
    InterruptMask = HardwareBlockInterrupt( pHardware );

    // The following 'for' loop handles one packet in the array each time
    // through.

    for ( uiIndex = 0; uiIndex < uiCount; uiIndex++ )
    {
        // Count packets sent.  Point pPacket to the next packet in the array.
#ifdef DEBUG_COUNTER
        pHardware->m_nGood[ COUNT_GOOD_SEND_PACKET ]++;
#endif
        pPacket = pPacketArray[ uiIndex ];

#ifdef SEND_QUEUE
        // If there are packets waiting in the queue to be transmitted add this
        // packet to the end of that queue.  We have to keep things in order.
        if ( pAdapter->m_pPacketQueueTail )
        {
#ifdef DEBUG_COUNTER
            pHardware->m_nGood[ COUNT_GOOD_SEND_QUEUE ]++;
#endif
            AddPacketToTransmitQueue( pAdapter, pPacket );
        }
        else
#endif
        {
            NdisQueryPacket( pPacket, NULL, NULL, &pndisBuffer,
                &uiPacketLength );

            // If the returned packet length is zero, there is nothing to
            // transmit.
            if ( !uiPacketLength )
            {
#ifdef DEBUG_COUNTER
                pHardware->m_nGood[ COUNT_GOOD_SEND_ZERO ]++;
#endif
                NDIS_SET_PACKET_STATUS( pPacket, NDIS_STATUS_SUCCESS );
                continue;
            }

            if ( !HardwareAllocPacket( pHardware, uiPacketLength ) )
            {

#ifdef SEND_QUEUE
                AddPacketToTransmitQueue( pAdapter, pPacket );

#else
                NDIS_SET_PACKET_STATUS( pPacket, NDIS_STATUS_FAILURE );
#endif
            }
            else
            {
                nsStatus = SendPacket( pAdapter, pPacket,pndisBuffer, uiPacketLength );

                NDIS_SET_PACKET_STATUS( pPacket, nsStatus );
            }
        }
    }

    /* Restore the interrupt mask. */
    HardwareRestoreInterrupt( pHardware, InterruptMask);

    ReleaseAdapter( pAdapter );
}  // MiniportSendPackets

