/*
 *Copyright (c) 2000-2002 Viola Systems Ltd.
 *All rights reserved.
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions 
 *are met:
 *
 *1. Redistributions of source code must retain the above copyright 
 *notice, this list of conditions and the following disclaimer.
 *
 *2. Redistributions in binary form must reproduce the above copyright 
 *notice, this list of conditions and the following disclaimer in the 
 *documentation and/or other materials provided with the distribution.
 *
 *3. The end-user documentation included with the redistribution, if 
 *any, must include the following acknowledgment:
 *	"This product includes software developed by Viola 
 *	Systems (http://www.violasystems.com/)."
 *
 *Alternately, this acknowledgment may appear in the software itself, 
 *if and wherever such third-party acknowledgments normally appear.
 *
 *4. The names "OpenTCP" and "Viola Systems" must not be used to 
 *endorse or promote products derived from this software without prior 
 *written permission. For written permission, please contact 
 *opentcp@opentcp.org.
 *
 *5. Products derived from this software may not be called "OpenTCP", 
 *nor may "OpenTCP" appear in their name, without prior written 
 *permission of the Viola Systems Ltd.
 *
 *THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED 
 *WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 *MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *IN NO EVENT SHALL VIOLA SYSTEMS LTD. OR ITS CONTRIBUTORS BE LIABLE 
 *FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 *BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 *OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 *EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *====================================================================
 *
 *OpenTCP is the unified open source TCP/IP stack available on a series 
 *of 8/16-bit microcontrollers, please see <http://www.opentcp.org>.
 *
 *For more information on how to network-enable your devices, or how to 
 *obtain commercial technical support for OpenTCP, please see 
 *<http://www.violasystems.com/>.
 */

/** \file ethernet.c
 *	\brief OpenTCP Ethernet protocol and driver for Realtek's RTL8019AS
 *	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *		\li Vladan Jovanovic (vladan.jovanovic@violasystems.com)
 *	\version 1.0
 *	\date 19.2.2002
 *	\bug
 *	\warning
 *	\todo
 *		\li How to adopt to different MCU speeds in the code?
 *		\li NE2000DMAInit ??? What is it used for?
 *  
 *	OpenTCP implementation of Ethernet protocol and driver for
 *	Realtek's RTL8019AS Ethernet controller. Function declarations,data
 *	structures etc. may be found in inet/ethernet.h.
 */
#include <inet/system.h>


/************************************************/
/* Modified by Renesas Technology America, Inc. */
/* Makers of M16C, H8, SH                       */
/* http://www.renesas.com                       */
/************************************************/

#include "LAN91c96.h"

/************ Configurations ************/

#define BASE_ADDR 0xF80000
#define POLLED_ETHERNET 1
#define INTERRUPT_ETHERNET 0

#define EEPROM_PRESENT  0     /* set to 1 if EEPROM is being used */
#define HARDWARE_PAD    1     /* set to 1 to enable hardware padding */

static void waitalittlebit(void);
static void smsc_write(unsigned int, unsigned int);
static void smsc_write_ns(unsigned int, unsigned int);
static unsigned int smsc_read(unsigned int);
static unsigned int smsc_read_ns(unsigned int);

//#define SMSC_WRITE(OFFSET,DATA)  *((word16 far *)(base+(word16)(OFFSET))) = (DATA)
//#define SMSC_READ(OFFSET)        *((volatile word16 far *)(base+(word16)(OFFSET)))

#define BANKSWITCH(B)            smsc_write(REG_BANKSEL,(B))

UINT8	NE2000NextPktPtr;			/**< Start address of next packet in the Ethernet controller */
UINT8 	NE2000CurrPktPtr;			/**< Start address of current packet in the Ethernet controller */

UINT8 	EtherSleep = 0;	/**< Used for storing state of Ethernet controller (0 = awake; 1 = sleeping) */

UINT8 eth_pkt_rx_buf[1550];
UINT8 * eth_rx_buf_ptr;
UINT8 eth_pkt_tx_buf[1550];
UINT8 * eth_tx_buf_ptr;

struct _ETH_INFO
{
	UINT32 total_rx_packets;
	UINT32 total_tx_packets;
	UINT8 full_duplex:1;
	UINT8 speed_100:1;
	UINT8 linked:1;
} eth_info = {0,0,0,0,0};

/** \brief Used for storing various information about the received Ethernet frame
 *	
 *	Fields from Ethernet packet (dest/source hardware address, 
 *	protocol, frame size, start of the Ethernet packet in Ethernet controller)
 *	are stored in this structure. These values are later used from upper
 *	layer protocols (IP, ARP). See ethernet_frame definition for more
 *	information about struct fields.
 *
 */	
struct ethernet_frame received_frame;

/** \brief Used for storing various information about the Ethernet frame that will be sent
 *	
 *	Fields from Ethernet packet (dest/source hardware address, 
 *	protocol, frame size) are stored in this structure by the
 *	upper layer protocols (IP, ARP, other). These values are 
 *	then used for initializing transmission of an Ethernet frame.
 *	See ethernet_frame definition for more
 *	information about struct fields.
 *
 */
struct ethernet_frame send_frame;


/** \brief Write data to NE2000 register
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasytems.com)
 *	\date 19.02.2002
 *	\param reg NE2000 register address
 *	\param dat new register content
 *
 *	Write data to NE2000 register.
 *
 */
void outNE2000 (UINT8 reg , UINT8 dat)
{
	/* Replaced by WritePacketPage */
}	

/** \brief Write data to the same NE2000 register as before
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\param dat new register content
 *
 *	Invoke this function to write data to the same register as 
 *	in previously invoked outNE2000(). Saves some cycles because
 *	data direction registers and address are not set up.
 *
 *	Use SEND_NETWORK_B() macro instead of invoking this function
 *	directly.
 */
void outNE2000again (UINT8 dat)
{
	/* Actually, because this function wants to write a byte and our */
	/* Ethernet controller only wants to write words, we will write
	/* every thing to a buffer, then send it later */
	*eth_tx_buf_ptr++ = dat;
}
/** \brief Write buffer data to the same NE2000 register as before
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 20.07.2003
 *	\param buf pointer to buffer from which we're taking data
 *	\param len number of bytes from buffer to write
 *
 *	Use SEND_NETWORK_BUF() macro instead of invoking this function
 *	directly.
 */
void outNE2000againbuf (UINT8* buf, UINT16 len)
{

	while(len--)
	{
		*eth_tx_buf_ptr++ = *buf++;
	}
}

/** \brief Read byte from NE2000 register
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\param reg NE2000 register address
 *	\return Register content
 *
 *	Invoke this function to read contents of a NE2000 register.
 *
 */
UINT8 inNE2000 (UINT8 reg)
{
	/* Replaced by ReadPacketPage */
}

/** \brief Continue reading byte(s) from NE2000
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\return Register content
 *
 *	Invoke this function to continue reading contents of a NE2000 register,
 *	usefull when reading a sequence of bytes from Ethernet controller.
 *
 *	Use RECEIVE_NETWORK_B() macro instead of directly invoking this
 *	function for easier driver change.
 */
UINT8 inNE2000again (void)
{
	/* Actually, because this function wants to read a byte and our */
	/* Ethernet controller only wants to read words, we will read
	/* every thing from a buffer that we filled already */
	return 	*eth_rx_buf_ptr++;
}
/** \brief Read bytes from NE2000 to buffer
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 20.07.2003
 *
 *	Invoke this function to continue reading contents of a NE2000 register,
 *	only this time to a buffer directly.
 *
 *	Use RECEIVE_NETWORK_BUF() macro instead of directly invoking this
 *	function for easier driver change.
 */
void inNE2000againbuf (UINT8* buf, UINT16 len)
{
	while(len--)
	{
		*buf++ = *eth_rx_buf_ptr++;
	}
}


/** \brief Check to see if new frame has been received
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\return
 *		\li #TRUE - if there is a new packet waiting
 *		\li #FALSE - if there is no new packet waiting
 *
 *	Invoke this function to check if there is a new Ethernet frame
 *	in the NIC's buffer waiting to be processed.
 */
UINT8 NE2000CheckRxFrame (void)
{
	int UINT16 status;

	/* Turn Receive on if it's not already enabled */
	BANKSWITCH(0);
	smsc_write(REG_RCR,STRIPCRC|RXEN);		/* strip CRC from received packets, enable receiving */

	BANKSWITCH(2);

	/* Read Interrupt Status Register to determine if a new packet has arrived */
	status = smsc_read(REG_IST);

	/* Check if a receive error has occured */
	if( status & RX_OVRN_INT)
	{
		/* This bit will stay latched untill reset */
		smsc_write(REG_ACK,RX_OVRN_INT);
	}

	if(status & RCV_INT)
	{
		/* A new packet has arrived */
		return(TRUE);
	}

	return(FALSE);
}

/** \brief Discard current received frame
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *
 *	Discard the current received frame.
 */
void NE2000DumpRxFrame (void)
{
}
 
 
/** \brief Initialize and configure RTL8019AS
 *	\ingroup core_initializer
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\param mac Pointer to NIC's hardware address
 *
 *	Invoke this function at startup to properly initialize NIC's registers,
 *	HW address and operation.
 */ 
void NE2000Init (UINT8* mac)
{
	unsigned int reg_data;
	unsigned int mac10,mac32,mac54;		/* For EEPROM */
	UINT16 temp;

	// for interrupt
	volatile char junk;

	BANKSWITCH(0);
	smsc_write(REG_RCR,SOFTRST);			/* software reset ethernet chip */
	waitalittlebit();
	smsc_write(REG_RCR,CLEAR);				/* drive reset bit low again */
	waitalittlebit();
	smsc_write(REG_TCR,CLEAR);

   /* Note: remove the following two lines if porting to the SMSC91C11x */
	BANKSWITCH(1);
	reg_data = smsc_read(REG_CR);
	smsc_write(REG_CR,(reg_data & 0xFEFF));      /* use 10BASE-T */
	reg_data = smsc_read(REG_CTR);

#if (POLLED_ETHERNET)
	smsc_write(REG_CTR,reg_data | AUTORELEASE);  /* auto-release TX packets */
#else
	/* auto-release TX packets, Link error enable and xmit error enable */
	smsc_write(REG_CTR,reg_data | (AUTORELEASE | LEENABLE | TEENABLE) );
#endif      /* (POLLED_ETHERNET) */

	/* make sure we can read what we just wrote */
	reg_data = smsc_read(REG_CTR);
	if( !(reg_data & AUTORELEASE) )
		while(1);		/* ERROR - Stop here */

#if (EEPROM_PRESENT)
	/* read MAC address and write it to eth_src_hw_addr. This assumes the
	MAC address was stored in EEPROM.*/
	mac54 = smsc_read(REG_IA54);
	mac32 = smsc_read(REG_IA32);
	mac10 = smsc_read(REG_IA10);
	mac[5] = (UINT8)mac10;			// Save lower byte
	mac[4] = (UINT8)(mac10 >> 8);	// Save upper byte
	mac[3] = (UINT8)mac32;			// Save lower byte
	mac[2] = (UINT8)(mac32 >> 8);	// Save upper byte
	mac[1] = (UINT8)mac54;			// Save lower byte
	mac[0] = (UINT8)(mac54 >> 8);	// Save upper byte
#else
	/* ADDRESS 0 corresponds to the first address byte on the cable */
	/* OpenTCP defines the localHW[6] array as [5]:[4]:[3]:[2]:[1]:[0] */
	smsc_write(REG_IAR+4, *((UINT16 *)mac));
	mac += 2;
	smsc_write(REG_IAR+2, *((UINT16 *)mac));
	mac += 2;
	smsc_write(REG_IAR, *((UINT16 *)mac));
#endif      /* (EEPROM_PRESENT) */

	BANKSWITCH(2);
	smsc_write(REG_MMUCR,0x40);					/* Write RESET MMU command */
	while ( smsc_read(REG_MMUCR) & BUSY );		/* wait for MMU to become not busy */

	reg_data = smsc_read(REG_INTERRUPT);
	smsc_write(REG_INTERRUPT,reg_data & 0x00FF);	/* disable interrupts */

	BANKSWITCH(0);
	smsc_write(REG_RCR,STRIPCRC|RXEN);		/* strip CRC from received packets, enable receiving */
//	smsc_write(REG_TCR,PADEN);				/* turn on hardware padding */
//	smsc_write(REG_TCR,TXENA);				/* turn on xmit */
	smsc_write(REG_TCR,PADEN|TXENA);		/* turn on hardware padding & enable xmit */

   BANKSWITCH(2);

#if (!POLLED_ETHERNET)
	/* setup interrupts */
	reg_data = smsc_read(REG_INTERRUPT);
	reg_data |= (RXINT_EN | TXINT_EN | RXOVRNINT_EN | EPHINT_EN);
	smsc_write(REG_INTERRUPT,reg_data);
	waitalittlebit();

	P_INTC.IER.BIT.IRQ0E = 0;		/* disable IRQ0 interrupt */
	junk = P_INTC.ISR.BIT.IRQ0F;
	P_INTC.ISR.BIT.IRQ0F = 0;		/* clear IRQ0 interrupt */
	P_INTC.ISCR.BIT.IRQ0SC = 2;		/* interrupt on rising edge */
	P_INTC.ITSR.BIT.ITS0 = 1;		/* select P80 as input pin */
	P_INTC.IPRA.BIT.UU = 4;			/* set priority to 4 */
	P_INTC.IER.BIT.IRQ0E = 1;		/* enable IRQ0 interrupt */
#endif

	/* Set RXEN bit to enable receiving */

	/* If porting to the SMC91C11x add code to detect and setup PHY here */

}


/** \brief Check if receive-buffer overflow occured
 *	\ingroup periodic_functions
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *
 *	This function checks if receive-buffer overflow has happened. If it did,
 *	NIC is reinitialized to ensure proper operation.
 *
 *	Invoke this function periodically to ensure proper operation under
 *	heavy load.
 */
void NE2000CheckOverFlow (void)
{
 	/* Checks if Receive Buffer overflow has happened */
 	/* and re-initializes the NIC if needed			  */

}

/** \brief Checks if new Ethernet frame exists and initializes variables
 *		accordingly
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasytems.com)
 *	\date 19.02.2002
 *	\return
 *		\li #TRUE - new frame exists, variables initialized so further
 *			processing may start
 *		\li #FALSE - no new Ethernet frame detected
 *
 *	Invoke this function periodically (better not directly but by invoking
 *	NETWORK_CHECK_IF_RECEIVED() macro) to test for newly-arrived Ethernet
 *	packets.
 *
 *	Once this function detects a new Etherent packet, internal variables 
 *	are initialized (NE2000CurrPktPtr, NE2000NextPktPtr and received_frame
 *	structure) and TRUE is returned, indicating that processing of
 *	newly-arrived frame may begin.
 */
UINT8 NE2000ReceiveFrame (void)
{

	UINT16 frame_status;
	UINT16 packet_size;
	UINT16 temp;

 	if( NE2000CheckRxFrame() == FALSE ) 
 		return(FALSE);

//	BANKSWITCH(0);
//	smsc_write(REG_RCR,0);		/* Disable Receiveing packets. This helps in debug because if you hit
//							   a breakpoint, all your memory in the Ethernet chip will fill up and you
//							   won't be able to allocate memory to transmit. */

	/* A new packet has arrived so read it out */
	BANKSWITCH(2);
	smsc_write(REG_PTR,RCV|AUTO_INCR|READ);		/* Set up the POINTER REGISTER 
									   Set RCV (to signify the RX FIFO)
									   Set AUTO_INCR (to signify incrementing the fifo pointer)
									   Set READ (to signify reading out data)
									   Pointer index at 0 (start at begining) */

	/* First word in FIFO is status */
	frame_status = smsc_read(REG_DATA1);
	/* Make sure there were no errors in the packet */
	if ((frame_status & (BADCRC|ALGNERR|TOOSHORT|TOOLONG)) )
		return FALSE;;

	/* Next word in FIFO is count */
	packet_size = 0x0FFF & smsc_read(REG_DATA1);
	/* subtract 6 bytes for status, byte count, and control word. */
	packet_size -= 6;
	if( frame_status & ODDFRM )
		packet_size += 1;		/* adjust byte count for odd count */
	
	received_frame.frame_size = packet_size;		

/* NOT USED
 	NE2000CurrPktPtr = 0;
 	NE2000NextPktPtr = 0;
*/
 	/* Record destination Ethernet Address	*/
	*(UINT16 *)(received_frame.destination + 4) = smsc_read(REG_DATA1);
	*(UINT16 *)(received_frame.destination + 2) = smsc_read(REG_DATA1);
	*(UINT16 *)(received_frame.destination + 0) = smsc_read(REG_DATA1);

 	/* Record senders Ethernet address 		*/
	*(UINT16 *)(received_frame.source + 4) = smsc_read(REG_DATA1);
	*(UINT16 *)(received_frame.source + 2) = smsc_read(REG_DATA1);
	*(UINT16 *)(received_frame.source + 0) = smsc_read(REG_DATA1);

 	/* Record Protocol	*/
	/* Because the H8S is Big Endian, we can read the value directly into memory w/o byte swaping */
 	received_frame.protocol = smsc_read_ns(REG_DATA1);

	/* Now read in the rest of the data to our receive buffer */
	eth_rx_buf_ptr = eth_pkt_rx_buf;
	temp = received_frame.frame_size - ETH_HEADER_LEN;

	if(temp & 1)	// force number of bytes to read to an even number
		temp++;
	while(temp)
	{
		* (UINT16*)eth_rx_buf_ptr = smsc_read_ns(REG_DATA1);
		eth_rx_buf_ptr +=2;
		temp -=2;
	}

	/* If the frame size was even, then we still need to read out the Control Word at the end */
	if( !(frame_status & ODDFRM) )
		temp = smsc_read_ns(REG_DATA1);

	/* Reset pointer to the begining of the buffer */
	eth_rx_buf_ptr = eth_pkt_rx_buf;

	/* Free up that memory */
	BANKSWITCH(2);  
	while ( smsc_read(REG_MMUCR) & BUSY );		/* wait for MMU to become not busy */
	smsc_write(REG_MMUCR,0x80);					/* remove and release top of Rx FIFO */
//	while ( smsc_read(REG_MMUCR) & BUSY );		/* wait for MMU to become not busy */


 	/* Give the next layer data start buffer index from the start	*/
 	received_frame.buf_index = 0;
 	
	eth_info.total_rx_packets++;

 	ETH_DEBUGOUT("Ethernet Frame Received\n\r");
 	
 	return(TRUE);						/* Indicate we got packet */

}

/** \brief Initialize transmission of new packet
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\param page Address in Ethernet controller where outgoing packet
 *		buffered
 *
 *	This function is used to initialize transmission of an Ethernet 
 *	packet. Packet is created starting from <i>page</i> address.
 *
 *	Do not invoke this function directly, but instead use 
 *	NETWORK_SEND_INITIALIZE() macro.
 */
void InitTransmission (UINT8 page)
{
	/* Simply set up pointer for transmit buffer used by outNE2000again() */
	eth_tx_buf_ptr = eth_pkt_tx_buf;
}

/** \brief Write Ethernet Header to transmission buffer
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\param frame Pointer to ethernet_frame structure containing information
 *		about the new Ethernet frame
 *
 *	Invoke this function (through NETWORK_ADD_DATALINK() macro) to create
 *	an Ethernet header in the NIC's transmit buffer. Do this only after
 *	invoking NETWORK_SEND_INITIALIZE() macro.
 *
 */
void NE2000WriteEthernetHeader (struct ethernet_frame* frame)
{
	INT8 i;
	
	/* Write destination Ethernet Address	*/
	
	for(i=ETH_ADDRESS_LEN-1; i >= 0; i--) {
		outNE2000again(frame->destination[i]);
	}

	/* Write sender (our) Ethernet address	*/
	
	for(i=ETH_ADDRESS_LEN-1; i >= 0; i--) {
		outNE2000again(frame->source[i]);
	}
	
	/* Write protocol						*/
	
	outNE2000again( (UINT8)(frame->protocol >> 8) );
	outNE2000again( (UINT8)frame->protocol );

}

/** \brief Initialize NE2000 in preparation for remote DMA
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\param page Address for the DMA
 *
 *	
 */
void NE2000DMAInit (UINT8 page)
{
 	
}

/** \brief Initialize reading from NE2000
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\param pos Position from the start of current frame from which DMA
 *		is initialized.
 *
 *	Invoke this function (through NETWORK_RECEIVE_INITIALIZE() macro) 
 *	to prepair NIC for reading from the specified position.
 */
void NE2000DMAInit_position (UINT16 pos)
{
	/* Simply set up pointer for receive buffer used by inNE2000again() */
	eth_rx_buf_ptr = eth_pkt_rx_buf + pos;
}



/** \brief Instruct NIC to send the Ethernet frame
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *	\param len Length of packet stored in the NIC's buffer
 *	\warning
 *		\li Minimum Ethernet packet size is 64 bytes
 *
 *	Invoke this function (through NETWORK_COMPLETE_SEND() macro) when
 *	the whole packet is formed inside NIC's memory and is ready to be 
 *	sent. Proper length of the packet must be supplied so that NIC
 *	knows how much data to put on the line.
 */
void NE2000SendFrame ( UINT16 len )
{
 	/* After filling Tx buffer, call this function to send it */
 	/* Input: 	Len: packet length							  */

	UINT16 temp,num_pages,txpktno;

 	/* Check if we need to insert Pad bytes */
 	while(len < 50)
	{
 		outNE2000again((BYTE)0x00);
 		len++;
 	}
 	
 	len += ETH_HEADER_LEN;
	

#if (POLLED_ETHERNET)
	BANKSWITCH(0);
	smsc_write(REG_TCR,smsc_read(REG_TCR) | TXENA);    /* Set TXENA bit to enable transmits. This bi may have 
												  been cleared by an error with the last packet, so we
												  have to make sure it's set */
#endif      /* POLLED_ETHERNET */

	/* Allocate transmit memory in the Ethernet chip */
	/* Add 6 bytes for status word, byte count and control word */
	/* Determine the number of 256 byte pages that will be needed */
	num_pages = (len+6) / 256;		// NOTE: a value of 0 = 1 page, 1 = 2 pages, etc...

	BANKSWITCH(2);
	while(smsc_read(REG_MMUCR) & 0x01);				// Wait for BUSY bit to be clear
	smsc_write(REG_MMUCR,(0x20 | num_pages));		/* Write ALLOCATE TX MEM command along with the number of
												   pages needed */ 
	/* Sit in a loop and wait for space to get allocated. If we sit too long, then 
	   we know that the incoming receive packets have used up all the memory inside
	   the Ethernet chip, so we need to start taking some of them out */
	temp = 50;		// How many times we should check before we flush the Ethernet chip
	while(1)
	{
		/* Keep checking the Interrupt Status Reg for the ALLOC INT bit to go high */
		if ( smsc_read(REG_IST) & ALLOC_INT )
		{
			/* Find out what the packet number is of the memory we just allocated */
			txpktno = smsc_read(REG_ARR);
			
			/* Check if allocation failed by checking the FAILED bit */
			if( txpktno & 0x0080 )
				while(1);		/* ERROR - Allocation failed */

			/* Change the Packet Number register to the packet we just allocated */
			smsc_write(REG_PNR,txpktno);
			break;
		}
		// Flush incoming packets to make room...
		if(--temp == 0)
		{
			while(smsc_read(REG_MMUCR) & 0x01);		// Wait for BUSY bit to be clear
			smsc_write(REG_MMUCR,0x80);				/* Remove and relase top of Rx FIFO */
			while(smsc_read(REG_MMUCR) & 0x01);		// Wait for BUSY bit to be clear

			temp = 10;
		}
	}

	BANKSWITCH(2);
	smsc_write(REG_PTR,AUTO_INCR);	/* Set AUTO INCR. (to inc FIFO pointer automatically)
								   Clear RCV (to signify Tx FIFO
								   Clear READ (to signify write)
								   Celar ETEN (no Early trans underrun detection)
								   Clear AutoTx (we will manually enqueue this packet)
								   Pointer value is 0 (meaning we will start at the very begining of the FIFO) */

	/* Start writting data in */
	smsc_write(REG_DATA1,0x0000);        /* write status word */

	/* Now write the data out to the Ethernet chip */
	eth_tx_buf_ptr = eth_pkt_tx_buf;
	temp = len & 0xFFFE;				/* force to even number */

	/* Write in byte count (must always be even) */
	/* Add 6 for status word, byte count and control word */
	smsc_write(REG_DATA1,temp+6);		

	while(temp)
	{
		// write a word at a time
		smsc_write_ns(REG_DATA1, *(UINT16 *)eth_tx_buf_ptr);
		eth_tx_buf_ptr += 2;
		temp -= 2;
	}
	/* Now write the control word. If there is 1 left byte to send, set the */
	/* ODD bit to indicate that the bottom word is the last byte */
	if(len & 1)
	{
		/* odd number of data */
		smsc_write(REG_DATA1, *eth_tx_buf_ptr | 0x2000 );	/* Set ODD bit in control word */
	}
	else
	{
		/* even number of data */
		smsc_write_ns(REG_DATA1,0);		/* Write the control word */
	}


	/* Now write queue the packet for tranmitting by writting to the MMU Command
	   Register. We have to do this manually because we are not using Auto Transmit
	   The packet number to be enqueued is taken from the PNR register which we haven't
	   changed yet. */
	smsc_write(REG_MMUCR,0xC0);	/* Write ENQUEUE PACKET NUMBER INTO TX FIFO command */

	BANKSWITCH(0);
	/* Keeps checking transmit status register for the result of the transmit */
	while(1)
	{
		temp = smsc_read(REG_EPHSRS);
		if( temp & TX_SUC)
		{
			/* Transmit successful */
			/* Because we set the AutoRelease bit in the CTR register, the memory for
			   packet we just transmitted is automatically released because TX was successful */
			/* Can clear TXEN bit here in TCR if you want to */
			eth_info.total_tx_packets++;
			return;
		}
		/* Check if any error bits were set */
		if( temp & (TX_UNRN|SIXTEEN_COL|LAT_COL) )
		{
			break;			/* Deal with ERROR */
		}
		if( smsc_read(REG_TCR) & (MONCSN|STPSQET) )
		{
			break;			/* Deal with ERROR */
		}
	}

	/* If this far, then an error has occured */
	/* Put failed packet number in PNR register so we can free memory */
//	BANKSWITCH(2);  
//	temp = (smsc_read(REG_FIFO) & FIFO_TX_MASK) << 8;	/* get current FIFO packet number */
//	smsc_write(REG_PNR,temp);							/* set that packet as accessible */

	while ( smsc_read(REG_MMUCR) & BUSY );		/* wait for MMU to become not busy */
			smsc_write(REG_MMUCR,0xA0);		/* Write RELEASE SPECIFIC PACKET command (packet number
										       is in PNR) */
	while ( smsc_read(REG_MMUCR) & BUSY );		/* wait for MMU to become not busy */

}

/** \brief Put NE2000 to sleep mode
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *
 *	Invoke this function to put Ethernet controller to sleep mode.
 *
 */
void NE2000EnterSleep (void) 
{
	if (EtherSleep)
		return;

	EtherSleep = 1;
}

/** \brief Restore NE2000 from sleep mode
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 19.02.2002
 *
 *	Invoke this function to instruct NE2000 to exit sleep mode and 
 *	continue normal operation.
 */
void NE2000ExitSleep (void) 
{
	if (EtherSleep) {
		EtherSleep = 0;
	}
}



/*************************************************************************
 delay function
 *************************************************************************/
static void waitalittlebit(void)
{
   volatile unsigned int junk;
   junk = smsc_read(4);
   junk = smsc_read(4);
   junk = smsc_read(4);
}

static void smsc_write(unsigned int offset, unsigned int tdata)
{

   tdata = (unsigned int)( (tdata>>8) | (tdata<<8) );

   *((unsigned int *)(BASE_ADDR+(unsigned int)(offset))) = tdata;
}
/* write without byte swap */
static void smsc_write_ns(unsigned int offset, unsigned int tdata)
{
   *((unsigned int *)(BASE_ADDR+(unsigned int)(offset))) = tdata;
}

static unsigned int smsc_read(unsigned int offset)
{
   unsigned int retval;

   retval = *((volatile unsigned int *)(BASE_ADDR+(unsigned int)(offset)));
   retval = (unsigned int)( (retval>>8) | (retval<<8) );

   return (retval);
}

/* read without byte swap at the end (ns = No Swap)*/
static unsigned int smsc_read_ns(unsigned int offset)
{
   unsigned int retval;

   retval = *((volatile unsigned int *)(BASE_ADDR+(unsigned int)(offset)));
   return (retval);
}
