/* ---------------------------------------------------------------------------
             Copyright (c) 2004-2005 Micrel, Inc.  All rights reserved.
   ---------------------------------------------------------------------------

    cliParser.c - Parse the arguments of a command

    Author      Date        Description
    PCD         10/08/04    Created file.
   ---------------------------------------------------------------------------
*/

/* Include the required header files */
#include <inet/system.h>
#include <string.h>
#include "cliTable.h"



#define MAX_CLI_STRING_LEN   16         /* max number of characters of CLI command */
#define MAC_ARGV             10

#define PROMPT          "=>"

/*  Local variables */

static  char cliString[MAX_CLI_STRING_LEN];

#if (0)
static  char anmemb[] = { "|_?*-./:," };   /* for QC test: SEPARATE CLI ARGV BY ',' */
#endif

#if (1)
static  char anmemb[] = { "|_?*-./:" };    /* normal: SEPARATE CLI ARGV BY SPACE  */
#endif 

static  char *(pArgv[MAC_ARGV]);         /* store argv pointer */


/*  Global variables */

const char arrow_k[4][3][4]={
       {"\033\\A",        "\033[A",        "\033OA" },
       {"\033\\B",        "\033[B",        "\033OB" },
       {"\033\\C",        "\033[C",        "\033OC" },
       {"\033\\D",        "\033[D",        "\033OD" },
};

extern char   rcv_ch_buf[];         /* store rcv character from UART  */
extern char  *pRcv_ch_buf;
extern UINT16 rcv_ch_size;
extern UINT16 rcv_ch_buf_count;
extern UINT8  rcv_ch_buf_filled;	/* used to signal when a Rx a RETURN from UART */


/* Function Prototypes */

UINT8 whiteSpace(char);
UINT8 eol(char);
UINT8 alphanum(char); 
char  isArrow(void *);
UINT8 isASCII(char);
UINT8 isDigit(UINT8);
void  cliParser(void); 
void  uartReceiveEnter(void);


/******************************************************************************
*
* whiteSpace -
*
*
* RETURNS:
* This routine returns TRUE if the character passed in is a bland or tab,
* otherwise FALSE.
*/
UINT8 whiteSpace(char c) 
{
  if ( 
       ( ' ' == c ) 
       || ( '\t' == c ) 
#ifdef KS_QC_TEST
       || ( ',' == c )     /* for QC test: SEPARATE CLI ARGV BY ',' */
#endif
     ) 
      return(TRUE);
  return(FALSE);
}

/******************************************************************************
*
* eol -
*
*
* RETURNS:
* This routine returns TRUE if the character passed in is a new line or
* carriage return, otherwise FALSE.
*/
UINT8 eol(char c) 
{
  if (( '\n' == c ) || ( '\r' == c ) || ( '\0' == c )) return(TRUE);
  return(FALSE);
}

/******************************************************************************
*
* alphanum -
*
*
* RETURNS:
* This routine returns TRUE if the character passed in is between A and Z,
* a and z, 0 and 9, or is a member of anmemb, otherwise FALSE.
*/
UINT8 alphanum(char c) 
{
  char *s;

  if (( 'A' <= c) && ( 'Z' >= c )) return(TRUE);
  if (( '0' <= c) && ( '9' >= c )) return(TRUE);
  if (( 'a' <= c) && ( 'z' >= c )) return(TRUE);

#if (1)
  s = anmemb;

  while (*s)
    if (*s++ == c) return(TRUE);
#endif

  return(FALSE);
}

/******************************************************************
*
* function: isArrow
*
*******************************************************************/
char isArrow(void *buf)
{
   char i, j;

   for(i=0; i<4; i++)
   {
       for(j=0; j<3; j++)
       {
          if(!strcmp((const char *)buf, (const char *)&arrow_k[i][j]))
              return((char)(i+ARROW_U));
       }
   }
   return(0);
}

/******************************************************************
*
* function: isASCII
*
*******************************************************************/
UINT8 isASCII(char x)
{
    if((x>=0x41&&x<=0x5b)||(x>=0x61&&x<=0x7b))
        return TRUE;
    else 
        return FALSE;
};

/******************************************************************
*
* function: isDigit
*
*******************************************************************/
UINT8 isDigit(UINT8 x)
{
    if(0x30<=x&&0x39>=x) 
        return TRUE;
    else 
        return FALSE;
}


/******************************************************************************
*
* strtok_p -
*
* Break down a string into tokens.
*
* Parameters:
*    s1 - string to break into tokens
*    s2 - the separators
*    ppLast - pointers to serve as string index 
*
* RETURNS: Pointer to the first character of a token, or NULL if there is no token.
*/
char FAR *strtok_p( s1, s2, ppLast )  
char FAR *s1;                         
const char FAR *s2;                   
char FAR **ppLast;
{
    static char FAR  *_tokp = NULL;

    char FAR *ptr;
    int  n;

    if ( s1 == 0 ) s1 = _tokp;
    s1 += strspn( s1, s2 );
    if ( ( ptr = strpbrk( s1, s2 ) ) == 0 ) {
        for ( _tokp = s1; *_tokp; _tokp++ ) ;
        *ppLast = '\0';
        return ( ( *s1 == '\0' ) ? ( char * )0 : s1 );
    }
    if ( *ptr != '\0' ) *ptr++ = '\0';
    _tokp = ptr;
    *ppLast = _tokp;
    return ( s1 );                      
}




/******************************************************************************
*
* getToken -
*
*
* RETURNS: NULL
*/
void **getToken(void FAR *p, void FAR **pp, void FAR *pbuf, UINT16 FAR mac_argv)
{
	  UINT16 j=0;
      UINT16 FAR *argc = (UINT16 FAR *)p;
	  char FAR **argv = (char FAR **)pp;
	  char FAR *buf=(char FAR *)pbuf;


	  while ( mac_argv > *argc )
      {
	     /* Skip over white spaces */
	     while (whiteSpace(buf[j]))
            j++;

         if (eol(buf[j]))
            break;

         /* Point16 to beginning of token */
         argv[(*argc)++] = (char FAR *)&(buf[j++]);

         while (alphanum(buf[j]))
            j++;

         if (eol(buf[j]))
         {
            buf[j++] = '\0'; break;
         }

         /* don't put NULL for the last argv */
         if ( *argc < mac_argv )
             buf[j++] = '\0';
      }

      /* NULL terminate the argv list. */
	  //argv[*argc] = NULL;
      return (void **)argv;
}

/*****************************************************************************
Name:        cliParser       
Parameters:  none                   
Returns:     none   
Description: Compare the string from the rcv_ch_buf[] to the cliTbl[] 's 'cliString',
             if matched, perform the its asociated routine.
*****************************************************************************/

void cliParser(void) 
{
    char          *pRcvBuf; 
    UINT8          i=0;
    UINT8          found=FALSE;
    UINT16         argc=0;
    CLI_TBL_ENTRY *pCliTbl;



    memset ( cliString, 0, MAX_CLI_STRING_LEN);
    memset ( pArgv, 0, MAC_ARGV);

    /* 
     *  Parser for CLI command string.
     */

    /* Set pointer to the begining of the rcv char buffer */
    pRcvBuf = rcv_ch_buf;
    if ( *pRcvBuf == NULL )         
        return;                     /* empty rcv character buffer */

    /* skip the white space */
    while ( whiteSpace(*pRcvBuf++) );     
    pRcvBuf--;

    /* Get the cli command string, string end with space or NULL */
    while (( *pRcvBuf != NULL ) && (i < MAX_CLI_STRING_LEN) )
    {
        // if ( *pRcvBuf == ' ') 
        if ( whiteSpace(*pRcvBuf) )     
            break;

        cliString[i++] =  *pRcvBuf++;
    }


    /* 
     *  Valify the CLI command string, if valid, perform the CLI command.
     */

    /* Compare the cliString to rcv_ch_buf[] to the cliTbl[] */
    for ( pCliTbl = cliTbl; pCliTbl->cliFunc != CLI_TBL_END; pCliTbl++ )
    {
        /* if matched, perform the its asociated routine */
        if ( !strcmp ((const char *)cliString, (const char *)pCliTbl->cliString ) )
        {
            found = TRUE;
    /* 
     *  Parser for CLI command parameters.
     */

            /* Cut the line up into tokens */
	        getToken((char FAR*)&argc,(char FAR **)pArgv,(char FAR*)pRcvBuf, (UINT16 FAR )pCliTbl->nArgc );

            if ( argc != pCliTbl->nArgc )
                usagDisplay ( pCliTbl->indexHelpMsg );
            else
                if ( !(pCliTbl->cliFunc ( (void FAR *)&argc, (void FAR **)&pArgv )) )
                   usagDisplay ( pCliTbl->indexHelpMsg );

            break;
        }

    } /* for ( pCliTbl = cliTbl; pCliTbl->cliFunc != CLI_TBL_END; pCliTbl++ ) */

#if (1)    
    if ( !found )
        unknownDisplay ();
#endif

    if ( found )
       text_write(PROMPT);          /* send the Prompt sign ("=>") to screen */


}

/*****************************************************************************
Name:        uartReceiveEnter       
Parameters:  none                   
Returns:     none   
Description: Parser the rcv_ch_buf[] that stored the character received 
             from keyboard.	   
*****************************************************************************/

void uartReceiveEnter(void) 
{
    int i;


    /* not receive 'ENTER' key yet */
    if ( !rcv_ch_buf_filled  )
        return;

    /* 'ENTER' key has received, parser the input character string. */
    cliParser();

    /* Reset pointer to the begining of the rcv char buffer */
    pRcv_ch_buf = rcv_ch_buf;
    memset ( (char *)pRcv_ch_buf, 0, rcv_ch_size );

    /* allow the next key character to come in from keyboard */
	rcv_ch_buf_filled = FALSE;
    rcv_ch_buf_count = 0;
}
