/***********************************************************************
 * DISCLAIMER:                                                         *
 * The software supplied by Renesas Technology America Inc. is         *
 * intended and supplied for use on Renesas Technology products.       *
 * This software is owned by Renesas Technology America, Inc. or       *
 * Renesas Technology Corporation and is protected under applicable    *
 * copyright laws. All rights are reserved.                            *
 *                                                                     * 
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS,  *
 * IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO IMPLIED 		   *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE  *
 * APPLY TO THIS SOFTWARE. RENESAS TECHNOLOGY AMERICA, INC. AND        *
 * AND RENESAS TECHNOLOGY CORPORATION RESERVE THE RIGHT, WITHOUT       *
 * NOTICE, TO MAKE CHANGES TO THIS SOFTWARE. NEITHER RENESAS           *
 * TECHNOLOGY AMERICA, INC. NOR RENESAS TECHNOLOGY CORPORATION SHALL,  * 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR         *
 * CONSEQUENTIAL DAMAGES FOR ANY REASON WHATSOEVER ARISING OUT OF THE  *
 * USE OR APPLICATION OF THIS SOFTWARE.                                *
 ***********************************************************************/

/************************************************************************
 *											   						 	*
 *    DESCRIPTION: main routine for SIO - UART mode 		 	 	 	*
 *											  						 	*
 *    AUTHOR: System LSI BU, Applications Engineering 	 			 	*
 *			  January 2004					   						 	*
 *											   						 	*
 *	This program communicates from the M16C/62P to a terminal program	*
 * 	via UART0 and RS232.  UART0 configuration is:						*
 *	38400 baud, 8-bit data, 1 stop bit, no parity, no flow control. 	*
 *											   						 	*
 * 	This program was originally written for the MSV30262 Board.			*
 *	To see the data sent, use Hyperterminal configured as mentioned		*
 *	above and connect a null modem cable between P1 and the PC's		*
 *	com port.															* 
 *											   						 	*
 *	An incrementing data (0 to 9) is sent to the Hyperterminal window.	*
 *	To stop receiving data, press z on the keyboard. To resume, press	*
 * 	any key.															*
 ************************************************************************/
/*=========================================================================
	When running this demo on the SKP16C62P Board must be connected to 
	the expansion board (i.e. COM Board) for RS-232C connectivity. The
	expansion board has the RS-232C driver/receiver and DB9 D-Sub connector.
*=========================================================================*/

/* Include the required header files */
#include <inet/system.h>
#include <string.h>
#include "skp_bsp.h"	          /* include SKP board support package */


#define BAUD_RATE	    19200     /* 38400/2 */
#define RCV_CH_SIZE     240

#define ESC             0x1B      /* ESC special key */
#define DEL             0x7f      /* DELETE key */
#define BACKSPACE       8         /* BACKSPACE key */
#define SPACE           ' '       /* SPACE key */
#define PROMPT          "=>"

#define ARROW_U     101
#define ARROW_D     102
#define ARROW_R     103
#define ARROW_L     104

#define IS_ARROW(x)             (isArrow(x))
#define C_LEFT(x)               (c_left(x))
#define TEXT_WRITE(x)           (text_write(x))

extern char isArrow(void *);


/* Setup interrupt routine for UART0 receive. This must also be setup in the 
   vector table in SECT30_UART.INC */

#pragma INTERRUPT U0rec_ISR
void U0rec_ISR (void);

/* Function Prototypes */
void text_write (FAR char * msg_string);
void mcu_init(void);
void uart_init(void);

/*  Global variables */
char   U0_in;		 		          /* declare UART0 recieve variable */
char   rcv_ch_buf[RCV_CH_SIZE];       /* store rcv character from UART  */
char  *pRcv_ch_buf;
char   prev_rcv_ch_buf[RCV_CH_SIZE];  /* store history rcv character from UART  */
UINT16 rcv_ch_size = RCV_CH_SIZE;
UINT16 rcv_ch_buf_count=0;
UINT8  rcv_ch_buf_filled = FALSE;	  /* used to signal when a Rx a RETURN from UART */

char   esc_buf[4];                    /* store ESC key ( 3-character)  */
UINT8  iEsc=0;
UINT8  fESC = FALSE;	              /* used to signal when a Rx a ARROW key from UART */
UINT8  fEcho = TRUE;	              /* TRUE to echo received character to screen, FALSE, otherwise. */


/* String constants used for screen output **********************************/
const char cmd_clr_scr[] = {27,'[','2','J',0};
const char cmd_cur_home[] = {27,'[','H',0};



void c_left(int i)
{
     char chs[3]="00",
     string1[12]="\x1B[",

     itoa(i,chs);                   /* transfer i to character */
     strcat((char FAR *)string1,(char FAR *)chs);
     strcat((char FAR *)string1,(char FAR *)"D");
     text_write(string1);           /* send the command and i to buffer */
}

/*****************************************************************************
Name:    UART0 Receive Interrupt Routine       
Parameters:  none                   
Returns:     none   
Description: Interrupt routine for UART0 receive
			 Reads character received from keyboard and stores U0_in variable	   
*****************************************************************************/
void U0rec_ISR(void)
{
    char   arrow_key;

	while(ri_u0c1 == 0);	        /* make sure receive is complete */

	U0_in = (char) u0rb;            /* read in received data */

    /* has received 'ENTER' key, wait for finished parser input string */
    if ( rcv_ch_buf_filled )
        return;

    /* wait for previous transmission to complete  */
	while(ti_u0c1 == 0);  

	if ((U0_in == ESC) || (fESC) )  
    {                                /* ESC Special key, like arrow key */
        fESC = TRUE;
        if (iEsc < 2)
           esc_buf[iEsc++]=U0_in;    /* save to esc_buf[] for later process */
        else
        {                            /* got 3-character in the row, check what arrow key */
           esc_buf[iEsc++]=U0_in;    /* save last ESC char to esc_buf[]  */
           arrow_key = IS_ARROW((void *)esc_buf);
	       if ( (arrow_key == ARROW_U) || 
                (arrow_key == ARROW_D) )
           {                         /* repeat perform previous characters string */
               memcpy ( rcv_ch_buf, prev_rcv_ch_buf, RCV_CH_SIZE );
               TEXT_WRITE(rcv_ch_buf);  /* send the command line to screen */
               pRcv_ch_buf = (char *)(rcv_ch_buf + strlen(rcv_ch_buf) );
           }
           iEsc = 0;
           fESC = FALSE;
           memset ( (char *)esc_buf, 0, 4 );
        }
    }
	else if (U0_in == BACKSPACE)     /* 'BACKSPACE' key */
    {
        if ( fEcho )
        {
           C_LEFT(1);                /* move cursor to the left */
	       while(ti_u0c1 == 0);         
	  	   u0tb = SPACE;			 /* send ' ' to the screen */
           C_LEFT(1);                /* move cursor to the left */
        }

        pRcv_ch_buf--;
        *pRcv_ch_buf = 0;            /* clear last character from rcv_ch_buf[] */
    }
	else if (U0_in == 0x0D)          /* 'ENTER' key */
    {		  
        /* ENTER key */
        if ( fEcho )
        {
	  	   u0tb = 0x0D;			     /* echo 'ENTER' (0D 0A) to screen */
	       while(ti_u0c1 == 0);         
	  	   u0tb = 0x0A;			  
           TEXT_WRITE(PROMPT);       /* send the Prompt sign ("=>") to screen */
        }
        rcv_ch_buf_filled = TRUE;	 /* signal that the buffer has been filled */
                                     /* save this character string to history buffer */
        memcpy ( prev_rcv_ch_buf, rcv_ch_buf, RCV_CH_SIZE );
	}	
                                     /* OTHER key */
    else if ( (U0_in != 0x0A) && 
              ( !fESC ) &&
              ( rcv_ch_buf_count++ < RCV_CH_SIZE) )         
    {                                /* don't echo for 2nd character of 'ENTER' (0D 0A) */
        *pRcv_ch_buf++ = U0_in;      /* save to rcv_ch_buf[] */
        if ( fEcho )
        {
	  	   u0tb = U0_in;		     /* echo to screen */
        }
    }
}							

/*****************************************************************************
Name:		uart_init   
Parameters:	None				 
Returns:	None
Description: Uart0 initialization - 38400 baud, 8 data bits, 1 stop bit, no parity.
*****************************************************************************/
void uart_init(void) 
{

    u0brg = (unsigned char)(((XIN_FREQ/16)/BAUD_RATE)-1);	// set UART0 bit rate generator
		 /*
	  	  bit rate can be calculated by:
	  	  bit rate = ((BRG count source / 16)/baud rate) - 1

		  in this example: BRG count source = f1 (24MHz - Xin in SKP16C62P)
						   baud rate = 19200
						   bit rate = ((24MHz/16)/19200) - 1 = 38 

	  	  ** one has to remember that the value of BCLK does not affect BRG count source */

  	ucon = 0x00; 		// UART transmit/receive control register 2
		 /*
		  00000000; 	// transmit irq not used
		  ||||||||______UART0 transmit irq cause select bit, U0IRS
	   	  |||||||_______UART1 transmit irq cause select bit, U1IRS
	   	  ||||||________UART0 continuous receive mode enable bit, U0RRM - set to 0 in UART mode
	  	  |||||_________UART1 continuous receive mode enable bit, U1RRM	- set to 0 in UART mode
	   	  ||||__________CLK/CLKS select bit 0, CLKMD0 - set to 0 in UART mode
	   	  |||___________CLK/CLKS select bit 1, CLKMD1 - set to 0 in UART mode
	   	  ||____________Separate CTS/RTS bit, RCSP
	  	  |_____________Reserved, set to 0 */

  	u0c0 = 0x10; 		// UART0 transmit/receive control register 1
		 /*
		  00010000;		// f1 count source, CTS/RTS disabled, CMOS output  
		  ||||||||______BRG count source select bit, CLK0
	   	  |||||||_______BRG count source select bit, CLK1
	   	  ||||||________CTS/RTS function select bit, CRS
	  	  |||||_________Transmit register empty flag, TXEPT
	   	  ||||__________CTS/RTS disable bit, CRD
	   	  |||___________Data output select bit, NCH
	   	  ||____________CLK polarity select bit, CKPOL 		- set to 0 in UART mode
	  	  |_____________Transfer format select bit, UFORM 	- set to 0 in UART mode */

  	u0c1 = 0x00; 		// UART0 transmit/receive control register 1
		 /*
		  00000000;		// disable transmit and receive 
		  ||||||||______Transmit enable bit, TE
	   	  |||||||_______Transmit buffer empty flag, TI
	   	  ||||||________Receive enable bit, RE
	  	  |||||_________Receive complete flag, RI
	   	  ||||__________Reserved, set to 0
	   	  |||___________Reserved, set to 0
	   	  ||____________Data logic select bit, U0LCH
	  	  |_____________Error signal output enable bit, U0ERE */

  	u0mr = 0x05;		// UART0 transmit/receive mode register, not reversed
		 /*
		  00000101;		// 8-bit data, internal clock, 1 stop bit, no parity
		  ||||||||______Serial I/O Mode select bit, SMD0
	   	  |||||||_______Serial I/O Mode select bit, SMD1
	   	  ||||||________Serial I/O Mode select bit, SMD2
	  	  |||||_________Internal/External clock select bit, CKDIR
	   	  ||||__________Stop bit length select bit, STPS
	   	  |||___________Odd/even parity select bit, PRY
	   	  ||____________Parity enable bit, PRYE
	  	  |_____________TxD, RxD I/O polarity reverse bit */

  	u0tb = u0rb;		// clear UART0 receive buffer by reading
  	u0tb = 0;			// clear UART0 transmit buffer

    DISABLE_IRQ			// disable irqs before setting irq registers
	s0ric = 0x04;		// Enable UART0 receive interrupt, priority level 4	
	ENABLE_IRQ			// Enable all interrupts

  	u0c1 = 0x05; 		// UART0 transmit/receive control register 1
		 /*
		  00000101;		// enable transmit and receive 
		  ||||||||______Transmit enable bit, TE
	   	  |||||||_______Transmit buffer empty flag, TI
	   	  ||||||________Receive enable bit, RE
	  	  |||||_________Receive complete flag, RI
	   	  ||||__________Reserved, set to 0
	   	  |||___________Reserved, set to 0
	   	  ||____________Data logic select bit, U0LCH
	  	  |_____________Error signal output enable bit, U0ERE */


   /* Set pointer to the begining of the rcv char buffer */
   pRcv_ch_buf = rcv_ch_buf;
   memset ( (char *)rcv_ch_buf, 0, RCV_CH_SIZE );
   memset ( (char *)prev_rcv_ch_buf, 0, RCV_CH_SIZE );
   memset ( (char *)esc_buf, 0, 4 );

}

/*****************************************************************************
Name:   	 text_write        
Parameters:  msg_string -> the text string to output                    
Returns:     none   
Description: The following sends a text string to the terminal program     
*****************************************************************************/
void text_write ( FAR char * msg_string)
{
	char i;

	for (i=0; msg_string[i]; i++)		// This loop reads in the text string and 
    {
		while(ti_u0c1 == 0); 			//  puts it in the UART 0 transmit buffer 
		u0tb = msg_string[i];
	}
}

/*****************************************************************************
Name:		mcu_init   
Parameters:	None				 
Returns:	None
Description: Clock and port initialization routine. BCLK is switched to
             f1 (divide by 1) in the ncrt0.a30 startup file.
*****************************************************************************/
void mcu_init(void) 
{

	/* LED initialization */
 	// ENABLE_LEDS 	
}


