/* configNet.h - network configuration header for END driver*/

/* Copyright 1984-2002 Wind River Systems, Inc. */

/*
modification history
--------------------
01d,16jun03,h_k  fixed endDevTbl. (SPR #89022)
01c,11jun02,zl   added C++ header protection.
01b,30mar01,zl   re-written.
01a,09jan01,rsh  written based on hmse7751/configNet.h 01c.
*/
 
#ifndef INCconfigNeth
#define INCconfigNeth

#ifdef __cplusplus
extern "C" {
#endif

#if (defined(INCLUDE_END) && defined(INCLUDE_NETWORK))

/* includes */

#include "vxWorks.h"
#include "end.h"

/* defines */
 
#ifdef INCLUDE_LN_97X_END

#define END_LOAD_FUNC_LN_97X	sysLn97xEndLoad
#define END_BUFF_LOAN_LN_97X	1
 
IMPORT END_OBJ * END_LOAD_FUNC_LN_97X (char *, void*);

#endif /* INCLUDE_LN_97X_END */

#ifdef INCLUDE_KS884X_END
/* ks884x Ethernet Device with PCI bus interface */

#define END_LOAD_FUNC_KS_884X	sysKs884xEndLoad
#define END_BUFF_LOAN_KS_884X	1
 
IMPORT END_OBJ * END_LOAD_FUNC_KS_884X (char *, void*);

#endif /* INCLUDE_KS884X_END */

#ifdef INCLUDE_KS884X_SHBUS_END

/* ks884x Ethernet Device with Generic interface */

#define END_LOAD_FUNC_KS884X_SH    ks884xSHEndLoad
#define END_BUFF_LOAN_KS884X	     4

/*
 * Load string format
 * <unit>:<devMemAddr>:<vecNum>:<intLvl>:<memAdrs>:<memSize>:<<memWidth>:<endianMode>:<offset>:<flags>
 *
 * devMemAddr (hex)	- base addr for ks884x generic bus device (KS_MBD_ADDR)
 * vecNum (decimal)	- interrupt vector number (INT_VEC_KS884X)
 * intLvl (decimal)	- interrupt level  (INUM_IRL2)
 * memAdrs(hex)	    - Memory pool address or NONE (NONE).
 * memSize (hex)	- Memory pool size or zero (ZERO)
 * memWidth (decimal)	- Memory system size, 1, 2, or 4 bytes (optional) (4-Bytes)
 * boardBusEndianMode (decimal) - Board Bus Endian mode options (1- board bus is big endian mode)(Big Endian).
 * offset (decimal)  - Memory alignment offset (2).
 * flags (decimal) - Device specific flags reserved for future use (0).

 * Note that unit number is prepended in muxDevLoad, so we
 * don't put it here!
 */

#define END_LOAD_KS884X_STR		   "0xb0000000:27:27:-1:-1:4:1:2:0"    

 
IMPORT END_OBJ * END_LOAD_FUNC_KS884X_SH (char *, void*);

#endif /* INCLUDE_KS884X_SHBUS_END */

/* The END_LOAD_STRING is defined empty and created dynamicaly */
 
#define END_LOAD_STRING 	""
 
/* define IP_MAX_UNITS to the actual number in the table. */
 
#ifndef IP_MAX_UNITS
#define IP_MAX_UNITS		(NELEMENTS(endDevTbl) - 1)
#endif  /* ifndef IP_MAX_UNITS */
  
/* each entry will be processed by muxDevLoad() */

END_TBL_ENTRY endDevTbl [] =
    {
#ifdef INCLUDE_LN_97X_END
    {0, END_LOAD_FUNC_LN_97X, END_LOAD_STRING, END_BUFF_LOAN_LN_97X,
     NULL, FALSE},
#endif

#if (0)
#ifdef INCLUDE_KS884X_END
    {1, END_LOAD_FUNC_KS_884X, END_LOAD_STRING, END_BUFF_LOAN_KS_884X,   
     NULL, FALSE},                                                       /* KS884x with PCI bus interface */
#endif
#endif

#if (0)
#ifdef INCLUDE_KS884X_SHBUS_END
    {1, END_LOAD_FUNC_KS884X_SH, END_LOAD_KS884X_STR, END_BUFF_LOAN_KS884X,   
     NULL, FALSE},                                                       /* KS884x with Generic bus interface */
#endif
#endif
    {0, END_TBL_END, NULL, 0, NULL, FALSE},    /* must be last */
    };


END_TBL_ENTRY ks_endDevTbl [] =
    {
#ifdef INCLUDE_KS884X_END
    {0, END_LOAD_FUNC_KS_884X, END_LOAD_STRING, END_BUFF_LOAN_KS_884X,   
     NULL, FALSE},                                                       /* KS884x with PCI bus interface */
#endif
#ifdef INCLUDE_KS884X_SHBUS_END
    {0, END_LOAD_FUNC_KS884X_SH, END_LOAD_KS884X_STR, END_BUFF_LOAN_KS884X,   
     NULL, FALSE},                                                       /* KS884x with Generic bus interface */
#endif
    {0, END_TBL_END, NULL, 0, NULL, FALSE},    /* must be last */
    };

#endif /* (defined (INCLUDE_END) && defined (INCLUDE_NETWORK)) */

#ifdef __cplusplus
}
#endif
#endif /* INCnetConfigh */
 
 
