/* flashFsLib.h */

/* Copyright 1984 - 2001 Wind River Systems, Inc. */
/* modification history */
/*-----------------------------------------------------------------------------
  12Aug05,pcd  To support Fujitsu MBM29LV160T flash chip. 
               This code will be used for MS7751RSE01 board
-----------------------------------------------------------------------------*/
/*
DESCRIPTION
This file contains header information for generic FLASH memory routines.


2 chip of 1M*16bit of Fujitsu MBM29LV160T memoy layout (A000-0000 to A03F-FFFF)
Total 4MB size with 32bit bus access.
------------------------------------------------------------------
Sector  Address Range    CPU Virtual  Stored
                         address
SA0     000000-01FFFF    A000-0000    bootRom (8  secotr:1MB)
SA1     020000-03FFFF    A002-0000                           
              .
              .
SA8     100000-11FFFF    A010-0000    vxWorks (22 sector:(3MB-128KB))
SA9     120000-13FFFF    A012-0000                                    
              .
              .
SA24    3C0000-3DFFFF    A03C-0000    NVM+    (1  secotr:64KB)
              .
              .
SA31    3E0000-3FFFFF    A03E-0000    Boot line (boot params) (1 sector:64KB)
*/

#ifndef	FLASH_FS_LIB_H
#define	FLASH_FS_LIB_H

#include <config.h>

#include "blkIo.h"

typedef struct		/* FLASH_DEV - FLASH disk device descriptor */
{
    BLK_DEV	flash_blkdev;	    /* generic block device structure */
    UINT32  flash_offset;	    /* offset location of the FLASH disk */
    char    *flash_cache;       /* memory for writing to sectors */
    UINT32	flash_blkOffset;	/* block offset of this device from flash_addr */
} FLASH_DEV;


#define BOOTROM_BASE_FS         "/rfa0"        
#define APPL_BASE_FS            "flash:" 

#ifdef  INCLUDE_NVM_SPACE
#define NVM_BASE_FS             "/rfa2"
#define NUM_RFA_DEVICES         3 
#else
#define NVM_BASE_FS             APPL_BASE_FS
#define NUM_RFA_DEVICES         2 
#endif

#define VX_FILENAME             APPL_BASE_FS "/vxWorks.Z"


#define FLASH_BANK_WIDTH           2  		                /* 32-bit flash (2 chip with 16 bit access mode) */

#define	FLASH_SECTOR_SIZE	       0x10000                  /* size of per-sector (64KB) */
#define	FLASH_BANK_SECTOR_SIZE	(  FLASH_SECTOR_SIZE * FLASH_BANK_WIDTH)



#define EB_FLASH_SECTORS           32               /* total sectors to be erased */

#define SECTOR_0	               0      
#define SECTOR_8                   8
#define SECTOR_30                  30
#define SECTOR_31                  31

/* Boot ROM Beginning sector number */
#define BOOT_BEGIN_SECTOR          SECTOR_0      

/* Application (vxWorks.Z) Beginning sector number */
#define APPLICATION_BEGIN_SECTOR   SECTOR_8      

/* NVM+ Beginning sector number */
#define NVM_BEGIN_SECTOR           SECTOR_30      

/* boot line (boot params) Beginning sector number */
#define FLASH_ADRS_BEGIN_SECTOR    SECTOR_31      



/* FLASH BASE ADDRESS */
#define FLASH_BASE_ADDRESS         TFFS_BASE_ADDR


/* Boot ROM Beginning address */
#define BOOT_SECTOR_BASE_ADDRESS        FLASH_BASE_ADDRESS                     

/* Application (vxWorks.Z) Beginning address */
#define APPLICATION_SECTOR_BASE_ADDRESS (FLASH_BASE_ADDRESS + \
                                         APPLICATION_BEGIN_SECTOR * FLASH_BANK_SECTOR_SIZE)

/* NVM+ Beginning address */
#define NVM_SECTOR_BASE_ADDRESS         (FLASH_BASE_ADDRESS + \
                                         NVM_BEGIN_SECTOR * FLASH_BANK_SECTOR_SIZE)

/* boot line (boot params) Beginning address */
/* #define FLASH_ADRS		(SH7751_P2_BASE + 0x3F8000)  in config.h */



/* for AMD AM29LV033 flash */
#define	FLASH_FS_DOS_SECTOR_SIZE	512 

                                                                                                  
/* Boot ROM number of sectors */
#define FLASH_RAW_NUM_SEC		8
#define FLASH_RAW_NUM_BLK		FLASH_RAW_NUM_SEC * (FLASH_BANK_SECTOR_SIZE/FLASH_FS_DOS_SECTOR_SIZE) /* 1M bytes */ 

/* Application (vxWorks.Z) number of sectors */
#define FLASH_FS2_NUM_SEC		22
#define FLASH_FS2_NUM_BLK		FLASH_FS2_NUM_SEC * (FLASH_BANK_SECTOR_SIZE/FLASH_FS_DOS_SECTOR_SIZE) /* (3M-128K) bytes */

/* NVM+ number of sectors */
#define FLASH_FS1_NUM_SEC		1
#define FLASH_FS1_NUM_BLK		FLASH_FS1_NUM_SEC * (FLASH_BANK_SECTOR_SIZE/FLASH_FS_DOS_SECTOR_SIZE) /* (64K) bytes */ 



typedef enum {

    FLASH_FS_SYNC = 0x10000

} FLASH_FS_CUSTOM_IOCTL_DEFINITIONS;

STATUS flashFsLibInit();
STATUS flashFsBlkRead (FLASH_DEV *, int, int, char *);
STATUS flashFsBlkWrite(FLASH_DEV *, int, int, char *);
STATUS flashFsIoctl(BLK_DEV *, int, int);
STATUS flashFsSync();
STATUS flashFsGetPhys(int, int *, int *);

IMPORT int flashDrvLibInit();
IMPORT int flashEraseSector(int);
IMPORT int flashEraseBank();
IMPORT int flashBlkRead(int, char *, unsigned int, unsigned int);
IMPORT int flashBlkWrite(int, char *, unsigned int, unsigned int);
IMPORT int flashDiagnostic();
IMPORT int flashSyncFilesystem();

#endif /* FLASH_FS_LIB_H */


