/* ks8842End.h - END Ethernet driver header for Micrel KS884X PCI Bus Ethernet driver  */

/* Copyright 1998-2002 Wind River Systems, Inc. */
/* 
 *	Copyright (c) 2004-2005, Micrel Semiconductor
 *
 *	This END Ethernet driver is for Micrel's KS8841 or KS8842 PCI Bus interface.
 */

/*
modification history
--------------------
*/
#ifdef KS_PCI_BUS 

#ifndef __INCKS8842END_H
#define __INCKS8842END_H

#ifdef __cplusplus
extern "C" {
#endif

#include "end.h"
#include "cacheLib.h"
#include "netinet/if_ether.h"
#include "wdLib.h"
#include "hardware.h"

/* define the various levels of debugging if the DRV_DEBUG is defined */    

#ifdef DRV_DEBUG
#    include "logLib.h"
#endif /* DRV_DEBUG */


/*
 * KS884X_MAX_IF is 1 if it is KS8841, 2 if it is KS8842.
 */
#define KS884X_MAX_IF            1			/* Max ks884X END device */


/* Definitions for the drvCtrl specific flags field */

#define KS_PROMISCUOUS_FLAG     0x001   /* set the promiscuous mode */
#define KS_MEM_ALLOC_FLAG       0x002   /* allocating memory flag */
#define KS_POLLING              0x020   /* polling flag */
#define KS_MULTICAST_FLAG       0x040   /* set the receive all multicast frames */
#define KS_MODE_MEM_IO_MAP      0x100   /* device registers memory mapped */
#define KS_MODE_DWIO            0x200   /* device in 32 bit mode */
#define KS_MODE_NOSWAP_MASTER	0x400   /* don't swap PCI master I/O ops */

/* standard ethernet header */
#define ENET_HEADER_SIZE                14
#define MINIMUM_ETHERNET_PACKET_SIZE	60
#define ETH_CRC_LENGTH					4
#define ETH_LENGTH_OF_ADDRESS			6

/* Definitions for PHY status */
#define SPEED_UNKNOWN					0
#define SPEED_10						10
#define SPEED_100						100
#define FULL_DUPLEX						1		/* default for full duplex */
#define HALF_DUPLEX						2


typedef struct pciResource             /* PCI_RESOURCES */
{
    UINT32      iobaseCsr;              /* Base Address Register 0 */
    UINT32      membaseCsr;             /* Base Address Register 1 */
    UINT32      vecNum;                 /* Interrupt Request Level */
    UINT32      intLevel;               /* Interrupt Request vector */
    UINT32      configType;             /* type of configuration */
    UINT32      pciBus;                 /* PCI Bus number */
    UINT32      pciDevice;              /* PCI Device number */
    UINT32      pciFunc;                /* PCI Function number */
    UINT32   	pciVendorID;		    /* PCI vendor ID (0x16C6) */
    UINT32		pciDevID;			    /* PCI device ID (0x8841 or 0x8842) */
    UINT32		pciRevID;			    /* PCI revision number (chip rev) */
    END_OBJ   * endIf;                  /* Pointer to END structure */
} PCI_RESOURCES;
  

extern PCI_RESOURCES ks884xPciRsrcs [];

/* Receive Descriptor */

typedef struct
{
    volatile UINT32 RxFrameControl;
    volatile UINT32 RxDMABufLen;
    volatile UINT32 RxDMABufAddr;
    volatile UINT32 RxDMANextPtr;
} RXDESC, *PRXDESC;


/* Transmit descriptor */

typedef struct
{
    volatile UINT32 TxOwnBit;
    volatile UINT32 TxFrameControl;
    volatile UINT32 TxDMABufAddr;
    volatile UINT32 TxDMANextPtr;
} TXDESC, *PTXDESC;


/* Descriptor definitions */

#define RXBD_SIZE           sizeof(RXDESC)
#define TXBD_SIZE           sizeof(TXDESC)

#define TXDESC_DEFAULT		32            /* default number of receive decriptors */
#define TXDESC_MAX			64            /* max receive descriptors */
#define TXDESC_MIN			8             /* min receive descriptors */

#define RXDESC_DEFAULT		32            /* default number of transmit decriptors */
#define RXDESC_MAX			64            /* max transmit descriptors */
#define RXDESC_MIN			8             /* min transmit descriptors */


/* Configuration items */

#define KS_BUFSIZ               (ETHERMTU + SIZEOF_ETHERHEADER + 6) /* Ethernet buffer size */
#define LN_SPEED                (10000000)
#define KS_DEV_NAME             "ksPci"      /* name of the device */
#define KS_DEV_NAME_LEN         (6)       /* length of the name string */
#define KS_LITTLE_ENDIAN        (0)       /* little endian PCI bus      */
#define KS_BIG_ENDIAN           (1)       /* big endian PCI bus         */
#define KS_ACTIVE_LOW           (0)       /* active-low interrupt line  */
#define KS_ACTIVE_HI            (1)       /* active-high interrupt line */


/* Configuration items */


/* mBlks, clBlks, and cluster buffer */
typedef struct ks8842_buffer 
{
    M_BLK_ID 	pMblk;
} KS_MBLK;


/* ks884x DMA control structure */

typedef struct _DMA_INFO 
{
	INT32	nBaseAddr;			    /* base address */
	UINT32  nResetCount;			/* DMA reset counter */

	/* interrupt related */
	UINT32  uIntMask;			    /* interrupt mask checked within ISR */

	/* mac */
    UINT8   stMacStation[MAC_ADDRESS_LENGTH];

	/* Tx related */
	UINT8	bTxStarted;			    /* Tx DMA started or stopped! */
	UINT8	bTxFlowCtrl;		    /* flow control for Tx DMA */
	UINT8	bTxOffLoad;			    /* enable/disable Task offload for Tx DMA */
	UINT8	byTxPBL;			    /* Tx PBL */
    UINT8   bTxChecksum;			/* checksum enable/disable indicator */
	UINT8	fTxNoResource;		    /* flag indicates out of Tx resource */

	/* Tx desc related */
	INT32	nTxDesc;			    /* number of Tx descriptors */
    INT32   nTxDescNextAvail;		/* next available Tx descriptor */
    INT32   nTxDescUsed;			/* used Tx descriptor */
    volatile INT32 nTransmitCount;	/* number of packets to transmit */
    INT32   nTxProcessedCount;	    /* number of packets to transmitted */
	INT32	nTxDescTotal;		    /* total number fo Tx descriptors */
    INT32   nTransmitCoalescing;	/* Tx packets coalescing count */

    TXDESC *pTxDescBaseAddr;        /* begin of Tx descriptor address */
    volatile INT32 nTxDescAvail;
    KS_MBLK *pTxMblk;               /* House keeping of Tx Mblk */

	/* Rx related */
	UINT8	bRxStarted;			    /* Rx DMA started or stopped! */
	UINT8	bRxFlowCtrl;		    /* flow control for Rx DMA */
	UINT8	byRxPBL;			    /* Rx PBL */
    UINT8   bRxChecksum;			/* checksum enable/disable indicator */
    UINT8   bRxPromiscuous;		    /* enable rx promiscuous mode */
    UINT8   bRxMulticast;		    /* enable rx all multicast frames */
    UINT32  uRxBufferLen;		    /* rx buffer length */

    INT32   nRxDesc;				/* number of Rx descriptors */
    INT32   nRxDescNextAvail;		/* next available Rx descriptor */
    INT32   nRxDescNextToFill;	    /* next Rx desc to fill new buffer to */
    RXDESC *pRxDescBaseAddr;        /* begin of Rx descriptor address */
	INT32	nRxDescTotal;		    /* total number fo Rx descriptors */
    KS_MBLK *pRxMblk;               /* House keeping of Rx Mblk */

	UINT8   bPowerDownReset;		/* perform powerdown reset instead of soft reset */

	UINT32  uDebugDumpPkt;	        /* flag to dump Tx/Rx packet for debugging */

} DMA_INFO, *PDMA_INFO;


/* k884x switch port control structure */

typedef struct _PORT_INFO 
{
	/* PHY related */
	UINT8	bAutoNegoInProgress;    /* if set, means that auto nego is in progress!!! */
	UINT8	bLinkActive;		    /* flag indicates whether link is active or not */
    UINT8   bLinkChanged;		    /* link changed indicator */
	UINT16  myLinkType;			    /* Link type - auto-negotiation or manually */
    UINT16  myLinkSpeed;		    /* link speed */
    UINT8   myLinkDuplex;		    /* link duplex (full/half) */

    UINT16  hisLinkSpeed;		    /* link speed based on partner's settings */
    UINT8   hisLinkDuplex;	        /* HD/FD detected based on partner's settings */

	INT32	nLinkChangeCount;		/* trace link change count */

	/* PORT related */
	UINT8	bPortsFlowCtrl;		    /* flow control */
    UINT16  usTag;				    /* tag value for the port (ingress) */
	UINT8   byStormProtection;	    /* broadcast storm protection */
	UINT8   bySTPenable;		    /* spanning tree enable\disable control */
	UINT8   bySTPstate;	            /* STP states (disable\block\listern\learn\forward) */
	UINT8   byIngressPriority;	    /* ingress priority */
	UINT8   byIngressPriorityTOS;	/* TOS based ingress priority */
	UINT8   byIngressPriority802_1P;/* 802.1p based ingress priority */
	UINT8   byEgressPriority;		/* egress priority */

} PORT_KS_INFO, *PPORT_KS_INFO;



/* The definition of the driver control structure */

typedef struct drv_ctrl
{
    END_OBJ           endObj;            /* The class we inherit from */
    int               unit;              /* unit number of the device */

    int               inum;              /* interrupt number */
    int               ilevel;            /* interrupt level */

    volatile char *   pShMem;            /* real ptr to shared memory */
    volatile char *   memBase;           /* PCnet memory pool base */
    volatile char *   memAdrs;           /* PCnet memory pool base */

    UINT32            devAdrs;           /* device structure address */
    UINT16            boardBusEndianMode;/* board bus endian mode board specific */
    int               memSize;           /* PCnet memory pool size */
    int               memWidth;          /* width of data port */
    int               offset;            /* offset of data in the buffer */
    volatile UINT32   flags;             /* Our local flags */
    UINT32            pciMemBase;        /* memory base as seen from PCI*/
    UINT8             enetAddr[6];       /* ethernet address */
    CACHE_FUNCS       cacheFuncs;        /* cache function pointers */

    volatile BOOL     txCleaning;        /* transmit descriptor cleaning */
    volatile BOOL     txBlocked;         /* transmit flow control */
    volatile BOOL	  rxHandling;		 /* receive frames handling */
    volatile BOOL     errorHandling;     /* task level error handling */
    END_ERR           lastError;         /* Last error passed to muxError */

    CL_POOL_ID        pClPoolId;         /* cluster pool Id */
    M_CL_CONFIG       mClCfg;            /* mBlk & cluster config structure */
    CL_DESC           clDesc;            /* cluster descriptor table */

	PDMA_INFO	      pDMAInfo;		     /* ks884x DMA information */
	PORT_KS_INFO	  port[TOTAL_PORT_NUM]; /* ks884x port related (switch port only) */
    PHARDWARE         pHW;               /* The pointer to the ks884x HW structure. */

	WDOG_ID	          watchdog;			 /* watch dog id */
	int	              nWatchdogDelay;	 /* watch dog delay period */

	UINT32            fPollSendFromCLI;	 /* flag to indicate call ks884xEndSend from CLI hwbuftx command */
} DRV_CTRL;

IMPORT DRV_CTRL *gpDrvCtrl[];
#define	DI		(pDrvCtrl->pDMAInfo)	 /* SWITCH DMA Inforamtion */


/* IMPORT functions */

IMPORT void    hwPhyReset ( PHARDWARE, ULONG );
IMPORT void    hwAutoNegoAdvertisement ( PHARDWARE, ULONG, ULONG, BOOLEAN );
IMPORT void    hwAutoNegoStart ( PHARDWARE, ULONG );
IMPORT BOOLEAN hwGetPhyLinkStatus ( PHARDWARE, ULONG  );
IMPORT BOOLEAN hwGetPhyAutoNegoStatus ( PHARDWARE, ULONG );

IMPORT void    hwMacStartRx ( PHARDWARE, BOOLEAN );
IMPORT void    hwMacStartTx ( PHARDWARE, BOOLEAN );
IMPORT void    hwGetMacAddress ( PHARDWARE, UCHAR * );
IMPORT void    hwSetMacAddress ( PHARDWARE, UCHAR * );
IMPORT void    hwAddrFilterClear ( PHARDWARE );
IMPORT int     hwAddrFilterAdd ( PHARDWARE, UCHAR * );
IMPORT int     hwAddrFilterDel ( PHARDWARE, UCHAR * );
IMPORT void    hwSetDescriptorBase ( PHARDWARE, ULONG, ULONG);
IMPORT void    hwMacDmaInit ( PHARDWARE, ULONG, BOOLEAN, BOOLEAN, ULONG, BOOLEAN, BOOLEAN );
IMPORT void    hwSetMacDmaPromiscuousMode ( PHARDWARE, BOOLEAN );
IMPORT void    hwSetMacDmaMulticastMode ( PHARDWARE, BOOLEAN );

IMPORT void    hwIntClear ( PHARDWARE, ULONG );
IMPORT ULONG   hwGetIntStatus ( PHARDWARE );
IMPORT void    hwSetInt ( PHARDWARE, ULONG, BOOLEAN );
IMPORT ULONG   hwGetInt ( PHARDWARE );

IMPORT void    hwSwitchInit ( PHARDWARE, BOOLEAN );
IMPORT void    hwSwitchEnable ( PHARDWARE, BOOLEAN );
IMPORT void    hwPortInit ( PHARDWARE, ULONG, ULONG, BOOLEAN, BOOLEAN );
IMPORT void    hwGetPortSpeedDuplex ( PHARDWARE, ULONG, ULONG *, ULONG * );
IMPORT void    hwGetPortPartnerSpeedDuplex ( PHARDWARE, ULONG, ULONG *, ULONG * );
IMPORT void    hwSetPortFlowControl ( PHARDWARE, ULONG, BOOLEAN );
IMPORT void    hwSetPortBackPressure ( PHARDWARE, ULONG, BOOLEAN );




#ifdef __cplusplus
}
#endif

#endif /* __INCKS8842END_H */

#endif /* #ifdef KS_PCI_BUS */
