/* ms7751se.h - Hitachi ULSI MS7751SE01 CPU board header */

/* Copyright 1999-2003 Wind River Systems, Inc. */
	
/*
modification history
--------------------
01j,16jun03,h_k  moved SYS_xxx_FREQ to config.h. (SPR #87000)
01i,27nov01,zl   componentized virtual PCI support.
01h,16nov01,zl   corrected comment for SYS_PCLK_FREQ.
01g,24oct01,hk   added virtual PCI memory and I/O support.
01f,11jun01,h_k  added definition for USB vendor ID and device ID.
01e,11jun01,h_k  moved SH7751 PCI specific definition to sh7751.h.
01d,22may01,zl   changed PCI_LAT_TIMER value.
01c,30mar01,zl   corrected PCI support, added M1543C support.
01b,09jan01,rsh  add PCI and END functionality
01a,15dec00,rsh  written (based on ms7750Ase01.h and hmse7751).
*/

#ifndef	INCms7751seh
#define	INCms7751seh

#ifdef __cplusplus
extern "C" {
#endif

/*
 * This file contains I/O addresses and related constants for the
 * Hitachi ULSI MS7xxx SolutionEngine 01
 */


#include "arch/sh/sh7750.h"
#include "arch/sh/sh7751.h"


#include "cacheLib.h"
#include "drv/mem/memDev.h"
#include "arch/sh/cacheShLib.h"

/* Enforce some INCLUDE's which are not optional to the user */
 
#ifdef INCLUDE_SH7751_VIRTUAL_PCI_IO
#  ifndef INCLUDE_SH7751_VIRTUAL_PCI_MEMIO
#    define INCLUDE_SH7751_VIRTUAL_PCI_MEMIO	/* need basic PCI extension */
#  endif /* INCLUDE_SH7751_VIRTUAL_PCI_MEMIO */
#endif /* INCLUDE_SH7751_VIRTUAL_PCI_IO */

#ifdef INCLUDE_SH7751_VIRTUAL_PCI_MEMIO		/* need MMU and PCI */
#  ifndef INCLUDE_MMU_FULL
#    ifndef INCLUDE_MMU_BASIC
#      define INCLUDE_MMU_BASIC
#    endif /* INCLUDE_MMU_BASIC */
#  endif /* INCLUDE_MMU_FULL */
#  ifndef INCLUDE_PCI
#    define INCLUDE_PCI
#  endif /* INCLUDE_PCI */
#endif /* INCLUDE_SH7751_VIRTUAL_PCI_MEMIO */
 
/* #ifdef INCLUDE_LN_97X_END	*/		/* 79C973 needs PCI */

 /* 79C973 and KS884x needs PCI */
#if defined  (INCLUDE_LN_97X_END) || defined (INCLUDE_KS884X_END)	
#  ifndef INCLUDE_PCI
#    define INCLUDE_PCI
#  endif
#endif /* INCLUDE_NETWORK */
 
#ifdef  INCLUDE_PCI				/* use PCI autoconfig */
#  ifndef INCLUDE_PCI_AUTOCONF
#    define INCLUDE_PCI_AUTOCONF
#  endif
#endif /* INCLUDE_PCI */
 
/* Local memory */

#define	MS7751SE_FLASH_SIZE	0x00400000	/* 4MB flash */
#define MS7751SE_SDRAM_SIZE	0x04000000	/* 64MB SDRAM */

/* system clock and aux clock definitions */

#ifndef SYS_CLK_RATE_MIN
#define SYS_CLK_RATE_MIN	3		/* minimum system clk rate    */
#define SYS_CLK_RATE_MAX	5000		/* maximum system clk rate    */
#endif

#ifndef AUX_CLK_RATE_MIN
#define AUX_CLK_RATE_MIN	3		/* minimum auxiliary clk rate */
#define AUX_CLK_RATE_MAX	5000		/* maximum auxiliary clk rate */
#endif

/* On-board LED register address */

#define LED_REG_ADRS_P0		0x1A000000	/* LED reg. address in P0     */
#define LED_REG_SIZE		0x1000		/* min. sz. in sysPhisMemDesc */
#define LED_REG_ADRS		(LED_REG_ADRS_P0 + SH_CACHE_THRU)

#define LED_PATTERN1		0xAA		/* pattern for sysHwInit()    */
#define LED_PATTERN2		0xCC		/* pattern for sysHwInit2()   */


/* Relocate exception vector table into P1 for CPUs that have MMU */

#undef  VEC_BASE_ADRS
#define VEC_BASE_ADRS ((char *)((LOCAL_MEM_LOCAL_ADRS & SH7751_PHYS_MASK) \
				| SH7751_P1_BASE))

/* M1543C SuperIO definitions */

#define SUPERIO_REG_INTERVAL	1

#define UART1_REG_BASE		(ISA_BASE_LOCAL + UART1_OFFSET)
#define UART2_REG_BASE		(ISA_BASE_LOCAL + UART2_OFFSET)
#define KBD_REG_BASE	 	(ISA_BASE_LOCAL + KBD_OFFSET)
#define MOUSE_REG_BASE	 	(ISA_BASE_LOCAL + MOUSE_OFFSET)


/* PCI Bus definitions */

#define PCIC_CAR		((int) PCIPAR)	/* PCIC PIO Address Register */
#define PCIC_CDR		((int) PCIPDR)	/* PCIC PIO Data Register */

#define PCI_MAX_BUS     	1
#define PCI_LAT_TIMER		0xff

/*
 * PCI address memory map. The PCI Bus addresses are arbitrary values 
 * defined by PCI_MSTR_IO_BUS, PCI_SLV_MEM_BUS, PCI_MSTR_MEMIO_BUS, and
 * PCI_MSTR_PCIC_BUS.
 *
 *                           PCI BUS      LOCAL MSTR   LOCAL SLAVE
 *
 *  +---------------------+  0x00000000   0xfe240000   N/A
 *  | PCI I/O Space (ISA) |
 *  +---------------------+  0x00010000   0xfe250000   N/A
 *  |    PCI I/O Space    |
 *  +---------------------+  0x00040000   0xfe280000   N/A
 *  |                     |
 *  :                     :
 *  |                     |
 *  +---------------------+  0x00100000   0xfe200000   N/A
 *  |  PCIC config space  |
 *  +---------------------+  0x00100400   0xfe200400   N/A
 *  |                     |
 *  :                     :
 *  |                     |
 *  +---------------------+  0x0c000000   N/A          0x0c000000
 *  |                     |
 *  |    Local Memory     |
 *  |                     |
 *  +---------------------+  0x10000000   0xfd000000   N/A
 *  |                     |
 *  |  PCI Memory space   |
 *  |                     |
 *  +---------------------+  0x11000000   0xfe000000   N/A
 *  |                     |
 *  |                     |
 *  :                     :
 *  |                     |
 *  |                     |
 *  +---------------------+
 *
 * If you enable both of the virtual PCI space extension options by defining
 * INCLUDE_SH7751_VIRTUAL_PCI_MEMIO and INCLUDE_SH7751_VIRTUAL_PCI_IO in
 * config.h, the PCI Memory and I/O spaces are extended as follows:
 *
 *                           PCI BUS      LOCAL MSTR   LOCAL SLAVE
 *
 *  +---------------------+  0x00000000   0x28000000   N/A
 *  | PCI I/O Space (ISA) |
 *  +---------------------+  0x00010000   0x28010000   N/A
 *  |    PCI I/O Space    |
 *  +---------------------+  0x00400000   0x28400000   N/A
 *  |                     |
 *  :                     :
 *  |                     |
 *  +---------------------+  0x00100000   0xfe200000   N/A
 *  |  PCIC config space  |
 *  +---------------------+  0x00100400   0xfe200400   N/A
 *  |                     |
 *  :                     :
 *  |                     |
 *  +---------------------+  0x0c000000   N/A          0x0c000000
 *  |                     |
 *  |    Local Memory     |
 *  |                     |
 *  +---------------------+  0x10000000   0x20000000   N/A
 *  |                     |
 *  |  PCI Memory space   |
 *  |                     |
 *  +---------------------+  0x18000000   0x28000000   N/A
 *  |                     |
 *  |                     |
 *  :                     :
 *  |                     |
 *  |                     |
 *  +---------------------+
 */ 

#ifndef INCLUDE_PCI_PARAMS	/* Make sure we don't override prjParams.h */

/* PCI I/O master window */

#ifdef INCLUDE_SH7751_VIRTUAL_PCI_IO
#  define PCI_MSTR_IO_LOCAL	0x28000000		/* virtual PCI I/O */
#  define PCI_MSTR_IO_SIZE	0x00400000		/* 4M (extended) */
#else
#  define PCI_MSTR_IO_LOCAL	0xfe240000
#  define PCI_MSTR_IO_SIZE	0x00040000		/* 256K */
#endif
#define PCI_MSTR_IO_BUS         0x00000000

/* Slave window that makes local memory visible to PCI devices. */

#define PCI_SLV_MEM_LOCAL	0x0c000000
#define PCI_SLV_MEM_BUS		0x0c000000
#define PCI_SLV_MEM_SIZE	0x04000000		/* 64M */

/* PCI MEMIO master (non-prefetch memory) window */

#ifdef INCLUDE_SH7751_VIRTUAL_PCI_MEMIO
#  define PCI_MSTR_MEMIO_LOCAL  0x20000000		/* virtual PCI memory */
#  define PCI_MSTR_MEMIO_SIZE   0x08000000		/* 128M (extended) */
#else
#  define PCI_MSTR_MEMIO_LOCAL  0xfd000000
#  define PCI_MSTR_MEMIO_SIZE   0x01000000		/* 16M */
#endif
#define PCI_MSTR_MEMIO_BUS      0x10000000

#endif /* INCLUDE_PCI_PARAMS */

/* PCIC configuration register area */

#define PCI_MSTR_PCIC_LOCAL	0xfe200000
#define PCI_MSTR_PCIC_BUS	0x00100000
#define PCI_MSTR_PCIC_SIZE	0x100

/* PCIC local register area */

#define PCI_MSTR_PCICL_LOCAL	0xfe200100
#define PCI_MSTR_PCICL_SIZE	0x128

/* PCI system parameters */

#define PCI_MEM_ADRS		0			/* Prefetchable mem. */
#define PCI_MEM_SIZE		0
#define PCI_MEMIO_ADRS		PCI_MSTR_MEMIO_BUS	/* non-prefetchable */
#define PCI_MEMIO_SIZE		PCI_MSTR_MEMIO_SIZE
#define ISA_BASE_LOCAL		PCI_MSTR_IO_LOCAL	/* 64k ISA IO space */
#define PCI_ISA_IO_ADRS		PCI_MSTR_IO_BUS
#define PCI_ISA_IO_SIZE		0x10000
#define PCI_IO_ADRS		(PCI_MSTR_IO_BUS + PCI_ISA_IO_SIZE)
#define PCI_IO_SIZE		(PCI_MSTR_IO_SIZE - PCI_ISA_IO_SIZE)


/* PCI Access macro's */
 
/* PCI (non-prefetchable) memory adrs to CPU adrs */
 
#define PCI_MEMIO2LOCAL(x) \
    ((void *)((UINT)(x) - PCI_MSTR_MEMIO_BUS + PCI_MSTR_MEMIO_LOCAL))
 
/* PCI IO memory adrs to CPU adrs */
 
#define PCI_IO2LOCAL(x) \
    ((void *)((UINT)(x) - PCI_MSTR_IO_BUS + PCI_MSTR_IO_LOCAL))

/* Translate local memory address to PCI address (slave window) */
 
#define LOCAL2PCI_MEM(x) \
    ((void *)((UINT)(x) - PCI_SLV_MEM_LOCAL + PCI_SLV_MEM_BUS))

/* PCI IDs excluded from autoconfig */

#define PIBC_VENDOR_ID		0x10b9			/* PIBC ven. (M1543C) */
#define PIBC_DEVICE_ID		0x1533			/* PIBC dev. (M1543C) */
#define PMU_VENDOR_ID		0x10b9			/* PMU ven. (M1543C) */
#define PMU_DEVICE_ID		0x7101			/* PMU dev. (M1543C) */
#define USB_VENDOR_ID		0x10b9			/* USB ven. (M1543C) */
#define USB_DEVICE_ID		0x5237			/* USB dev. (M1543C) */

/* PCIC bus control register values */

#define PCIC_BCR1		0x00000008
#define PCIC_BCR2		0x2ffc
#define PCIC_WCR1		0x02770771
#define PCIC_WCR2		0x7ffe4fe7
#define PCIC_WCR3		0x01777771
#define PCIC_MCR		0x100901b4


/* KS884x Generic Bus Interface on Expansion Slot (CN 1) memory base address */

#define KS_MBD_ADDR_PHY		AREA4	                           /* KS884x Physical address in Area 4 (0x10000000) */
#define KS_REG_SIZE		    0x1000		                       /* min. sz. in sysPhisMemDesc */
#define KS_MBD_ADDR 		(KS_MBD_ADDR_PHY + SH7751_P2_BASE) /* KS884x Logical  address in Area 4 (0xB0000000) */


#ifdef __cplusplus
}
#endif

#endif	/* INCms7751seh */
