/* sysKS884xEnd.c - system configuration module for ks884xEnd */
 
/* Copyright 1984 - 2001 Wind River Systems, Inc. */
 
/*
modification history
--------------------
22July04,pcd written.
*/


/*
DESCRIPTION
This is the WRS-supplied configuration module for the VxWorks 
ks884xEnd (ks) END driver.  It performs the dynamic parameterization 
of the ks884xEnd driver.  This technique of 'just-in-time' 
parameterization allows driver parameter values to be declared 
as any other defined constants rather than as static strings. 
*/
 
/* includes */

#include "vxWorks.h"
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "end.h"
#include "config.h"

#if (defined (INCLUDE_KS884X_END) && defined (INCLUDE_NETWORK) \
     && defined (INCLUDE_END))

#include "ks884xEnd.h" 
#include "hardware.h" 
 
/* defines */
 

/* PCI vendor/device Ids and rev mask */

#define KS884X_MAX_UNITS         1			/* Max ks884X devices per system */
#define	NUM_PCI_DEV_IDS          2			/* number of support PCI device ID on the system */

 
#define KS_PCI_VENDOR_ID         0x16C6		/* Micrel                  */
#define KS8841_PCI_DEVICE_ID     0x8841		/* PCI KS8841 device ID    */
#define KS8842_PCI_DEVICE_ID     0x8842		/* PCI KS8842 device ID    */

#define KS884X_ENDIAN_BIG        1          /* board bus endian mode to access PCI CSR reg, 1:big-endian, 0:little-endian */
#define KS884X_OFFSET		     2	        /* For proper IP header alignment */
#define KS884X_USR_FLAGS		 0          /* furure use, set to zero        */
 
#if _BYTE_ORDER == _BIG_ENDIAN
/* This is used in the PCI_SWAP definition */

LOCAL int ks884xBusEndianess = KS_LITTLE_ENDIAN;
#endif

/***** DEBUG MACROS *****/

#define KS_PCI_DEBUG

#ifdef  KS_PCI_DEBUG
#define KS_PCI_DEBUG_OFF			0x0000
#define KS_PCI_DEBUG_ERROR			0x0001
#define KS_PCI_DEBUG_SHOW 			0x0002
#define KS_PCI_DEBUG_ALL            0xffff

/* Turn it off initially. */
LOCAL int nsDebug = (KS_PCI_DEBUG_ERROR | KS_PCI_DEBUG_SHOW );   

#define DRV_PRINT(FLG,X) \
    if (nsDebug & FLG) printf X;

#else /* KS_PCI_DEBUG */

#define DRV_PRINT(FLG,X)

#endif /* KS_PCI_DEBUG */


/* externs */

IMPORT END_OBJ* ks884xEndLoad (char *);

/* typedefs */
 

typedef struct pciStatusCount
{
    /* KS884X PCI Configuration Space - Status register Count */

   UINT32	    dperr;			    /* Detected Parity Error */
   UINT32		syserr;		        /* Signal System Error */
   UINT32		rvmab;              /* Received Master Abort */
   UINT32		rvtab;			    /* Received Target Abort */
   UINT32		dst;			    /* Device Select Timing */
   UINT32		dpr;			    /* Data Parity Error */
 
} PCISTATUSCOUNT, *PPCISTATUSCOUNT ;

PCISTATUSCOUNT	ghwPciStatusCount[KS884X_MAX_UNITS];

typedef struct vendDev
{
    int vendorId;
    int deviceId;
} VEND_DEV;

/* locals */

LOCAL  int	ks884xUnits = 0;

 
LOCAL VEND_DEV vendDevs[NUM_PCI_DEV_IDS] =
{
    {KS_PCI_VENDOR_ID, KS8841_PCI_DEVICE_ID},
    {KS_PCI_VENDOR_ID, KS8842_PCI_DEVICE_ID}
};

/*
 * This array stores the board-specific PCI resources, the base address
 * register configuration mode. It's indexed using the device number returned
 * from pciFindDevice().
 */
 
#define KS884X_IOBASE      0x00020000
#define KS884X_MEMBASE     0x10000800  

PCI_RESOURCES ks884xPciRsrcs [KS884X_MAX_UNITS] =
{
    {KS884X_IOBASE, KS884X_MEMBASE, 0, 0, PCI_CFG_AUTO , 0, 0, 0, 0, 0, 0, NULL}
};


/* global */

/*
 * ks884x_max_if is 1 if it is KS8841, 2 if it is KS8842.
 */

int   ks884x_max_if = KS884X_MAX_IF;  
BOOL  ks884x_found = FALSE;


/* export functions */

LOCAL void ks884xPciSetup ( PCI_RESOURCES * );

void  ks884xPciStatusPolling ( UINT32 );
void  ks884xPciStatusCountShow ( UINT32 );
void  ks884xPciShow ( UINT32 );


/******************************************************************************
*
* sysKs884xEndLoad - load and create load string, for a ks884x (ks) device.
*
* This routine loads the ks device with initial parameters. 
*
* RETURNS: pointer to END object or NULL.
*
* SEE ALSO: ks884xEndLoad()
*/
 
END_OBJ * sysKs884xEndLoad
(
    char * 		pParamStr,   /* ptr to initialization parameter str. */
    void * 		unused       /* unused optional argument */
)
{
    PCI_RESOURCES * pRsrc;
    int    		    unit;
    char          * cp; 			
    char 		    paramStr [END_INIT_STR_MAX];   /* from end.h */
    END_OBJ * 		pEnd;
    static char 	KS884xParamTemplate[] = 
				      "0x%x:0x%x:0x%x:%d:%d:-1:-1:-1:%d:%d:0x%x";




    if (ks884x_found != TRUE) 
        printf ("Error: ks884x device not found !\n");

    /*
     * The ks884xEnd driver END_LOAD_STRING should be:
     * <unit>:<devMemAddr>:<devIoAddr>:<pciMemBase:<vecNum>:<intLvl>:<memAdrs>:
     * <memSize>:<memWidth>:<endianMode>:<offset>:<flags>
     *
     * Note that ks884xEnd unit number is prepended in muxDevLoad, so we 
     * don't put it here!
     */

    if (strlen (pParamStr) == 0)
    {
        /* 
         * muxDevLoad() calls us twice.  If the string is
         * zero length, then this is the first time through
         * this routine, so we just return.
         */

        pEnd = ks884xEndLoad (pParamStr);
    }
    else
	{
        /*
         * On the second pass though here, we actually create 
         * the initialization parameter string on the fly.   
         * Note that we will be handed our unit number on the 
         * second pass through and we need to preserve that information.
         * So we use the unit number handed from the input string.
         */

	    /* Get the unit number from the paramStr */

	    unit = atoi (pParamStr);
	

	    if ((unit < 0) || (unit >= ks884x_max_if))
	        return ((END_OBJ *) NULL);

	    /* pointer to resources */

	    pRsrc = ks884xPciRsrcs;

        cp = strcpy (paramStr, pParamStr); /* cp points to paramStr */


        /* Now, we advance cp, by finding the end the string */

        cp += strlen (paramStr);
        
        /* finish off the initialization parameter string */

	    sprintf (cp, KS884xParamTemplate,
                 (UINT) PCI_MEMIO2LOCAL (pRsrc->membaseCsr),  /* memIo base */
                 (UINT) PCI_IO2LOCAL (pRsrc->iobaseCsr),      /* Io base */
                 (UINT) LOCAL2PCI_MEM (0),	                  /* pciMemBase */
                 pRsrc->vecNum,			                      /* interrupt IRQ vector */
                 pRsrc->intLevel,		                      /* interrupt level */
                 KS884X_ENDIAN_BIG,	                          /* board bus endian mode to access PCI CSR register */
                 KS884X_OFFSET,			                      /* offset */
                 KS884X_USR_FLAGS		                      /* flags */
                );


        if ( (nsDebug & KS_PCI_DEBUG_SHOW) == KS_PCI_DEBUG_SHOW );   
        {
             #if (0)
             printf("sysKs884xEndLoad(): pciBus=%d, pciDevice=%d, pciFunc=%d\n", 
                     pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc);
             printf("sysKs884xEndLoad(): membaseCsr = 0x%08X \n", (int)pRsrc->membaseCsr );
             printf("sysKs884xEndLoad(): iobaseCsr = 0x%08X \n", (int)pRsrc->iobaseCsr );
             printf("KS884X CSR base address = 0x%X \n", (int)pRsrc->membaseCsr );
             printf("sysKs884xEndLoad(): paramStr:%s\n",paramStr);
             #endif

             ks884xPciShow ( unit );
        }

        if ((pEnd = ks884xEndLoad (paramStr)) == (END_OBJ *)NULL)
	    {
            printf ("Error: device failed ks884xEndLoad routine.\n");
	    }

	    pRsrc->endIf = pEnd;

    } /* if (strlen (pParamStr) == 0) */

    return (pEnd);
}

 
/*******************************************************************************
*
* sysKs884xPciInit - prepare LAN adapter for KS884x initialization
*
* This routine finds out the PCI device, maps its memory and IO address.
* It must be done prior to initializing the KS884x.  Also
* must be done prior to MMU initialization, usrMmuInit(), and muxLoad.
*
* RETURNS: OK/ERROR
*/
 
STATUS sysKs884xPciInit (void)
{
    UINT32              membaseCsr;
    UINT32              iobaseCsr;
    char                irq;
    UINT8               chip_rev = 0; 
    int                 pciBus;
    int                 pciDevice;
    int                 pciFunc;
    int                 unit;
    int                 devIndex;     
    PCI_RESOURCES * 	pRsrc;


    /* for all the support lan devices find if some of them exist */
 
    for (devIndex = 0; devIndex < NUM_PCI_DEV_IDS; devIndex++ )
    {
        for (unit = 0; unit < KS884X_MAX_UNITS; unit++)
        {
            if (pciFindDevice (vendDevs[devIndex].vendorId,
                               vendDevs[devIndex].deviceId,
                               unit, &pciBus, &pciDevice, &pciFunc) == OK)
            {
                 /* board detected */
                 /* for now, we're ignoring the possibility of duplicates */

                 ks884x_found = TRUE;
 
                 /* load up the PCI device table */
 
                 pRsrc = ks884xPciRsrcs + ks884xUnits; /* get the pci entry */
 
                /* 
                 * Record basic PCI info 
                 */

                 pRsrc->pciBus    = pciBus;
                 pRsrc->pciDevice = pciDevice;
                 pRsrc->pciFunc   = pciFunc;
 
                 pRsrc->pciVendorID = vendDevs[devIndex].vendorId;
                 pRsrc->pciDevID    = vendDevs[devIndex].deviceId;

                 ks884xUnits++;      /* number of units found */
            }
        }
    }
 
    if ((ks884x_found != TRUE) || (pciDevice > PCI_MAX_DEV))
    {
        DRV_PRINT (KS_PCI_DEBUG_ERROR, 
                   ("sysKS8842PciInit(): ks884xUnits=%d. WARNING: No KS884X Switch unit found\n", ks884xUnits));
        return (ERROR);
    }
 
    /* Now initialize all the units we found */
 
    for (unit = 0; unit < ks884xUnits; unit++)
    {

        /* Fill in the resource entry */
 
        pRsrc = ks884xPciRsrcs + unit;

        /* 
         * if PCI_CFG_FORCE is specified then set configuration parameters (memory base)
         */ 

        if ( pRsrc->configType == PCI_CFG_FORCE )
        {
            pciConfigOutLong (pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
                              PCI_CFG_BASE_ADDRESS_0, pRsrc->membaseCsr );
            pciConfigOutLong (pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
                              PCI_CFG_BASE_ADDRESS_1, (pRsrc->iobaseCsr | PCI_BASE_IO));
        }
 
        /*
         * get memory base address and IO base address
         */
 
        pciConfigInLong (pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
                         PCI_CFG_BASE_ADDRESS_0, &membaseCsr );
        pciConfigInLong (pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
                         PCI_CFG_BASE_ADDRESS_1, &iobaseCsr);
        pciConfigInByte (pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
                         PCI_CFG_DEV_INT_LINE, (UINT8*) &irq);
        pciConfigInByte (pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
                         PCI_CFG_REVISION, (UINT8*) &chip_rev);

        /* over write the resource table with values read */
 
        pRsrc->membaseCsr   = membaseCsr & PCI_MEMBASE_MASK;
        pRsrc->iobaseCsr    = iobaseCsr & PCI_IOBASE_MASK;
        pRsrc->vecNum       = IVEC_TO_INUM (PIC_IRQ_TO_IVEC(irq));
        pRsrc->intLevel     = irq;
        pRsrc->pciRevID     = (UINT32)chip_rev;
 
        /* 
         * Set up ks884X PCI Configuratin Space
         */

        ks884xPciSetup(pRsrc);

    }
 
    return (OK);
}


/*******************************************************************************
*
* sysKs884xIntEnable - enable KS884x interrupts
*
* This routine enables KS884x interrupts.  This may involve operations on
* interrupt control hardware.
*
* RETURNS: OK or ERROR for invalid arguments.
*/
 
STATUS sysKs884xIntEnable
(
    int level           /* level number */
)
{
    /* level here means the PIC IRQ number */

    return (sysPicIntEnable(level));
}
 
/*******************************************************************************
*
* sysKs884xIntDisable - disable KS884x interrupts
*
* This routine disables KS884x interrupts.  This may involve operations on
* interrupt control hardware.
*
* RETURNS: OK or ERROR for invalid arguments.
*/
 
STATUS sysKs884xIntDisable
(
    int level           /* level number */
)
{
    /* level here means the PIC IRQ number */

    return (sysPicIntDisable(level));
}


/*******************************************************************************
 * ks884xPciSetup -
 *
 * DESCRIPTION
 *
 *	Setup PCI Configuration ( Space ) Registers through SH7751R PCIC registers by . 
 *  calling pciConfigOutLong to write one longword to the PCI configuration space.
 *
 *  So we do not have to do any software byte swap.
 *
 * PARAMETERS:
 *
 *      PHWPCI	pRsrc
 *		   pointer to KS884X PCI device struct contains pci bus\device\function number.
 *
 */

LOCAL void ks884xPciSetup 
( 
    PCI_RESOURCES * 	pRsrc
)
{
    UINT32  dwRegDataWrite, dwRegDataRead;


    /* 
     * Command and Status Configuration Register. 
     *  enable System error detect, Parity error detect, MWI, Master operation, Memory space access
     */

    dwRegDataWrite = ( COMM_SETTING & CFCS_COMM );
#if (0)
    dwRegDataWrite &= ~( CFCS_COMM_PERRSP );
#endif
    pciConfigOutLong(pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
				     CFCS,
				     dwRegDataWrite
                     );

    pciConfigInLong(pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
				     CFCS,
				     &dwRegDataRead
                     );

    if ( (dwRegDataRead & CFCS_COMM) != dwRegDataWrite )
    {
        DRV_PRINT (KS_PCI_DEBUG_ERROR, 
                  ("ks884xPciSetup(): ERROR: can't set Command and Status Configuration Register.\n"));
    }


    /* 
     * Configuration Latency Timer Register 
     *  Latency time is 0x80, Cache line size is 8 dwords
     */

    dwRegDataWrite = ( (LATENCY_TIMER << 8) & CFLT_LATENCY_TIMER ) | ( CACHE_LINESZ & CFLT_CACHE_LINESZ );
    pciConfigOutLong(pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
				     CFLT,
				     dwRegDataWrite
                     );

    pciConfigInLong(pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
				     CFLT,
				     &dwRegDataRead
                     );

    if ( (dwRegDataRead & (CFLT_LATENCY_TIMER | CFLT_CACHE_LINESZ)) != dwRegDataWrite )
    {
        DRV_PRINT (KS_PCI_DEBUG_ERROR, 
                  ("ks884xPciSetup(): ERROR: can't set Configuration Latency Timer Register.\n"));
    }


    /* 
     * Configuration Interrupt Register 
     *  Max latency is 10us (40 x 0.25us), Min grant is 5us (20 x 0.25us) 
     */

    dwRegDataWrite = ( (MAX_LATENCY << 24) & CFIT_MAX_L ) | ( (MIN_GRANT << 16) & CFIT_MIN_G );
    pciConfigOutLong(pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
				     CFIT,
				     dwRegDataWrite
                     );

    pciConfigInLong(pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
				     CFIT,
				     &dwRegDataRead
                     );

    if ( (dwRegDataRead & (CFIT_MAX_L | CFIT_MIN_G)) != dwRegDataWrite )
    {
        DRV_PRINT (KS_PCI_DEBUG_ERROR, 
                  ("ks884xPciSetup(): ERROR: can't set Configuration Interrupt Register.\n"));
    }


    /* Clear CFCS Status bits */
    pciConfigOutWord(pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
				     PCI_CFG_STATUS,
				     (UINT16)(CFCS_STAT >> 16)
                     );

}



/*******************************************************************************
 * ks884xPciStatusPolling -
 *
 * DESCRIPTION
 *  Polling KS884X PCI Status Register periodically (1 second) by  
 *  calling pciConfigInWord to read one word from the PCI configuration space.
 *
 *  So we do not have to do any software byte swap.
 *
 * PARAMETERS:
 *
 */

void ks884xPciStatusPolling
( 
    UINT32           devIndex
)
{

    PPCISTATUSCOUNT  pPciStatusCount;
    PCI_RESOURCES  * pRsrc;
    UINT16           pciStatus;    /* PCI status */


    if ((devIndex < 0) || (devIndex >= KS884X_MAX_UNITS))
        return;

    /* Get address with this PCI device */
    pRsrc = &ks884xPciRsrcs [devIndex];

    pPciStatusCount = &ghwPciStatusCount[devIndex];

    /* 
     * Get KS884X PCI Status Register. 
     */

    pciConfigInWord( pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
				     PCI_CFG_STATUS,
				     &pciStatus
                   );

    /*
     * Clear KS884X PCI Status.
     */

    pciConfigOutWord( pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
				      PCI_CFG_STATUS,
				      (UINT16)(CFCS_STAT >> 16)
                    );

    /*
     * Check KS884X PCI Status, and record them.
     */

    /* Detected Parity Error */
    if (pciStatus & CFCS_STAT_DPERR)
          pPciStatusCount->dperr++;

    /* Signal System Error  */
    if (pciStatus & CFCS_STAT_SYSERR)
          pPciStatusCount->syserr++;

    /* Received Master Abort */
    if (pciStatus & CFCS_STAT_RVMAB)
          pPciStatusCount->rvmab++;

    /* Received Target Abort */
    if (pciStatus & CFCS_STAT_RVTAB)
          pPciStatusCount->rvtab++;

    /* Device Select Timing */
    pPciStatusCount->dst = ( (pciStatus & CFCS_STAT_DST) >> 9 );

    /* Data Parity Error */
    if (pciStatus & CFCS_STAT_DPR)
          pPciStatusCount->dpr++;

}

/*******************************************************************************
 * ks884xPciStatusCountShow - 
 *
 * DESCRIPTION
 *  Display KS884X PCI Status Count struct. 
 *
 * PARAMETERS:
 */
void ks884xPciStatusCountShow
(
    UINT32           devIndex
)
{
   PPCISTATUSCOUNT  pStaC1;


   pStaC1 = &ghwPciStatusCount[devIndex];

   printf (" KS884X PCI Status  \n" );
   printf ("\n");

   printf (" Detected Parity Error                 :   %10d  \n", (int)pStaC1->dperr );
   printf (" Data Parity Error                     :   %10d  \n", (int)pStaC1->dpr );
   printf (" Signal System Error                   :   %10d  \n", (int)pStaC1->syserr );
   printf (" Received Master Abort                 :   %10d  \n", (int)pStaC1->rvmab );
   printf (" Received Target Abort                 :   %10d  \n", (int)pStaC1->rvtab );
   printf (" Device Select Timing                  :   %10d  \n", (int)pStaC1->dst );
   printf ("\n");
   
}


/*******************************************************************************
 * ks884xPciShow -
 *
 * DESCRIPTION
 *
 *	Show PCI Configuration ( Space ) Registers information by . 
 *  calling pciConfigInLong to read one longword from the PCI configuration space.
 *
 *  So we do not have to do any software byte swap.
 *
 *
 */

void ks884xPciShow 
(
    UINT32     devIndex     /* Device index for KS884X */
)
{

    PCI_RESOURCES * pRsrc;
    UINT32	        data;


    if ((devIndex < 0) || (devIndex >= KS884X_MAX_UNITS))
        return;


    /* Get address with this PCI device */
    pRsrc = &ks884xPciRsrcs [devIndex];

    printf("KS884X PCI Configuration Space info at  PCI bus no.=%d, device no.=%d, function no.=%d:\n",
            pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc);
 
    /* Configuration ID Register */
    pciConfigInLong( pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc, 
                     CFID, &data );
    printf("Device ID        :   0x%x\n", ((data & CFID_DEVID) >> 16));
    printf("Vendor ID        :   0x%x\n", (data & CFID_VENID));

    /* Command and Status Configuration Register */
    pciConfigInLong( pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc, 
                     CFCS, &data );
    printf("Status           :   0x%x\n",  ((data & CFCS_STAT) >> 16));
    printf("Command          :   0x%x\n",  (data & CFCS_COMM));

    /* Configuration Revision Register */
    pciConfigInLong( pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc, 
                     CFRV, &data );
    printf("Basci Class Code :   0x%x\n",  ((data & CFRV_BASCLASS) >> 24));
    printf("Sub-class Code   :   0x%x\n",  ((data & CFRV_SUBCLASS) >> 16));
    printf("Revision ID      :   0x%x\n",  ((data & CFRV_REVID)));

    /* Configuration Latency Timer Register */
    pciConfigInLong( pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc, 
                     CFLT, &data );
    printf("Latency Timer    :   0x%x\n", ((data & CFLT_LATENCY_TIMER) >> 8));
    printf("Cache Line Size  :   0x%x\n", data & CFLT_CACHE_LINESZ);

    /* Configuration Memory Base Address Register */
    pciConfigInLong( pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc, 
                     CMBA, &data );
    printf("Base Memory Addr :   0x%x\n", data);

    /* Subsystem ID Register */
    pciConfigInLong( pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc, 
                     CSID, &data );
    printf("Subsys ID        :   0x%x\n", (data & CSID_SUBSYSID) >> 16);
    printf("Subsys Vendor ID :   0x%x\n", data & CSID_SUBVENID);

    /* Configuration Interrupt Register */
    pciConfigInLong( pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc, 
                     CFLT, &data );
    printf("Max Latency      :   0x%x\n", (data & CFIT_MAX_L) >> 24);
    printf("Min Grant        :   0x%x\n", (data & CFIT_MIN_G) >> 16);
    printf("Interrupt Pin    :   0x%x\n", (data & CFIT_IPIN) >>  8);
    printf("Interrupt Line   :   0x%x\n", (data & CFIT_ILINE));

}


#endif /* defined (INCLUDE_KS884X_END) && defined (INCLUDE_NETWORK) */
