/* ---------------------------------------------------------------------------
          Copyright (c) 2004-2007 Micrel, Inc.  All rights reserved.
   ---------------------------------------------------------------------------

    interrupt.c - Linux network device driver interrupt processing.

    Author      Date        Description
    THa         01/05/04    Created file.
    THa         06/27/05    Updated for version 0.1.5.
    THa         06/29/05    Updated for Linux 2.6.
    THa         10/18/05    Updated for Renesas 7751R board.
    THa         01/13/06    Transfer dword-aligned data for performance.
    THa         04/03/06    Use TWO_NETWORK_INTERFACE instead of TWO_PORTS.
    THa         06/29/06    Change statistics reporting to provide meaningful
                            data.
    THa         07/26/06    Add preliminary DMA support.
    THa         10/16/06    Add ST7100 DIM_0_x_SG FDMA support.
    THa         12/05/06    Version 1.0.8 adds revision A6 turbo mode support.
    THa         05/11/07    Update for 2.6.19 and newer kernels.
   ---------------------------------------------------------------------------
*/


#include "target.h"
#include "hardware.h"
#include "device.h"
#include <linux/etherdevice.h>

#ifdef SH_BUS_DMA
#include <linux/delay.h>

#ifdef CONFIG_STM_DMA
#include <asm/dma.h>

extern struct dma_channel* stm_dma[];
extern int stm_dma_chan[];
#endif

#ifdef CONFIG_SH_7751_SOLUTION_ENGINE
BOOLEAN Platform_DmaStartXfer ( PHARDWARE, UCHAR, ULONG, ULONG, ULONG, BOOLEAN );
BOOLEAN Platform_IsDmaComplete ( PHARDWARE, UCHAR );
#endif

extern unsigned char* dma_src;
extern unsigned char* dma_dst;

void enable_dma ( PHARDWARE, int );
#endif

#ifndef SH_BUS_DMA
#if 1
#define RX_TASK
#endif
#endif
#if 1
#define RX_TASK_MAX  2
#endif

#ifdef DEBUG_COUNTER
int rx_cnt1 = 0;
int rx_cnt2 = 0;
int rx_cnt3 = 0;
#endif

#ifdef CHECK_OVERRUN
static ULONGLONG last_pause_count = 0;
#endif

/* Enable to check interrupt loop. */
#if 0
#define CHECK_LOOP
#endif

#ifdef DBG
extern int ks_rx_overrun;
#endif

/* -------------------------------------------------------------------------- */

#ifdef SH_BUS_DMA
static UINT interrupt_blocked = 0;
#endif

#ifdef SH_BUS_DMA
struct sk_buff* dma_rx_skb = NULL;

#ifndef SH_BUS_DMA_USE_2ND_CHAN
static int dma_rx_len = 0;
static int skb_rx_len = 0;
#endif

void dma_rcv_packet_done (
    struct net_device* dev );

int dma_rcv_packet (
    struct net_device* dev )
{
    int              packet_len;
    struct dev_priv* priv = ( struct dev_priv* ) dev->priv;
    struct dev_info* hw_priv = priv->pDevInfo;
    PHARDWARE        pHardware = &hw_priv->hw;
    struct sk_buff*  skb;

#ifdef CONFIG_STM_DMA
#ifdef SH_BUS_DMA_USE_SG
    int                 sg_count;
    int                 new_sg_len;
    struct scatterlist  old_sg;
    struct scatterlist* new_sg_ptr;

#ifdef SH_BUS_DMA_USE_2ND_CHAN
    int                 old_sg_len;
    struct scatterlist  new_sg[ 2 ];
    struct scatterlist* old_sg_ptr;
#endif
#endif
#endif
    BYTE*            data;
    USHORT           wData;

#if defined( TWO_NETWORK_INTERFACE )  ||  !defined( NO_STATS )
    int              port = MAIN_PORT;
#endif

    packet_len = HardwareReceiveLength( pHardware );
    while ( !packet_len ) {

next_packet:
        HardwareAcknowledgeReceive( pHardware );
        HardwareReadRegWord( pHardware, REG_RX_STATUS_BANK,
            REG_RX_STATUS_OFFSET, &wData );
        if ( !wData  ) {
            return 0;
        }
        packet_len = HardwareReceiveLength( pHardware );
    }
    if ( packet_len > MAX_BUF_SIZE ) {

free_next_packet:
        HardwareReleaseReceive( pHardware );
        goto next_packet;
    }

#ifdef TWO_NETWORK_INTERFACE
    /* Need to use the other net device if coming from the second port. */
    if ( PORT_2 == pHardware->m_bPortRX ) {
        port = OTHER_PORT;
        dev = hw_priv->pDev[ OTHER_PORT ];

        /* The second net device is not created. */
        if ( !dev ) {
            goto free_next_packet;
        }

        priv = ( struct dev_priv* ) dev->priv;

        /* This port device is not opened for use. */
        if ( !priv->opened ) {
            goto free_next_packet;
        }
    }
#endif

    /* Allocate extra bytes for 32-bit transfer. */
    skb = dev_alloc_skb( packet_len + 6 );
    if ( !skb ) {
        priv->stats.rx_dropped++;
        goto free_next_packet;
    }

    /* Align socket buffer in 4-byte boundary for better performance. */
    skb_reserve( skb, 2 );
    data = skb_put( skb, packet_len );
    skb->dev = dev;

    /* Update receive statistics. */
#ifndef NO_STATS
    pHardware->m_cnCounter[ port ][ OID_COUNTER_DIRECTED_FRAMES_RCV ]++;
    pHardware->m_cnCounter[ port ][ OID_COUNTER_DIRECTED_BYTES_RCV ] +=
        packet_len;
#endif
    priv->stats.rx_packets++;
    priv->stats.rx_bytes += packet_len;

#ifdef SH_32BIT_ALIGNED
    data = pHardware->m_bLookahead;
#endif

    packet_len = ( packet_len + 3 ) & ~3;

#ifdef RCV_TURBO_MODE
    packet_len -= 4;
#endif

#ifndef SH_BUS_DMA_USE_2ND_CHAN
    /* Too small for DMA transfer. */
    if ( packet_len < FDMA_MIN_SIZE ) {
        HW_READ_BUFFER( pHardware, REG_DATA_OFFSET, data, packet_len );
        HardwareReceiveBufferLast( pHardware, data, packet_len );

#ifdef SH_32BIT_ALIGNED
        memcpy( skb->data, pHardware->m_bLookahead, skb->len );
#endif

        skb->protocol = eth_type_trans( skb, dev );
#if 0
        skb->ip_summed = CHECKSUM_NONE;
#endif

        /* Notify upper layer for received packet. */
        dev->last_rx = jiffies;
        netif_rx( skb );

        goto next_packet;
    }
#endif

    /* Save socket buffer in DMA transfer. */
    dma_rx_skb = skb;

    pHardware->m_bAcquire++;

#if 1
    if ( !( priv->dev_state &
            ( DEV_STATE_STOP_TX_BY_RX | DEV_STATE_STOP_TX_BY_TX )) ) {
        netif_stop_queue( dev );
    }
    priv->dev_state |= DEV_STATE_STOP_TX_BY_RX;
#endif

    dma_cache_inv( data, packet_len );

#ifdef CONFIG_STM_DMA
    stm_dma[ 0 ]->sar = virt_to_bus(( void* )(
        pHardware->m_ulVIoAddr + REG_DATA_OFFSET ));
    dma_rx_len = packet_len - ( packet_len % FDMA_MIN_SIZE );
    skb_rx_len = packet_len;

#ifdef SH_BUS_DMA_USE_0
    stm_dma[ 0 ]->flags = NODE_DONE_ISR | DIM_0_x_1 | LIST_TYPE_UNLINKED;
    stm_dma[ 0 ]->dar = virt_to_bus( data );
    stm_dma[ 0 ]->count = dma_rx_len;
#endif

#ifdef SH_BUS_DMA_USE_SG
    old_sg.page = virt_to_page( data );
    old_sg.offset = ( long ) data & ~PAGE_MASK;

    new_sg_len = 1;

#ifndef SH_BUS_DMA_USE_2ND_CHAN
    old_sg.length = dma_rx_len;
    new_sg_ptr = &old_sg;

#else
    old_sg.length = packet_len;

    new_sg_ptr = new_sg;
    old_sg_ptr = &old_sg;
    old_sg_len = new_sg_len;
    while ( old_sg_len-- ) {
        *new_sg_ptr = *old_sg_ptr;
        if ( new_sg_ptr->length < FDMA_MIN_SIZE ) {
        }
        else if ( new_sg_ptr->length % FDMA_MIN_SIZE ) {
            int biglen = new_sg_ptr->length -
                ( new_sg_ptr->length % FDMA_MIN_SIZE );

            new_sg_ptr->length = biglen;
            new_sg_ptr++;
            *new_sg_ptr = *old_sg_ptr;
            new_sg_ptr->offset += biglen;
            new_sg_ptr->length -= biglen;
            new_sg_len++;
        }
        new_sg_ptr++;
        old_sg_ptr++;
    }
    new_sg_ptr = new_sg;
#endif
    sg_count = dma_map_sg( NULL, new_sg_ptr, new_sg_len, DMA_FROM_DEVICE );

    stm_dma[ 0 ]->list_len = sg_count;
    stm_dma[ 0 ]->dst_sg = new_sg_ptr;
    stm_dma[ 0 ]->dst_sg_len = sg_count;
    stm_dma[ 0 ]->src_sz = sg_count;
    stm_dma[ 0 ]->flags = CALLBACK_CONTEXT_INTERRUPT | DIM_0_x_SG |
        LIST_TYPE_UNLINKED | DMA_SETUP_NOBLOCK;
#endif

#ifdef SH_BUS_DMA_PACED
    stm_dma[ 0 ]->flags |= 12;

#ifdef SH_BUS_DMA_USE_2ND_CHAN
    stm_dma[ 0 ]->flags |= HORRIBLE_SMSC911X_HACK;
    stm_dma[ 0 ]->smsc911x_short_chan = 13;
#endif
#endif
    dma_configure_channel( stm_dma[ 0 ]);
    dma_xfer( stm_dma_chan[ 0 ]);
#endif

#ifdef CONFIG_SH_7751_SOLUTION_ENGINE
    Platform_DmaStartXfer( pHardware, 0,
        pHardware->m_ulVIoAddr + REG_DATA_OFFSET, 
        virt_to_bus( data ),
        packet_len, FALSE );
#endif
    return 1;
}  /* dma_rcv_packet */


void dma_rcv_packet_done (
    struct net_device* dev )
{
    struct dev_priv* priv = ( struct dev_priv* ) dev->priv;
    struct dev_info* hw_priv = priv->pDevInfo;
    PHARDWARE        pHardware = &hw_priv->hw;
    struct sk_buff*  skb = dma_rx_skb;
    int              rc = 0;
    BYTE*            data;
    USHORT           wData;

    /* Release received packet. */
#ifdef SH_32BIT_ALIGNED
    data = pHardware->m_bLookahead;

#else
    data = skb->data;
#endif

#ifndef SH_BUS_DMA_USE_2ND_CHAN
    if ( skb_rx_len > dma_rx_len ) {
        HW_READ_BUFFER( pHardware, REG_DATA_OFFSET,
            &data[ dma_rx_len ], skb_rx_len - dma_rx_len );
    }
#endif

    HardwareReceiveBufferLast( pHardware, data, skb_rx_len );

#ifdef SH_32BIT_ALIGNED
    memcpy( skb->data, pHardware->m_bLookahead, skb->len );
#endif

    skb->protocol = eth_type_trans( skb, skb->dev );
#if 0
    skb->ip_summed = CHECKSUM_NONE;
#endif

    /* Notify upper layer for received packet. */
    skb->dev->last_rx = jiffies;
    netif_rx( skb );

#if 1
    dma_rx_skb = NULL;
#endif

    pHardware->m_bAcquire--;

#if 1
    if ( !( priv->dev_state & DEV_STATE_STOP_TX_BY_TX ) ) {

        /* Reset transmit timeout timer. */
        dev->trans_start = jiffies;
        netif_wake_queue( dev );
    }
    priv->dev_state &= ~DEV_STATE_STOP_TX_BY_RX;
#endif

    HardwareAcknowledgeReceive( pHardware );

    /* Check receive buffer memory. */
    HardwareReadRegWord( pHardware, REG_RX_STATUS_BANK,
        REG_RX_STATUS_OFFSET, &wData );
    if ( wData )
        rc = dma_rcv_packet( dev );
    if ( !rc ) {
        if ( interrupt_blocked ) {
            HardwareEnableInterrupt( pHardware );
        }
    }
}  /* dma_rcv_packet_done */
#endif

/* -------------------------------------------------------------------------- */

/*
    dev_rcv

    Description:
        This routine is called to receive a packet.

    Parameters:
        struct net_device* dev
            Pointer to network device.

    Return (None):
*/

static inline int dev_rcv (
    struct net_device* dev )
{
    int              packet_len;
    struct dev_priv* priv = ( struct dev_priv* ) dev->priv;
    struct dev_info* hw_priv = priv->pDevInfo;
    PHARDWARE        pHardware = &hw_priv->hw;
    struct sk_buff*  skb;
    int              receive = 0;
    BYTE*            data;
    USHORT           wData;

#if defined( TWO_NETWORK_INTERFACE )  ||  !defined( NO_STATS )
    int              port = MAIN_PORT;
#endif

    HardwareReadRegWord( pHardware, REG_RX_STATUS_BANK,
        REG_RX_STATUS_OFFSET, &wData );
    while ( wData ) {

#ifdef DEBUG_COUNTER
        pHardware->m_nGood[ COUNT_GOOD_NEXT_PACKET ]++;
#endif
        packet_len = HardwareReceiveLength( pHardware );
        if ( !packet_len ) {
            goto next_packet;
        }
        else if ( packet_len > MAX_BUF_SIZE ) {

free_next_packet:
            HardwareReleaseReceive( pHardware );
            goto next_packet;
        }

#ifdef TWO_NETWORK_INTERFACE
        /* Need to use the other net device if coming from the second port. */
        if ( PORT_2 == pHardware->m_bPortRX ) {
            port = OTHER_PORT;
            dev = hw_priv->pDev[ OTHER_PORT ];

            /* The second net device is not created. */
            if ( !dev ) {
                goto free_next_packet;
            }

            priv = ( struct dev_priv* ) dev->priv;

            /* This port device is not opened for use. */
            if ( !priv->opened ) {
                goto free_next_packet;
            }
        }
#endif

        /* Allocate extra bytes for 32-bit transfer. */
        skb = dev_alloc_skb( packet_len + 6 );
        if ( !skb ) {
            priv->stats.rx_dropped++;
            goto free_next_packet;
        }

        /* Align socket buffer in 4-byte boundary for better performance. */
        skb_reserve( skb, 2 );
        data = skb_put( skb, packet_len );

        HardwareReceiveBuffer( pHardware, data, packet_len );

        skb->dev = dev;
        skb->protocol = eth_type_trans( skb, dev );
#if 0
        skb->ip_summed = CHECKSUM_NONE;
#endif

        /* Update receive statistics. */
#ifndef NO_STATS
        pHardware->m_cnCounter[ port ][ OID_COUNTER_DIRECTED_FRAMES_RCV ]++;
        pHardware->m_cnCounter[ port ][ OID_COUNTER_DIRECTED_BYTES_RCV ] +=
            packet_len;
#endif
        priv->stats.rx_packets++;
        priv->stats.rx_bytes += packet_len;

        /* Notify upper layer for received packet. */
        dev->last_rx = jiffies;
        netif_rx( skb );
        ++receive;

#ifdef DEBUG_COUNTER
        pHardware->m_nGood[ COUNT_GOOD_RCV_COMPLETE ]++;
#endif

next_packet:

        /* Acknowledge the interrupt. */
        HardwareAcknowledgeReceive( pHardware );

        HardwareReadRegWord( pHardware, REG_RX_STATUS_BANK,
            REG_RX_STATUS_OFFSET, &wData );
    }

#ifdef DEBUG_COUNTER
    if ( !receive )
        pHardware->m_nGood[ COUNT_GOOD_NO_NEXT_PACKET ]++;
    else if ( 1 == receive )
        rx_cnt1++;
    else if ( 2 == receive )
        rx_cnt2++;
    else
        rx_cnt3++;
#endif
    return( receive );
}  /* dev_rcv */

/* -------------------------------------------------------------------------- */

static int rx_task_no_rcv = 0;

void RxProcessTask (
    unsigned long data )
{
    struct net_device* dev = ( struct net_device* ) data;
    struct dev_priv*   priv = ( struct dev_priv* ) dev->priv;
    struct dev_info*   hw_priv = priv->pDevInfo;
    PHARDWARE          pHardware = &hw_priv->hw;
    UINT               Interrupt;

    if ( !pHardware->m_bEnabled )
        return;
    Interrupt = HardwareBlockInterrupt( pHardware );
    if ( dev_rcv( dev ) < 1 ) {

#ifdef RX_TASK_MAX
        if ( ++rx_task_no_rcv < RX_TASK_MAX ) {
            goto rx_schedule;
        }
#endif

        /* tasklets are interruptible. */
        spin_lock_irq( &priv->lock );
        HardwareTurnOnInterrupt( pHardware, INT_RX, NULL );
        spin_unlock_irq( &priv->lock );
    }
    else {

#ifdef DEBUG_COUNTER
        if ( rx_task_no_rcv )
            pHardware->m_nGood[ COUNT_GOOD_RCV_NOT_DISCARD ]++;
#endif
        rx_task_no_rcv = 0;

#ifdef RX_TASK_MAX
rx_schedule:
#endif
        tasklet_schedule( &hw_priv->rx_tasklet );
    }
    HardwareRestoreInterrupt( pHardware, Interrupt );
}  /* RxProcessTask */

/* -------------------------------------------------------------------------- */

#ifdef CHECK_LOOP
static int int_count = 0;
static unsigned int int_jiffies = 0;
#endif

#ifdef CHECK_OVERRUN
void inline dev_reset (
    struct net_device* dev )
{
    struct dev_priv* priv = ( struct dev_priv* ) dev->priv;
    struct dev_info* hw_priv = priv->pDevInfo;
    PHARDWARE        pHardware = &hw_priv->hw;

#ifdef TWO_NETWORK_INTERFACE
    int              port;
#endif

    HardwareReset( pHardware );
    HardwareSetup( pHardware );
            
#ifdef TWO_NETWORK_INTERFACE
    PortSet_STP_State( pHardware, MAIN_PORT, STP_STATE_BLOCKED );
    PortSet_STP_State( pHardware, OTHER_PORT, STP_STATE_BLOCKED );
    for ( port = MAIN_PORT; port <= OTHER_PORT; port++ ) {
        struct net_device* net_if = priv->pDevInfo->pDev[ port ];

        /* The second net device is not created. */
        if ( !net_if )
            continue;

        priv = ( struct dev_priv* ) net_if->priv;

        /* This port device is opened for use. */
        if ( priv->opened ) {
            PortSet_STP_State( pHardware, port, STP_STATE_FORWARDING );
        }
    }
#endif

    if ( pHardware->m_bMulticastListSize )
        HardwareSetGroupAddress( pHardware );

    HardwareEnable( pHardware );
    if ( !pHardware->m_InterruptBlocked ) {
        HardwareEnableInterrupt( pHardware );
    }
}  /* dev_reset */
#endif


/*
    dev_interrupt

    Description:
        This routine is called by upper network layer to signal interrupt.

    Parameters:
        int irq
            Interrupt number.

        void* dev_id
            Pointer to network device.

        struct pt_regs regs
            Pointer to registers.

    Return (None):
*/

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
void

#else
irqreturn_t
#endif
ks8842i_\
dev_interrupt (
    int             irq,
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19))
    void*           dev_id,
    struct pt_regs* regs )
#else
    void*           dev_id )
#endif
{
    UINT               IntEnable;
    struct net_device* dev = ( struct net_device* ) dev_id;
    struct dev_priv*   priv = ( struct dev_priv* ) dev->priv;
    struct dev_info*   hw_priv = priv->pDevInfo;
    PHARDWARE          pHardware = &hw_priv->hw;

#ifdef RX_TASK
    int                receive;
#endif
    int                port = MAIN_PORT;

#ifdef CHECK_LOOP
    int                int_loop = 0;
#endif

#if defined( TEST_BLOCK_INTERRUPT ) || defined( CHECK_LOOP )
    USHORT             wInterrupt;
#endif

#ifdef SH_BUS_DMA
    if ( pHardware->m_bAcquire ) {
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0))
        return IRQ_HANDLED;
#else
        return;
#endif
    }
#endif

    HardwareSaveBank( pHardware );

#if defined( TEST_BLOCK_INTERRUPT ) || defined( CHECK_LOOP )
    HardwareReadRegWord( pHardware, REG_INT_MASK_BANK,
        REG_INT_MASK_OFFSET, &wInterrupt );
#endif

#ifdef TEST_BLOCK_INTERRUPT
    if ( pHardware->m_bBlocked ) {
        printk( "B! %d; %x; %x, %x\n", pHardware->m_bBlocked,
            wInterrupt, pHardware->m_InterruptBlocked,
            pHardware->m_wInterruptMask );
    }
#endif

#ifdef SH_BUS_DMA
    interrupt_blocked = 0;
    if ( !pHardware->m_InterruptBlocked ) {
        HardwareDisableInterrupt( pHardware );
        interrupt_blocked = pHardware->m_InterruptBlocked;
    }
#endif

#ifdef DEBUG_COUNTER
    pHardware->m_nGood[ COUNT_GOOD_INT ]++;
#endif
    HardwareReadInterrupt( pHardware, &IntEnable );

#ifdef BLOCK_STATUS
    IntEnable |= pHardware->m_InterruptStatus;
    pHardware->m_InterruptStatus = 0;
#endif

#ifdef RX_TASK
#ifdef RX_TASK_MAX
    rx_task_no_rcv = RX_TASK_MAX;
#endif
#endif

#ifdef CHECK_LOOP
    if ( int_jiffies == jiffies ) {
        if ( ++int_count == 100 ) {
            printk( "L:%04x %04x %04x\n", IntEnable,
                pHardware->m_wInterruptMask, wInterrupt );
        }
    }
    else {
        int_count = 0;
        int_jiffies = jiffies;
    }
#endif
    while ( IntEnable ) {

#ifdef CHECK_LOOP
        if ( ++int_loop >= 20 ) {
            printk( "!L!\n" );
            break;
        }
#endif

#ifdef DEBUG_COUNTER
        pHardware->m_nGood[ COUNT_GOOD_INT_LOOP ]++;
#endif
        HardwareAcknowledgeInterrupt( pHardware, IntEnable );

        if ( ( IntEnable & INT_TX ) ) {
            ks8842i_transmit_done( dev );
        }

        if ( ( IntEnable & INT_TX_UNDERRUN ) ) {
            ks8842i_transmit_done( dev );

            /* Wait for the transmit timeout to reset the hardware. */
            netif_stop_queue( dev );
        }

#if defined( DBG ) || defined( CHECK_OVERRUN )
        if ( ( IntEnable & INT_RX_OVERRUN ) ) {

#ifdef CHECK_OVERRUN
            PPORT_CONFIG pPort;
            UINT         blocked;
            UCHAR        bPort;
            UCHAR        bCounter;
#endif

#ifdef DBG
            ks_rx_overrun++;
            if ( ks_rx_overrun < 5 )
            DBG_PRINT( "!R" );
#endif

#ifdef CHECK_OVERRUN
#ifdef DEF_KS8842
            bPort = HOST_PORT;
            bCounter = MIB_COUNTER_RX_PAUSE;
#else
            bPort = MAIN_PORT;
            bCounter = MIB_COUNTER_TX_PAUSE;
#endif
            pPort = &pHardware->m_Port[ bPort ];
            blocked = pHardware->m_InterruptBlocked;
            pHardware->m_InterruptBlocked = 1;
            PortReadMIBCounter( pHardware, bPort, bCounter,
                &pPort->cnCounter[ bCounter ]);
            pHardware->m_InterruptBlocked = blocked;
            if ( last_pause_count == pPort->cnCounter[ bCounter ] ) {

#ifdef DBG
                ks_rx_overrun = 0;
                DBG_PRINT( "-R-"NEWLINE );
#endif
                priv->stats.tx_fifo_errors++;
                dev_reset( dev );
                break;
            }
            last_pause_count = pPort->cnCounter[ bCounter ];
#endif

#ifdef TWO_NETWORK_INTERFACE
            /* See which port is being used. */
            port = pHardware->m_bPort;
#endif
            pHardware->m_cnCounter[ port ][ OID_COUNTER_RCV_NO_BUFFER ]++;
            priv->stats.rx_fifo_errors++;
        }
#endif

        if ( ( IntEnable & INT_RX ) ) {

#ifdef DEBUG_COUNTER
            pHardware->m_nGood[ COUNT_GOOD_INT_RX ]++;
#endif

            /* Receive the packet. */
#ifndef SH_BUS_DMA
#ifdef RX_TASK
            receive =
#endif
            dev_rcv( dev );
#ifdef RX_TASK
            if ( receive > 0 ) {
                HardwareTurnOffInterrupt( pHardware, INT_RX );
                rx_task_no_rcv = 0;
                tasklet_schedule( &hw_priv->rx_tasklet );
                break;
            }
#endif

#else
            if ( dma_rcv_packet( dev ) )

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0))
                return IRQ_HANDLED;
#else
                return;
#endif
#endif
        }

        if ( ( IntEnable & INT_PHY ) ) {

#if SET_DEFAULT_LED
            UCHAR bData;

#ifdef KS_ISA_BUS
            HardwareSelectBank( pHardware, REG_SWITCH_CTRL_BANK );
#endif
            HW_READ_BYTE( pHardware, REG_SWITCH_CTRL_5_HI_OFFSET, &bData );
            if ( ( bData & SET_DEFAULT_LED ) != SET_DEFAULT_LED ) {
                DBG_PRINT( "link is not good: %x!"NEWLINE, bData );
            }
#endif
            pHardware->m_bLinkIntWorking = TRUE;
            SwitchGetLinkStatus( pHardware );
        }

#ifdef BLOCK_STATUS
        if ( ( IntEnable & 0x80000000 ) ) {
            IntEnable &= ~INT_TX_STOPPED;
                pHardware->m_InterruptCheck &= ~INT_TX_STOPPED;
        }
#endif

#ifdef DBG
        if ( ( IntEnable & INT_RX_ERROR ) ) {
            DBG_PRINT( "!E"NEWLINE );
        }
        if ( ( IntEnable & INT_RX_STOPPED ) ) {
            WORD wData;

            DBG_PRINT( "Rx stopped\n" );
            HardwareReadRegWord( pHardware, REG_RX_CTRL_BANK,
                REG_RX_CTRL_OFFSET, &wData );
            if ( ( wData & RX_CTRL_ENABLE ) ) {
                DBG_PRINT( "Rx disabled\n" );
            }
            if ( ( pHardware->m_wReceiveConfig & RX_CTRL_ENABLE ) ) {
                HW_WRITE_WORD( pHardware, REG_RX_CTRL_OFFSET, 0 );
                HW_WRITE_WORD( pHardware, REG_RX_CTRL_OFFSET,
                    pHardware->m_wReceiveConfig );
            }
            else {
                HardwareTurnOffInterrupt( pHardware, INT_RX_STOPPED );
                pHardware->m_InterruptCheck &= ~INT_RX_STOPPED;
            }
            HardwareAcknowledgeInterrupt( pHardware, INT_RX_STOPPED );
            break;
        }

        if ( ( IntEnable & INT_TX_STOPPED ) ) {
            WORD wData;

            DBG_PRINT( "Tx stopped\n" );
            HardwareReadRegWord( pHardware, REG_RX_CTRL_BANK,
                REG_TX_CTRL_OFFSET, &wData );
            if ( ( wData & TX_CTRL_ENABLE ) ) {
                DBG_PRINT( "Tx disabled\n" );
            }
            if ( ( pHardware->m_wTransmitConfig & TX_CTRL_ENABLE ) ) {
                HW_WRITE_WORD( pHardware, REG_TX_CTRL_OFFSET, 0 );
                HW_WRITE_WORD( pHardware, REG_TX_CTRL_OFFSET,
                    pHardware->m_wTransmitConfig );
            }
            else {
                HardwareTurnOffInterrupt( pHardware, INT_TX_STOPPED );
                pHardware->m_InterruptCheck &= ~INT_TX_STOPPED;
            }
            HardwareAcknowledgeInterrupt( pHardware, INT_TX_STOPPED );
            break;
        }
#endif
        HardwareReadInterrupt( pHardware, &IntEnable );
    }

#ifdef SH_BUS_DMA
    if ( interrupt_blocked ) {
        HardwareEnableInterrupt( pHardware );
    }
#endif
    HardwareRestoreBank( pHardware );

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0))
    return IRQ_HANDLED;
#endif
}  /* dev_interrupt */
